/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;

import net.sourceforge.atunes.gui.dockableView.CloseListener;
import net.sourceforge.atunes.gui.dockableView.DockableDialog;
import net.sourceforge.atunes.gui.dockableView.DockableFrame;
import net.sourceforge.atunes.gui.dockableView.DockableFrameController;
import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.views.controls.playList.PlayListTable;
import net.sourceforge.atunes.gui.views.menu.ApplicationMenuBar;
import net.sourceforge.atunes.gui.views.panels.ContextInformationPanel;
import net.sourceforge.atunes.gui.views.panels.FilePropertiesPanel;
import net.sourceforge.atunes.gui.views.panels.NavigationPanel;
import net.sourceforge.atunes.gui.views.panels.PlayListPanel;
import net.sourceforge.atunes.gui.views.panels.PlayerControlsPanel;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.handlers.VisualHandler;
import net.sourceforge.atunes.utils.LanguageTool;

/**
 * The Class MultipleFrame.
 */
public class MultipleFrame implements Frame {

	/** The Constant frameDimension. */
	private static final Dimension frameDimension = new Dimension(500, 400);

	/** The Constant navigatorDimension. */
	private static final Dimension navigatorDimension = new Dimension(300, 689);

	/** The Constant filePropertiesDimension. */
	private static final Dimension filePropertiesDimension = new Dimension(500, 130);

	/** The Constant audioScrobblerDimension. */
	private static final Dimension audioScrobblerDimension = new Dimension(300, 689);

	/** The dockable controller. */
	private DockableFrameController dockableController;

	/** The frame. */
	private DockableFrame frame;

	/** The navigator dialog. */
	private DockableDialog navigatorDialog;

	/** The file properties dialog. */
	private DockableDialog filePropertiesDialog;

	/** The audio scrobbler dialog. */
	private DockableDialog contextInformationDialog;

	/** The menu bar. */
	private ApplicationMenuBar menuBar; // NOT USED

	/** The tool bar. */
	private ToolBar toolBar; // NOT USED

	/** The navigation panel. */
	private NavigationPanel navigationPanel;

	/** The play list panel. */
	private PlayListPanel playListPanel;

	/** The player controls panel. */
	private PlayerControlsPanel playerControlsPanel;

	/** The file properties panel. */
	private FilePropertiesPanel filePropertiesPanel;

	/** The audio scrobbler panel. */
	private ContextInformationPanel contextInformationPanel;

	/**
	 * Instantiates a new multiple frame.
	 */
	public MultipleFrame() {
		dockableController = new DockableFrameController(new CloseListener() {
			@Override
			public void close() {
				VisualHandler.getInstance().finish();
			}

			@Override
			public void minimize() {
				// Nothing to do
			}
		});
	}

	/**
	 * Adds the content to file properties.
	 */
	private void addContentToFileProperties() {
		filePropertiesPanel = new FilePropertiesPanel();
		filePropertiesDialog.addContent(filePropertiesPanel);
		GuiUtils.applyComponentOrientation(filePropertiesPanel);
	}

	/**
	 * Adds the content to frame.
	 */
	private void addContentToFrame() {
		frame.setIcon(ImageLoader.APP_ICON.getImage());
		playerControlsPanel = new PlayerControlsPanel();

		JPanel auxPanel = new JPanel(new BorderLayout());
		auxPanel.add(getPlayListPanel(), BorderLayout.CENTER);
		auxPanel.add(playerControlsPanel, BorderLayout.SOUTH);

		GuiUtils.applyComponentOrientation(auxPanel);

		frame.addContent(auxPanel);
		JPopupMenu menu = menuBar.getMenuAsPopupMenu();
		GuiUtils.applyComponentOrientation(menu);
		frame.setMenu(menu);
	}

	/**
	 * Adds the content to navigator.
	 */
	private void addContentToNavigator() {
		navigationPanel = new NavigationPanel();
		navigatorDialog.addContent(navigationPanel);
		GuiUtils.applyComponentOrientation(navigationPanel);
	}

	/**
	 * Adds the content to open strands.
	 */
	private void addContentToOpenStrands() {
		contextInformationPanel = new ContextInformationPanel();
		contextInformationDialog.addContent(contextInformationPanel);
		GuiUtils.applyComponentOrientation(contextInformationPanel);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#create()
	 */
	@Override
	public void create() {
		// Created but not used
		menuBar = new ApplicationMenuBar();
		toolBar = new ToolBar();

		frame = dockableController.getNewFrame("", frameDimension.width, frameDimension.height, null, DockableFrameController.NONE, frameDimension);

		Point p = null;
		if (Kernel.getInstance().state.getMultipleViewXPosition() > 0 && Kernel.getInstance().state.getMultipleViewYPosition() > 0) {
			p = new Point(Kernel.getInstance().state.getMultipleViewXPosition(), Kernel.getInstance().state.getMultipleViewYPosition());
		}
		if (p == null) {
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			frame.setLocation((screenSize.width - frameDimension.width) / 2, 100);
		} else {
			frame.setLocation(p.x, p.y);
		}

		Dimension d = null;
		if (Kernel.getInstance().state.getMultipleViewWidth() != 0 && Kernel.getInstance().state.getMultipleViewHeight() != 0) {
			d = new Dimension(Kernel.getInstance().state.getMultipleViewWidth(), Kernel.getInstance().state.getMultipleViewHeight());
		}
		if (d != null) {
			frame.setSize(d);
		}

		addContentToFrame();

		JDialog.setDefaultLookAndFeelDecorated(false);
		navigatorDialog = dockableController.getNewDialog(frame, LanguageTool.getString("NAVIGATOR"), navigatorDimension.width, navigatorDimension.height, frame,
				DockableFrameController.WEST, navigatorDimension);
		addContentToNavigator();

		filePropertiesDialog = dockableController.getNewDialog(frame, LanguageTool.getString("PROPERTIES"), filePropertiesDimension.width, filePropertiesDimension.height, frame,
				DockableFrameController.SOUTH, filePropertiesDimension);
		addContentToFileProperties();

		contextInformationDialog = dockableController.getNewDialog(frame, LanguageTool.getString("AUDIO_SCROBBLER"), audioScrobblerDimension.width, audioScrobblerDimension.height,
				frame, DockableFrameController.EAST, audioScrobblerDimension);
		addContentToOpenStrands();
		JDialog.setDefaultLookAndFeelDecorated(true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#getAppMenuBar()
	 */
	@Override
	public ApplicationMenuBar getAppMenuBar() {
		return menuBar;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#getAudioScrobblerPanel()
	 */
	@Override
	public ContextInformationPanel getContextInformationPanel() {
		return contextInformationPanel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#getExtendedState()
	 */
	@Override
	public int getExtendedState() {
		return frame.getExtendedState();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#getFrame()
	 */
	@Override
	public JFrame getFrame() {
		return frame.getFrame();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#getLocation()
	 */
	@Override
	public Point getLocation() {
		return frame.getLocation();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#getNavigationPanel()
	 */
	@Override
	public NavigationPanel getNavigationPanel() {
		return navigationPanel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#getPlayerControls()
	 */
	@Override
	public PlayerControlsPanel getPlayerControls() {
		return playerControlsPanel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#getPlayListPanel()
	 */
	@Override
	public PlayListPanel getPlayListPanel() {
		if (playListPanel == null) {
			playListPanel = new PlayListPanel();
		}
		return playListPanel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#getPlayListTable()
	 */
	@Override
	public PlayListTable getPlayListTable() {
		return getPlayListPanel().getPlayListTable();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#getPropertiesPanel()
	 */
	@Override
	public FilePropertiesPanel getPropertiesPanel() {
		return filePropertiesPanel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#getSize()
	 */
	@Override
	public Dimension getSize() {
		return frame.getFrame().getSize();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#getToolBar()
	 */
	@Override
	public ToolBar getToolBar() {
		return toolBar;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#isVisible()
	 */
	@Override
	public boolean isVisible() {
		return frame.isVisible();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.gui.Frame#setCenterStatusBar(java.lang.String,
	 * java.lang.String)
	 */
	@Override
	public void setCenterStatusBar(String text, String toolTip) {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#setDefaultCloseOperation(int)
	 */
	@Override
	public void setDefaultCloseOperation(int op) {
		frame.setDefaultCloseOperation(op);
		navigatorDialog.setDefaultCloseOperation(op);
		filePropertiesDialog.setDefaultCloseOperation(op);
		contextInformationDialog.setDefaultCloseOperation(op);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#setExtendedState(int)
	 */
	@Override
	public void setExtendedState(int state) {
		//frame.setExtendedState(state);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.gui.Frame#setLeftStatusBarText(java.lang.String,
	 * java.lang.String)
	 */
	@Override
	public void setLeftStatusBarText(String text, String toolTip) {
		//	Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#setLocation(java.awt.Point)
	 */
	@Override
	public void setLocation(Point location) {
		//frame.setLocation(location.x, location.y);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.gui.Frame#setLocationRelativeTo(java.awt.Component
	 * )
	 */
	@Override
	public void setLocationRelativeTo(Component c) {
		frame.setLocationRelativeTo(c);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#setRightStatusBar(java.lang.String,
	 * java.lang.String)
	 */
	@Override
	public void setRightStatusBar(String text, String toolTip) {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.gui.Frame#setStatusBarImageLabelIcon(javax.swing
	 * .Icon, java.lang.String)
	 */
	@Override
	public void setStatusBarDeviceLabelText(String text) {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#setTitle(java.lang.String)
	 */
	@Override
	public void setTitle(String title) {
		frame.setTitle(title);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#setVisible(boolean)
	 */
	@Override
	public void setVisible(boolean visible) {
		frame.setVisible(visible);
		if (!visible || Kernel.getInstance().state.isShowNavigationPanel()) {
			navigatorDialog.setVisible(visible);
		}
		if (!visible || Kernel.getInstance().state.isShowSongProperties()) {
			filePropertiesDialog.setVisible(visible);
		}
		if (!visible || Kernel.getInstance().state.isUseContextInformation()) {
			contextInformationDialog.setVisible(visible);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#showAudioScrobblerPanel(boolean,
	 * boolean)
	 */
	@Override
	public void showContextInformationPanel(boolean show, boolean changeSize) {
		contextInformationDialog.setVisible(show);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#showNavigationPanel(boolean,
	 * boolean)
	 */
	@Override
	public void showNavigationPanel(boolean show, boolean changeSize) {
		navigatorDialog.setVisible(show);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#showNavigationTable(boolean)
	 */
	@Override
	public void showNavigationTable(boolean show) {
		navigationPanel.getNavigationTableContainer().setVisible(show);
		if (show) {
			navigationPanel.getSplitPane().setDividerLocation(Kernel.getInstance().state.getLeftHorizontalSplitPaneDividerLocation());
		} else {
			// Save location
			Kernel.getInstance().state.setLeftHorizontalSplitPaneDividerLocation(navigationPanel.getSplitPane().getDividerLocation());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#showProgressBar(boolean,
	 * java.lang.String)
	 */
	@Override
	public void showProgressBar(boolean visible, String tooltip) {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#showSongProperties(boolean)
	 */
	@Override
	public void showSongProperties(boolean show) {
		filePropertiesDialog.setVisible(show);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#showStatusBar(boolean)
	 */
	@Override
	public void showStatusBar(boolean show) {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#showToolBar(boolean)
	 */
	@Override
	public void showToolBar(boolean show) {
		getToolBar().setVisible(show);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.gui.Frame#showStatusBarImageLabel(boolean)
	 */
	@Override
	public void showStatusBarDeviceLabel(boolean visible) {
		// Nothing to do
	}

	@Override
	public void showStatusBarNewPodcastFeedEntriesLabel(boolean visible) {
		// Nothing to do
	}

}
