/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.dockableView;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JPanel;

/**
 * The Class MouseResizeListener.
 * 
 * @author fleax
 */
public class MouseResizeListener implements MouseListener, MouseMotionListener {

	/** The Constant NORTH_RESIZE. */
	public static final int NORTH_RESIZE = 0;

	/** The Constant EAST_RESIZE. */
	public static final int EAST_RESIZE = 1;

	/** The Constant SOUTH_RESIZE. */
	public static final int SOUTH_RESIZE = 2;

	/** The Constant WEST_RESIZE. */
	public static final int WEST_RESIZE = 3;

	/** The Constant NORTH_WEST_RESIZE. */
	public static final int NORTH_WEST_RESIZE = 4;

	/** The Constant NORTH_EAST_RESIZE. */
	public static final int NORTH_EAST_RESIZE = 5;

	/** The Constant SOUTH_WEST_RESIZE. */
	public static final int SOUTH_WEST_RESIZE = 6;

	/** The Constant SOUTH_EAST_RESIZE. */
	public static final int SOUTH_EAST_RESIZE = 7;

	/** The Constant resizeStep. */
	private static final int resizeStep = 20;

	/** The location. */
	private Point location;

	/** The frame to resize. */
	private Window frameToResize;

	/** The panel to resize. */
	private JPanel panelToResize;

	/** The orientation. */
	private int orientation;

	/** The minimun width. */
	private int minimunWidth;

	/** The minimun height. */
	private int minimunHeight;

	/** The frame. */
	private DockableWindow frame;

	/**
	 * Instantiates a new mouse resize listener.
	 * 
	 * @param frame
	 *            the frame
	 * @param frameToResize
	 *            the frame to resize
	 * @param panelToResize
	 *            the panel to resize
	 * @param orientation
	 *            the orientation
	 */
	public MouseResizeListener(DockableWindow frame, Window frameToResize, JPanel panelToResize, int orientation) {
		this.frame = frame;
		this.frameToResize = frameToResize;
		this.panelToResize = panelToResize;
		this.orientation = orientation;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	@Override
	public void mouseClicked(MouseEvent e) {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent
	 * )
	 */
	@Override
	public void mouseDragged(MouseEvent me) {
		Dimension sizeOfFrame = frameToResize.getSize();
		int x = me.getX() - location.x;
		int y = me.getY() - location.y;
		if (!frame.isHidden()) {
			if (x > resizeStep || x < -resizeStep || y > resizeStep || y < -resizeStep) {
				if (orientation == NORTH_RESIZE) {
					if (sizeOfFrame.height - y >= minimunHeight) {
						frameToResize.setSize(sizeOfFrame.width, sizeOfFrame.height - y);
						frameToResize.setLocation(frameToResize.getLocation().x, frameToResize.getLocation().y + y);
					}
				} else if (orientation == SOUTH_RESIZE) {
					if (sizeOfFrame.height + y >= minimunHeight) {
						frameToResize.setSize(sizeOfFrame.width, sizeOfFrame.height + y);
					}
				} else if (orientation == EAST_RESIZE) {
					if (sizeOfFrame.width + x >= minimunWidth) {
						frameToResize.setSize(sizeOfFrame.width + x, sizeOfFrame.height);
					}
				} else if (orientation == WEST_RESIZE) {
					if (sizeOfFrame.width - x >= minimunWidth) {
						frameToResize.setSize(sizeOfFrame.width - x, sizeOfFrame.height);
						frameToResize.setLocation(frameToResize.getLocation().x + x, frameToResize.getLocation().y);
					}
				} else if (orientation == NORTH_WEST_RESIZE) {
					if (sizeOfFrame.width + x >= minimunWidth && sizeOfFrame.height - y >= minimunHeight) {
						frameToResize.setSize(sizeOfFrame.width - x, sizeOfFrame.height - y);
						frameToResize.setLocation(frameToResize.getLocation().x + x, frameToResize.getLocation().y + y);
					}
				} else if (orientation == NORTH_EAST_RESIZE) {
					if (sizeOfFrame.width + x >= minimunWidth && sizeOfFrame.height - y >= minimunHeight) {
						frameToResize.setSize(sizeOfFrame.width + x, sizeOfFrame.height - y);
						frameToResize.setLocation(frameToResize.getLocation().x, frameToResize.getLocation().y + y);
					}
				} else if (orientation == SOUTH_WEST_RESIZE) {
					if (sizeOfFrame.width - x >= minimunWidth && sizeOfFrame.height + y >= minimunHeight) {
						frameToResize.setSize(sizeOfFrame.width - x, sizeOfFrame.height + y);
						frameToResize.setLocation(frameToResize.getLocation().x + x, frameToResize.getLocation().y);
					}
				} else if (orientation == SOUTH_EAST_RESIZE) {
					if (sizeOfFrame.width + x >= minimunWidth && sizeOfFrame.height + y >= minimunHeight) {
						frameToResize.setSize(sizeOfFrame.width + x, sizeOfFrame.height + y);
					}
				}
				panelToResize.setSize(frameToResize.getSize());
				frameToResize.validate();
				frameToResize.repaint();
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	@Override
	public void mouseEntered(MouseEvent e) {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	@Override
	public void mouseExited(MouseEvent e) {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	@Override
	public void mouseMoved(MouseEvent me) {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	@Override
	public void mousePressed(MouseEvent me) {
		location = me.getPoint();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	@Override
	public void mouseReleased(MouseEvent me) {
		// Nothing to do
	}

	/**
	 * Sets the minimun height.
	 * 
	 * @param minimunHeight
	 *            the new minimun height
	 */
	public void setMinimunHeight(int minimunHeight) {
		this.minimunHeight = minimunHeight;
	}

	/**
	 * Sets the minimun width.
	 * 
	 * @param minimunWidth
	 *            the new minimun width
	 */
	public void setMinimunWidth(int minimunWidth) {
		this.minimunWidth = minimunWidth;
	}

}
