/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.model;

import java.util.ArrayList;
import java.util.List;

import javax.swing.table.AbstractTableModel;

import net.sourceforge.atunes.kernel.modules.radio.Radio;
import net.sourceforge.atunes.utils.LanguageTool;

/**
 * The Class RadioBrowserTableModel.
 */
public class RadioBrowserTableModel extends AbstractTableModel {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1997644065009669746L;

	/** The radios. */
	private List<Radio> radios = new ArrayList<Radio>();

	/**
	 * Adds the radio.
	 * 
	 * @param r
	 *            the r
	 */
	public void addRadio(Radio r) {
		radios.add(r);
	}

	/**
	 * Gets the radio.
	 * 
	 * @param row
	 *            the row
	 * 
	 * @return the radio
	 */
	public Radio getRadio(int row) {
		return radios.get(row);
	}

	/**
	 * Clear.
	 */
	public void clear() {
		radios.clear();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	@Override
	public int getColumnCount() {
		return 3;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.table.AbstractTableModel#getColumnName(int)
	 */
	@Override
	public String getColumnName(int column) {
		if (column == 0) {
			return LanguageTool.getString("NAME");
		} else if (column == 1) {
			return LanguageTool.getString("URL");
		} else if (column == 2) {
			return LanguageTool.getString("LABEL");
		}
		return "";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	@Override
	public int getRowCount() {
		return radios.size();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		Radio radio = radios.get(rowIndex);
		if (columnIndex == 0) {
			return radio.getName();
		} else if (columnIndex == 1) {
			return radio.getUrl();
		} else if (columnIndex == 2) {
			return radio.getLabel();
		}
		return "";
	}

}
