/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.views.dialogs;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import net.sourceforge.atunes.Constants;
import net.sourceforge.atunes.gui.GuiUtils;
import net.sourceforge.atunes.gui.views.controls.CustomButton;
import net.sourceforge.atunes.gui.views.controls.CustomModalDialog;
import net.sourceforge.atunes.utils.LanguageTool;

/**
 * The Class EditTagDialog.
 */
public class EditTagDialog extends CustomModalDialog {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 3395292301087643037L;

	/** The title label. */
	private JLabel titleLabel;

	/** The title text field. */
	private JTextField titleTextField;

	/** The album label. */
	private JLabel albumLabel;

	/** The album text field. */
	private JComboBox albumTextField;

	/** The artist label. */
	private JLabel artistLabel;

	/** The artist text field. */
	private JComboBox artistTextField;

	/** The year label. */
	private JLabel yearLabel;

	/** The year text field. */
	private JTextField yearTextField;

	/** The genre label. */
	private JLabel genreLabel;

	/** The genre combo box. */
	private JComboBox genreComboBox;

	/** The comment label. */
	private JLabel commentLabel;

	/** The comment text area. */
	private JTextArea commentTextArea;

	/** The track number label. */
	private JLabel trackNumberLabel;

	/** The track number text field. */
	private JTextField trackNumberTextField;

	/** The cover label */
	private JLabel coverLabel;

	/** The cover */
	private JLabel cover;

	/** The button for adding a cover */
	private JButton coverButton;

	/** The button for removing the cover */
	private JButton removeCoverButton;

	/** The lyrics label. */
	private JLabel lyricsLabel;

	/** The lyrics text area. */
	private JTextArea lyricsTextArea;

	/** The composer label. */
	private JLabel composerLabel;

	/** The composer text field. */
	private JTextField composerTextField;

	/** The album artist label. */
	private JLabel albumArtistLabel;

	/** The album artist text field. */
	private JTextField albumArtistTextField;

	/** The ok button. */
	private JButton okButton;

	/** The cancel button. */
	private JButton cancelButton;

	/**
	 * Instantiates a new edits the tag dialog.
	 * 
	 * @param owner
	 *            the owner
	 */
	public EditTagDialog(JFrame owner) {
		super(owner, 480, 560, true);
		setTitle(LanguageTool.getString("EDIT_TAG"));
		setResizable(false);
		add(getContent());
		setLocationRelativeTo(null);
		GuiUtils.applyComponentOrientation(this);
		enableCloseActionWithEscapeKey();
	}

	/**
	 * The main method.
	 * 
	 * @param args
	 *            the arguments
	 */
	public static void main(String[] args) {
		new EditTagDialog(null).setVisible(true);
	}

	/**
	 * Gets the album artist text field.
	 * 
	 * @return the album artist text field
	 */
	public JTextField getAlbumArtistTextField() {
		return albumArtistTextField;
	}

	/**
	 * Gets the album label.
	 * 
	 * @return the album label
	 */
	public JLabel getAlbumLabel() {
		return albumLabel;
	}

	/**
	 * Gets the album text field.
	 * 
	 * @return the album text field
	 */
	public JComboBox getAlbumTextField() {
		return albumTextField;
	}

	/**
	 * Gets the artist label.
	 * 
	 * @return the artist label
	 */
	public JLabel getArtistLabel() {
		return artistLabel;
	}

	/**
	 * Gets the artist text field.
	 * 
	 * @return the artist text field
	 */
	public JComboBox getArtistTextField() {
		return artistTextField;
	}

	/**
	 * Gets the cancel button.
	 * 
	 * @return the cancel button
	 */
	public JButton getCancelButton() {
		return cancelButton;
	}

	/**
	 * Gets the comment label.
	 * 
	 * @return the comment label
	 */
	public JLabel getCommentLabel() {
		return commentLabel;
	}

	/**
	 * Gets the comment text area.
	 * 
	 * @return the comment text area
	 */
	public JTextArea getCommentTextArea() {
		return commentTextArea;
	}

	/**
	 * Gets the composer label.
	 * 
	 * @return the composer label
	 */
	public JLabel getComposerLabel() {
		return composerLabel;
	}

	/**
	 * Gets the composer text field.
	 * 
	 * @return the composer text field
	 */
	public JTextField getComposerTextField() {
		return composerTextField;
	}

	/**
	 * Gets the content.
	 * 
	 * @return the content
	 */
	private JPanel getContent() {
		JPanel panel = new JPanel(new GridBagLayout());
		titleLabel = new JLabel(LanguageTool.getString("TITLE"));
		titleTextField = new JTextField();
		albumLabel = new JLabel(LanguageTool.getString("ALBUM"));
		albumTextField = new JComboBox();
		albumTextField.setEditable(true);
		artistLabel = new JLabel(LanguageTool.getString("ARTIST"));
		artistTextField = new JComboBox();
		artistTextField.setEditable(true);
		yearLabel = new JLabel(LanguageTool.getString("YEAR"));
		yearTextField = new JTextField();
		genreLabel = new JLabel(LanguageTool.getString("GENRE"));
		genreComboBox = new JComboBox();
		coverLabel = new JLabel(LanguageTool.getString("COVER"));
		genreComboBox.setEditable(true);
		cover = new JLabel();
		cover.setPreferredSize(new Dimension(Constants.DIALOG_IMAGE_WIDTH, Constants.DIALOG_IMAGE_HEIGHT));
		cover.setMinimumSize(new Dimension(Constants.DIALOG_IMAGE_WIDTH, Constants.DIALOG_IMAGE_HEIGHT));
		cover.setBorder(BorderFactory.createLineBorder(GuiUtils.getBorderColor()));
		coverButton = new JButton(LanguageTool.getString("EDIT_COVER"));
		removeCoverButton = new JButton(LanguageTool.getString("REMOVE_COVER"));
		JPanel coverButtonsPanel = new JPanel(new FlowLayout());
		coverButtonsPanel.add(coverButton);
		coverButtonsPanel.add(removeCoverButton);
		commentLabel = new JLabel(LanguageTool.getString("COMMENT"));
		commentTextArea = new JTextArea();
		commentTextArea.setBorder(BorderFactory.createEmptyBorder());
		JScrollPane scrollPane = new JScrollPane(commentTextArea);
		lyricsLabel = new JLabel(LanguageTool.getString("LYRICS"));
		lyricsTextArea = new JTextArea();
		lyricsTextArea.setBorder(BorderFactory.createEmptyBorder());
		JScrollPane scrollPane2 = new JScrollPane(lyricsTextArea);
		trackNumberLabel = new JLabel(LanguageTool.getString("TRACK"));
		trackNumberTextField = new JTextField();
		composerLabel = new JLabel(LanguageTool.getString("COMPOSER"));
		composerTextField = new JTextField();
		albumArtistLabel = new JLabel(LanguageTool.getString("ALBUM_ARTIST"));
		albumArtistTextField = new JTextField();

		okButton = new CustomButton(null, LanguageTool.getString("OK"));
		cancelButton = new CustomButton(null, LanguageTool.getString("CANCEL"));

		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(10, 10, 2, 10);
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		c.anchor = GridBagConstraints.NORTHWEST;
		panel.add(titleLabel, c);

		c.gridx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		c.gridwidth = 2;
		panel.add(titleTextField, c);

		c.gridx = 0;
		c.gridy = 1;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		c.insets = new Insets(2, 10, 2, 10);
		c.gridwidth = 1;
		panel.add(albumArtistLabel, c);

		c.gridx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		c.gridwidth = 2;
		panel.add(albumArtistTextField, c);

		c.gridx = 0;
		c.gridy = 2;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		c.insets = new Insets(2, 10, 2, 10);
		c.gridwidth = 1;
		panel.add(artistLabel, c);

		c.gridx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		c.gridwidth = 2;
		panel.add(artistTextField, c);

		c.gridx = 0;
		c.gridy = 3;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		c.gridheight = 1;
		c.gridwidth = 1;
		panel.add(albumLabel, c);

		c.gridx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		c.gridwidth = 2;
		panel.add(albumTextField, c);

		c.gridx = 0;
		c.gridy = 4;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		c.gridwidth = 1;
		panel.add(yearLabel, c);

		c.gridx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		c.gridwidth = 2;
		panel.add(yearTextField, c);

		c.gridx = 0;
		c.gridy = 5;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		c.gridwidth = 1;
		panel.add(trackNumberLabel, c);

		c.gridx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		c.gridwidth = 2;
		panel.add(trackNumberTextField, c);

		c.gridx = 0;
		c.gridy = 6;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		c.gridwidth = 1;
		panel.add(genreLabel, c);

		c.gridx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		c.gridwidth = 2;
		panel.add(genreComboBox, c);

		c.gridx = 0;
		c.gridy = 7;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		c.insets = new Insets(2, 10, 2, 10);
		c.gridwidth = 1;
		panel.add(composerLabel, c);

		c.gridx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		c.gridwidth = 2;
		panel.add(composerTextField, c);

		c.gridx = 0;
		c.gridy = 8;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		c.gridwidth = 1;
		panel.add(coverLabel, c);

		c.gridx = 1;
		c.gridy = 8;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		c.gridwidth = 1;
		panel.add(cover, c);

		c.gridx = 2;
		c.gridy = 8;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		c.gridwidth = 1;
		panel.add(coverButtonsPanel, c);

		c.gridx = 0;
		c.gridy = 9;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		c.gridwidth = 1;
		panel.add(commentLabel, c);

		c.gridx = 1;
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 1;
		c.weighty = 1;
		c.gridwidth = 2;
		panel.add(scrollPane, c);

		c.gridx = 0;
		c.gridy = 10;
		c.fill = GridBagConstraints.NONE;
		c.weightx = 0;
		c.gridwidth = 1;
		panel.add(lyricsLabel, c);

		c.gridx = 1;
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 1;
		c.weighty = 0;
		c.gridwidth = 2;
		panel.add(scrollPane2, c);

		JPanel auxPanel = new JPanel();
		auxPanel.setOpaque(false);
		auxPanel.add(okButton);
		auxPanel.add(cancelButton);

		c.gridx = 0;
		c.gridy = 11;
		c.gridwidth = 3;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		c.weighty = 0;
		panel.add(auxPanel, c);

		return panel;
	}

	/**
	 * Gets the genre combo box.
	 * 
	 * @return the genre combo box
	 */
	public JComboBox getGenreComboBox() {
		return genreComboBox;
	}

	/**
	 * Gets the genre label.
	 * 
	 * @return the genre label
	 */
	public JLabel getGenreLabel() {
		return genreLabel;
	}

	/**
	 * Gets the lyrics text area.
	 * 
	 * @return the lyrics text area
	 */
	public JTextArea getLyricsTextArea() {
		return lyricsTextArea;
	}

	/**
	 * Gets the ok button.
	 * 
	 * @return the ok button
	 */
	public JButton getOkButton() {
		return okButton;
	}

	/**
	 * Gets the title label.
	 * 
	 * @return the title label
	 */
	public JLabel getTitleLabel() {
		return titleLabel;
	}

	/**
	 * Gets the title text field.
	 * 
	 * @return the title text field
	 */
	public JTextField getTitleTextField() {
		return titleTextField;
	}

	/**
	 * Gets the track number label.
	 * 
	 * @return the track number label
	 */
	public JLabel getTrackNumberLabel() {
		return trackNumberLabel;
	}

	/**
	 * Gets the track number text field.
	 * 
	 * @return the track number text field
	 */
	public JTextField getTrackNumberTextField() {
		return trackNumberTextField;
	}

	/**
	 * Gets the year label.
	 * 
	 * @return the year label
	 */
	public JLabel getYearLabel() {
		return yearLabel;
	}

	/**
	 * Gets the year text field.
	 * 
	 * @return the year text field
	 */
	public JTextField getYearTextField() {
		return yearTextField;
	}

	public JLabel getCoverLabel() {
		return coverLabel;
	}

	public JLabel getCover() {
		return cover;
	}

	public JButton getCoverButton() {
		return coverButton;
	}

	public JButton getRemoveCoverButton() {
		return removeCoverButton;
	}

}
