/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.views.dialogs;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.WindowConstants;

import net.sourceforge.atunes.gui.GuiUtils;
import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.views.controls.CustomFrame;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;

/**
 * The Class PodcastFeedEntryDownloadProgressDialog.
 */
public class PodcastFeedEntryDownloadProgressDialog extends CustomFrame {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 5792663290880992661L;

	/** The progress bar. */
	private JProgressBar progressBar;

	/** The imagelabel. */
	private JLabel imagelabel;

	/** The info label. */
	private JLabel infoLabel;

	/** The received label. */
	private JLabel receivedLabel;

	/** The separator label. */
	private JLabel separatorLabel;

	/** The total size label. */
	private JLabel totalSizeLabel;

	/** The cancel button. */
	private JButton cancelButton;

	/**
	 * Instantiates a new podcast feed entry download progress dialog.
	 * 
	 * @param title
	 *            the title
	 */
	public PodcastFeedEntryDownloadProgressDialog(String title) {
		super(title);
		add(getConentent());
		setSize(450, 150);
		setResizable(false);
		setLocationRelativeTo(null);
		GuiUtils.applyComponentOrientation(this);
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	}

	/**
	 * Gets the conentent.
	 * 
	 * @return the conentent
	 */
	private JPanel getConentent() {
		JPanel panel = new JPanel(new GridBagLayout());
		progressBar = new JProgressBar();
		progressBar.setBorder(BorderFactory.createEmptyBorder());
		progressBar.setStringPainted(true);
		imagelabel = new JLabel(ImageLoader.RSS);
		infoLabel = new JLabel();
		receivedLabel = new JLabel();
		separatorLabel = new JLabel("/");
		totalSizeLabel = new JLabel();
		cancelButton = new JButton(LanguageTool.getString("CANCEL"));

		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.gridheight = 5;
		c.insets = new Insets(0, 20, 0, 0);
		panel.add(imagelabel, c);
		c.gridheight = 1;
		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(5, 20, 0, 20);
		c.anchor = GridBagConstraints.WEST;
		panel.add(infoLabel, c);
		c.gridx = 2;
		c.weightx = 0;
		c.fill = GridBagConstraints.NONE;
		c.insets = new Insets(5, 0, 0, 3);
		c.anchor = GridBagConstraints.EAST;
		panel.add(receivedLabel, c);
		c.gridx = 3;
		c.insets = new Insets(5, 0, 0, 0);
		panel.add(separatorLabel, c);
		c.gridx = 4;
		c.insets = new Insets(5, 0, 0, 20);
		panel.add(totalSizeLabel, c);
		c.gridx = 1;
		c.gridy = 1;
		c.weightx = 1;
		c.gridwidth = 4;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(5, 20, 5, 20);
		c.anchor = GridBagConstraints.CENTER;
		panel.add(progressBar, c);
		c.gridy = 2;
		c.fill = GridBagConstraints.NONE;
		c.anchor = GridBagConstraints.CENTER;
		panel.add(cancelButton, c);

		return panel;
	}

	/**
	 * Sets the info text.
	 * 
	 * @param s
	 *            the new info text
	 */
	public void setInfoText(String s) {
		infoLabel.setText(s);
	}

	/**
	 * Sets the progress.
	 * 
	 * @param value
	 *            the new progress
	 */
	public void setProgress(int value) {
		progressBar.setValue(value);
	}

	/**
	 * Sets the byte progress.
	 * 
	 * @param value
	 *            the new byte progress
	 */
	public void setByteProgress(long value) {
		receivedLabel.setText(StringUtils.fromByteToMegaOrGiga(value));
	}

	/**
	 * Sets the total bytes.
	 * 
	 * @param value
	 *            the new total bytes
	 */
	public void setTotalBytes(long value) {
		totalSizeLabel.setText(StringUtils.fromByteToMegaOrGiga(value));
	}

	/**
	 * Adds the cancel button action listener.
	 * 
	 * @param a
	 *            the a
	 */
	public void addCancelButtonActionListener(ActionListener a) {
		cancelButton.addActionListener(a);
	}

	/**
	 * The main method.
	 * 
	 * @param args
	 *            the arguments
	 */
	public static void main(String[] args) {
		new PodcastFeedEntryDownloadProgressDialog("test").setVisible(true);
	}
}
