/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.views.dialogs.editPreferences;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;

import net.sourceforge.atunes.gui.GuiUtils;
import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.kernel.handlers.DesktopHandler;
import net.sourceforge.atunes.utils.LanguageTool;

/**
 * The Class AudioScrobblerPanel.
 */
public class ContextInformationPanel extends PreferencesPanel {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -9216216930198145476L;

	/** The activate audio scrobbler. */
	JCheckBox activateContextInformation;

	/** The save pictures. */
	JCheckBox savePictures;

	/**
	 * Checkbox to show text in tabs
	 */
	JCheckBox showTextInTabs;

	/** The clear cache. */
	private JButton clearCache;

	/** The info label. */
	private JLabel info;

	/**
	 * Instantiates a new audio scrobbler panel.
	 */
	public ContextInformationPanel() {
		super();
		activateContextInformation = new JCheckBox(LanguageTool.getString("ACTIVATE_CONTEXT_INFORMATION"));
		savePictures = new JCheckBox(LanguageTool.getString("SAVE_PICTURES_TO_AUDIO_FOLDERS"));
		showTextInTabs = new JCheckBox(LanguageTool.getString("SHOW_CONTEXT_TABS_TEXT"));
		activateContextInformation.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				savePictures.setEnabled(activateContextInformation.isSelected());
				showTextInTabs.setEnabled(activateContextInformation.isSelected());
			}
		});
		clearCache = new JButton(LanguageTool.getString("CLEAR_CACHE"));
		info = new JLabel(ImageLoader.POWERED_BY_LAST_FM, GuiUtils.getComponentOrientationAsSwingConstant());
		info.addMouseListener((new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				DesktopHandler.getInstance().openURL("http://www.last.fm");
			}
		}));

		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		add(activateContextInformation, c);
		c.gridx = 1;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		add(info, c);
		c.gridx = 0;
		c.gridy = 1;
		add(savePictures, c);
		c.gridy = 2;
		add(showTextInTabs, c);
		c.gridy = 3;
		c.weightx = 0;
		c.weighty = 1;
		c.insets = new Insets(10, 0, 10, 0);
		c.anchor = GuiUtils.getComponentOrientation().isLeftToRight() ? GridBagConstraints.NORTHWEST : GridBagConstraints.NORTHEAST;
		add(clearCache, c);
	}

	/**
	 * Gets the clear cache.
	 * 
	 * @return the clear cache
	 */
	public JButton getClearCache() {
		return clearCache;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.gui.views.dialogs.editPreferences.PreferencesPanel
	 * #getResult()
	 */

	@Override
	public Map<String, Object> getResult() {
		Map<String, Object> result = new HashMap<String, Object>();
		result.put(LanguageTool.getString("ACTIVATE_CONTEXT_INFORMATION"), activateContextInformation.isSelected());
		result.put(LanguageTool.getString("SAVE_PICTURES_TO_AUDIO_FOLDERS"), savePictures.isSelected());
		result.put(LanguageTool.getString("SHOW_CONTEXT_TABS_TEXT"), showTextInTabs.isSelected());
		return result;
	}

	/**
	 * Sets the activate audio scrobbler.
	 * 
	 * @param activate
	 *            the new activate audio scrobbler
	 */
	public void setActivateContextInformation(boolean activate) {
		activateContextInformation.setSelected(activate);
		savePictures.setEnabled(activate);
	}

	/**
	 * Sets the save pictures.
	 * 
	 * @param save
	 *            the new save pictures
	 */
	public void setSavePictures(boolean save) {
		savePictures.setSelected(save);
	}

	/**
	 * Sets the show text in tabs checkbox value
	 * 
	 * @param show
	 */
	public void setShowTextInTabs(boolean show) {
		showTextInTabs.setSelected(show);
	}

}
