/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.views.menu;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;

import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.kernel.actions.Actions;
import net.sourceforge.atunes.kernel.actions.ClearPlayListAction;
import net.sourceforge.atunes.kernel.actions.CreatePlayListWithSelectedAlbumsAction;
import net.sourceforge.atunes.kernel.actions.CreatePlayListWithSelectedArtistsAction;
import net.sourceforge.atunes.kernel.actions.EditPreferencesAction;
import net.sourceforge.atunes.kernel.actions.ExitAction;
import net.sourceforge.atunes.kernel.actions.LoadPlayListAction;
import net.sourceforge.atunes.kernel.actions.MoveDownAction;
import net.sourceforge.atunes.kernel.actions.MoveToBottomAction;
import net.sourceforge.atunes.kernel.actions.MoveToTopAction;
import net.sourceforge.atunes.kernel.actions.MoveUpAction;
import net.sourceforge.atunes.kernel.actions.RefreshRepositoryAction;
import net.sourceforge.atunes.kernel.actions.RemoveFromPlayListAction;
import net.sourceforge.atunes.kernel.actions.RepairAlbumNamesAction;
import net.sourceforge.atunes.kernel.actions.RepairGenresAction;
import net.sourceforge.atunes.kernel.actions.RepairTrackNumbersAction;
import net.sourceforge.atunes.kernel.actions.RipCDAction;
import net.sourceforge.atunes.kernel.actions.SavePlayListAction;
import net.sourceforge.atunes.kernel.actions.SelectRepositoryAction;
import net.sourceforge.atunes.kernel.actions.SetPlayListSelectionAsFavoriteAlbumAction;
import net.sourceforge.atunes.kernel.actions.SetPlayListSelectionAsFavoriteArtistAction;
import net.sourceforge.atunes.kernel.actions.SetPlayListSelectionAsFavoriteSongAction;
import net.sourceforge.atunes.kernel.actions.ShowAudioObjectPropertiesAction;
import net.sourceforge.atunes.kernel.actions.ShowContextInformationAction;
import net.sourceforge.atunes.kernel.actions.ShowDeviceViewAction;
import net.sourceforge.atunes.kernel.actions.ShowEqualizerAction;
import net.sourceforge.atunes.kernel.actions.ShowFavoritesViewAction;
import net.sourceforge.atunes.kernel.actions.ShowNavigatorAction;
import net.sourceforge.atunes.kernel.actions.ShowPlayListItemInfoAction;
import net.sourceforge.atunes.kernel.actions.ShowPodcastViewAction;
import net.sourceforge.atunes.kernel.actions.ShowRadioViewAction;
import net.sourceforge.atunes.kernel.actions.ShowRepositoryViewAction;
import net.sourceforge.atunes.kernel.actions.ShowStatsAction;
import net.sourceforge.atunes.kernel.actions.ShowToolbarAction;
import net.sourceforge.atunes.kernel.actions.ShufflePlayListAction;
import net.sourceforge.atunes.kernel.actions.TogglePlayListFilterAction;
import net.sourceforge.atunes.kernel.actions.VolumeDownAction;
import net.sourceforge.atunes.kernel.actions.VolumeUpAction;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;

/**
 * The Class ApplicationMenuBar.
 */
public class ApplicationMenuBar extends JMenuBar {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 234977404080329591L;

	/** The file. */
	private JMenu file;

	/** The edit. */
	private JMenu edit;

	/** The view. */
	private JMenu view;

	/** The play list. */
	private JMenu playList;

	/** The tools. */
	private JMenu tools;

	/** The device. */
	private JMenu device;

	/** The about. */
	private JMenu about;

	// File Menu
	/** The select repository. */
	private JMenuItem selectRepository;

	/** The refresh repository. */
	private JMenuItem refreshRepository;

	/** The exit. */
	private JMenuItem exit;

	// Edit Menu
	/** The repair. */
	private JMenuItem repair;

	/** The edit preferences. */
	private JMenuItem editPreferences;

	// View Menu
	/** The repository view. */
	private JMenuItem repositoryView;

	/** The favorite view. */
	private JMenuItem favoriteView;

	/** The device view. */
	private JMenuItem deviceView;

	/** The radio view. */
	private JMenuItem radioView;

	/** The podcast feed view. */
	private JMenuItem podcastFeedView;

	/**
	 * Check box to show / hide tool bar
	 */
	private JCheckBoxMenuItem showToolBar;

	/** The show status bar. */
	private JCheckBoxMenuItem showStatusBar;

	/** The show navigation panel. */
	private JCheckBoxMenuItem showNavigationPanel;

	/** The show navigation table. */
	private JCheckBoxMenuItem showNavigationTable;

	/** The show properties. */
	private JCheckBoxMenuItem showProperties;

	/** The show context information panel. */
	private JCheckBoxMenuItem showContextInformationPanel;

	/** The show osd. */
	private JCheckBoxMenuItem showOSD;

	/** The full screen. */
	private JMenuItem fullScreen;

	// Player Menu
	/** The player. */
	private JMenuItem player;

	/** The equalizer. */
	private JMenuItem equalizer;

	/** The volume down. */
	private JMenuItem volumeDown;

	/** The volume up. */
	private JMenuItem volumeUp;

	/** The mute. */
	private JMenuItem mute;

	// PlayList Menu
	/** The play list play. */
	private JMenuItem playListPlay;

	/** The play list open folder. */
	private JMenuItem playListOpenFolder;

	/** The play list tags. */
	private JMenuItem playListTags;

	/** The play list edit tag. */
	private JMenuItem playListEditTag;

	/** The play list auto set title. */
	private JMenuItem playListAutoSetTitle;

	/** The play list auto set lyrics. */
	private JMenuItem playListAutoSetLyrics;

	/** The play list auto set track. */
	private JMenuItem playListAutoSetTrack;

	/** The play list auto set genre. */
	private JMenuItem playListAutoSetGenre;

	/** The play list save. */
	private JMenuItem playListSave;

	/** The play list load. */
	private JMenuItem playListLoad;

	/** The play list filter. */
	private JMenuItem playListFilter;

	/** The play list info. */
	private JMenuItem playListInfo;

	/** The play list delete. */
	private JMenuItem playListDelete;

	/** The play list clear. */
	private JMenuItem playListClear;

	/** The play list top. */
	private JMenuItem playListTop;

	/** The play list up. */
	private JMenuItem playListUp;

	/** The play list down. */
	private JMenuItem playListDown;

	/** The play list bottom. */
	private JMenuItem playListBottom;

	/** The play list shuffle. */
	private JMenuItem playListShuffle;

	/** The play list favorite song. */
	private JMenuItem playListFavoriteSong;

	/** The play list favorite album. */
	private JMenuItem playListFavoriteAlbum;

	/** The play list favorite artist. */
	private JMenuItem playListFavoriteArtist;

	/** The play list artist. */
	private JMenuItem playListArtist;

	/** The play list album. */
	private JMenuItem playListAlbum;

	/** The play list show buttons. */
	private JCheckBoxMenuItem playListShowButtons;

	// Tools Menu
	/** The tools export. */
	private JMenuItem toolsExport;

	/** The rip cd. */
	private JMenuItem ripCd;

	/** The stats. */
	private JMenuItem stats;

	/** The cover navigator. */
	private JMenuItem coverNavigator;

	/** The custom search. */
	private JMenuItem customSearch;

	// Radio Menu
	/** The add radio. */
	private JMenuItem addRadio;

	/** The show radio browser. */
	private JMenuItem showRadioBrowser;

	// Podcast Menu
	/** The add podcast feed. */
	private JMenuItem addPodcastFeed;

	// Repair Menu
	/** The repair track numbers. */
	private JMenuItem repairTrackNumbers;

	/** The repair genres. */
	private JMenuItem repairGenres;

	/** The repair album names. */
	private JMenuItem repairAlbumNames;

	// Device Menu
	/** The device connect. */
	private JMenuItem deviceConnect;

	/** The device refresh. */
	private JMenuItem deviceRefresh;

	/** The device disconnect. */
	private JMenuItem deviceDisconnect;

	// Smart playlist Sub-Menu - in Playlist menu
	/** The smart play list. */
	private JMenuItem smartPlayList;

	/** The add10 random songs. */
	private JMenuItem add10RandomSongs;

	/** The add50 random songs. */
	private JMenuItem add50RandomSongs;

	/** The add100 random songs. */
	private JMenuItem add100RandomSongs;

	/** The add10 songs most played. */
	private JMenuItem add10SongsMostPlayed;

	/** The add50 songs most played. */
	private JMenuItem add50SongsMostPlayed;

	/** The add100 songs most played. */
	private JMenuItem add100SongsMostPlayed;

	/** The add1 artists most played. */
	private JMenuItem add1ArtistsMostPlayed;

	/** The add5 artists most played. */
	private JMenuItem add5ArtistsMostPlayed;

	/** The add10 artists most played. */
	private JMenuItem add10ArtistsMostPlayed;

	/** The add1 albums most played. */
	private JMenuItem add1AlbumsMostPlayed;

	/** The add5 albums most played. */
	private JMenuItem add5AlbumsMostPlayed;

	/** The add10 albums most played. */
	private JMenuItem add10AlbumsMostPlayed;

	/** The add10 unplayed songs. */
	private JMenuItem add10UnplayedSongs;

	/** The add50 unplayed songs. */
	private JMenuItem add50UnplayedSongs;

	/** The add100 unplayed songs. */
	private JMenuItem add100UnplayedSongs;

	// About Menu
	/** The check updates. */
	private JMenuItem checkUpdates;

	/** The report bug or request feature. */
	private JMenuItem reportBugOrRequestFeature;

	/** The show log file. */
	private JMenuItem showLogFile;

	/** The about item. */
	private JMenuItem aboutItem;

	/**
	 * Instantiates a new application menu bar.
	 */
	public ApplicationMenuBar() {
		super();
		addMenus();
	}

	// Menu structure
	/**
	 * Adds the menus.
	 */
	private void addMenus() {
		file = new JMenu(LanguageTool.getString("FILE"));
		selectRepository = new JMenuItem(Actions.getAction(SelectRepositoryAction.class));
		file.add(selectRepository);
		refreshRepository = new JMenuItem(Actions.getAction(RefreshRepositoryAction.class));
		file.add(refreshRepository);
		file.add(new JSeparator());
		exit = new JMenuItem(Actions.getAction(ExitAction.class));
		file.add(exit);

		edit = new JMenu(LanguageTool.getString("EDIT"));
		player = new JMenu(LanguageTool.getString("VOLUME"));
		edit.add(player);
		volumeUp = new JMenuItem(Actions.getAction(VolumeUpAction.class));
		volumeDown = new JMenuItem(Actions.getAction(VolumeDownAction.class));
		mute = new JMenuItem(LanguageTool.getString("MUTE"));
		equalizer = new JMenuItem(Actions.getAction(ShowEqualizerAction.class));
		player.add(equalizer);
		player.add(volumeUp);
		player.add(volumeDown);
		player.add(mute);
		editPreferences = new JMenuItem(Actions.getAction(EditPreferencesAction.class));
		edit.add(editPreferences);
		edit.add(new JSeparator());
		repair = new JMenu(LanguageTool.getString("REPAIR"));
		repairTrackNumbers = new JMenuItem(Actions.getAction(RepairTrackNumbersAction.class));
		repair.add(repairTrackNumbers);
		repairGenres = new JMenuItem(Actions.getAction(RepairGenresAction.class));
		repair.add(repairGenres);
		repairAlbumNames = new JMenuItem(Actions.getAction(RepairAlbumNamesAction.class));
		repair.add(repairAlbumNames);
		edit.add(repair);

		view = new JMenu(LanguageTool.getString("VIEW"));
		repositoryView = new JMenuItem(Actions.getAction(ShowRepositoryViewAction.class));
		favoriteView = new JMenuItem(Actions.getAction(ShowFavoritesViewAction.class));
		deviceView = new JMenuItem(Actions.getAction(ShowDeviceViewAction.class));
		radioView = new JMenuItem(Actions.getAction(ShowRadioViewAction.class));
		podcastFeedView = new JMenuItem(Actions.getAction(ShowPodcastViewAction.class));
		showToolBar = new JCheckBoxMenuItem(Actions.getAction(ShowToolbarAction.class));
		showStatusBar = new JCheckBoxMenuItem(LanguageTool.getString("SHOW_STATUS_BAR"), ImageLoader.STATUS_BAR);
		showNavigationPanel = new JCheckBoxMenuItem(Actions.getAction(ShowNavigatorAction.class));
		showNavigationTable = new JCheckBoxMenuItem(LanguageTool.getString("SHOW_NAVIGATION_TABLE"), ImageLoader.NAVIGATION_TABLE);
		showProperties = new JCheckBoxMenuItem(Actions.getAction(ShowAudioObjectPropertiesAction.class));
		showContextInformationPanel = new JCheckBoxMenuItem(Actions.getAction(ShowContextInformationAction.class));
		showOSD = new JCheckBoxMenuItem(LanguageTool.getString("SHOW_OSD"), ImageLoader.OSD);
		fullScreen = new JMenuItem(LanguageTool.getString("FULL_SCREEN"), ImageLoader.FULLSCREEN);
		playListShowButtons = new JCheckBoxMenuItem(LanguageTool.getString("SHOW_PLAYLIST_CONTROLS"), ImageLoader.PLAY_LIST_CONTROLS);

		view.add(repositoryView);
		view.add(favoriteView);
		view.add(deviceView);
		view.add(radioView);
		view.add(podcastFeedView);
		view.add(new JSeparator());
		view.add(showToolBar);
		view.add(showStatusBar);
		view.add(playListShowButtons);
		view.add(showNavigationPanel);
		view.add(showNavigationTable);
		view.add(showProperties);
		view.add(showContextInformationPanel);
		view.add(showOSD);
		view.add(new JSeparator());
		view.add(fullScreen);

		playList = new JMenu(LanguageTool.getString("PLAYLIST"));
		playListPlay = new JMenuItem(LanguageTool.getString("PLAY"), ImageLoader.PLAY_MENU);
		playListOpenFolder = new JMenuItem(LanguageTool.getString("OPEN_FOLDER"), ImageLoader.FOLDER_ORANGE);
		playListTags = new JMenu(LanguageTool.getString("TAGS"));
		playListEditTag = new JMenuItem(LanguageTool.getString("EDIT_TAG"));
		playListAutoSetLyrics = new JMenuItem(LanguageTool.getString("AUTO_SET_LYRICS"));
		playListAutoSetTitle = new JMenuItem(LanguageTool.getString("AUTO_SET_TITLE"));
		playListAutoSetTrack = new JMenuItem(LanguageTool.getString("AUTO_SET_TRACK_NUMBER"));
		playListAutoSetGenre = new JMenuItem(LanguageTool.getString("AUTO_SET_GENRE"));
		playListTags.add(playListEditTag);
		playListTags.add(playListAutoSetLyrics);
		playListTags.add(playListAutoSetTitle);
		playListTags.add(playListAutoSetTrack);
		playListTags.add(playListAutoSetGenre);

		smartPlayList = new JMenu(LanguageTool.getString("SMART_PLAYLIST"));
		add10RandomSongs = new JMenuItem(LanguageTool.getString("ADD_10_RANDOM_SONGS"));
		add50RandomSongs = new JMenuItem(LanguageTool.getString("ADD_50_RANDOM_SONGS"));
		add100RandomSongs = new JMenuItem(LanguageTool.getString("ADD_100_RANDOM_SONGS"));
		smartPlayList.add(add10RandomSongs);
		smartPlayList.add(add50RandomSongs);
		smartPlayList.add(add100RandomSongs);
		smartPlayList.add(new JSeparator());
		add10SongsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_10_SONGS_MOST_PLAYED"));
		add50SongsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_50_SONGS_MOST_PLAYED"));
		add100SongsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_100_SONGS_MOST_PLAYED"));
		smartPlayList.add(add10SongsMostPlayed);
		smartPlayList.add(add50SongsMostPlayed);
		smartPlayList.add(add100SongsMostPlayed);
		smartPlayList.add(new JSeparator());
		add1AlbumsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_ALBUM_MOST_PLAYED"));
		add5AlbumsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_5_ALBUMS_MOST_PLAYED"));
		add10AlbumsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_10_ALBUMS_MOST_PLAYED"));
		smartPlayList.add(add1AlbumsMostPlayed);
		smartPlayList.add(add5AlbumsMostPlayed);
		smartPlayList.add(add10AlbumsMostPlayed);
		smartPlayList.add(new JSeparator());
		add1ArtistsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_ARTIST_MOST_PLAYED"));
		add5ArtistsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_5_ARTISTS_MOST_PLAYED"));
		add10ArtistsMostPlayed = new JMenuItem(LanguageTool.getString("ADD_10_ARTISTS_MOST_PLAYED"));
		smartPlayList.add(add1ArtistsMostPlayed);
		smartPlayList.add(add5ArtistsMostPlayed);
		smartPlayList.add(add10ArtistsMostPlayed);
		add10UnplayedSongs = new JMenuItem(LanguageTool.getString("ADD_10_UNPLAYED_SONGS"));
		add50UnplayedSongs = new JMenuItem(LanguageTool.getString("ADD_50_UNPLAYED_SONGS"));
		add100UnplayedSongs = new JMenuItem(LanguageTool.getString("ADD_100_UNPLAYED_SONGS"));
		smartPlayList.add(new JSeparator());
		smartPlayList.add(add10UnplayedSongs);
		smartPlayList.add(add50UnplayedSongs);
		smartPlayList.add(add100UnplayedSongs);

		playListSave = new JMenuItem(Actions.getAction(SavePlayListAction.class));
		playListLoad = new JMenuItem(Actions.getAction(LoadPlayListAction.class));
		playListFilter = new JMenuItem(Actions.getAction(TogglePlayListFilterAction.class));
		playListInfo = new JMenuItem(Actions.getAction(ShowPlayListItemInfoAction.class));
		playListDelete = new JMenuItem(Actions.getAction(RemoveFromPlayListAction.class));
		playListClear = new JMenuItem(Actions.getAction(ClearPlayListAction.class));
		playListTop = new JMenuItem(Actions.getAction(MoveToTopAction.class));
		playListUp = new JMenuItem(Actions.getAction(MoveUpAction.class));
		playListDown = new JMenuItem(Actions.getAction(MoveDownAction.class));
		playListBottom = new JMenuItem(Actions.getAction(MoveToBottomAction.class));
		playListShuffle = new JMenuItem(Actions.getAction(ShufflePlayListAction.class));
		playListFavoriteSong = new JMenuItem(Actions.getAction(SetPlayListSelectionAsFavoriteSongAction.class));
		playListFavoriteAlbum = new JMenuItem(Actions.getAction(SetPlayListSelectionAsFavoriteAlbumAction.class));
		playListFavoriteArtist = new JMenuItem(Actions.getAction(SetPlayListSelectionAsFavoriteArtistAction.class));
		playListArtist = new JMenuItem(Actions.getAction(CreatePlayListWithSelectedArtistsAction.class));
		playListAlbum = new JMenuItem(Actions.getAction(CreatePlayListWithSelectedAlbumsAction.class));

		playList.add(playListPlay);
		playList.add(playListInfo);
		playList.add(new JSeparator());
		playList.add(playListOpenFolder);
		playList.add(new JSeparator());
		playList.add(playListTags);
		playList.add(new JSeparator());
		playList.add(playListSave);
		playList.add(playListLoad);
		playList.add(new JSeparator());
		playList.add(playListFilter);
		playList.add(new JSeparator());
		playList.add(playListDelete);
		playList.add(playListClear);
		playList.add(new JSeparator());
		playList.add(playListTop);
		playList.add(playListUp);
		playList.add(playListDown);
		playList.add(playListBottom);
		playList.add(new JSeparator());
		playList.add(playListShuffle);
		playList.add(new JSeparator());
		playList.add(smartPlayList);
		playList.add(new JSeparator());
		playList.add(playListFavoriteSong);
		playList.add(playListFavoriteAlbum);
		playList.add(playListFavoriteArtist);
		playList.add(new JSeparator());
		playList.add(playListArtist);
		playList.add(playListAlbum);

		tools = new JMenu(LanguageTool.getString("TOOLS"));
		toolsExport = new JMenuItem(StringUtils.getString(LanguageTool.getString("EXPORT"), "..."), ImageLoader.EXPORT_FILE);
		ripCd = new JMenuItem(Actions.getAction(RipCDAction.class));
		stats = new JMenuItem(Actions.getAction(ShowStatsAction.class));
		coverNavigator = new JMenuItem(LanguageTool.getString("COVER_NAVIGATOR"), ImageLoader.CD_COVER);
		addRadio = new JMenuItem(LanguageTool.getString("ADD_RADIO"), ImageLoader.RADIO_ADD);
		showRadioBrowser = new JMenuItem(LanguageTool.getString("RADIO_BROWSER"), ImageLoader.RADIO_LITTLE);
		addPodcastFeed = new JMenuItem(LanguageTool.getString("ADD_PODCAST_FEED"), ImageLoader.RSS_ADD);
		customSearch = new JMenuItem(LanguageTool.getString("SEARCH"), ImageLoader.SEARCH);
		tools.add(toolsExport);
		tools.add(ripCd);
		tools.add(new JSeparator());
		tools.add(stats);
		tools.add(coverNavigator);
		tools.add(new JSeparator());
		tools.add(addRadio);
		tools.add(showRadioBrowser);
		tools.add(addPodcastFeed);
		tools.add(new JSeparator());
		tools.add(customSearch);

		device = new JMenu(LanguageTool.getString("DEVICE"));
		deviceConnect = new JMenuItem(LanguageTool.getString("CONNECT"), ImageLoader.DEVICE_CONNECT);
		device.add(deviceConnect);
		deviceRefresh = new JMenuItem(LanguageTool.getString("REFRESH"), ImageLoader.DEVICE_REFRESH);
		device.add(deviceRefresh);
		deviceDisconnect = new JMenuItem(LanguageTool.getString("DISCONNECT"), ImageLoader.DEVICE_DECONNECT);
		device.add(deviceDisconnect);

		about = new JMenu(LanguageTool.getString("ABOUT"));
		checkUpdates = new JMenuItem(LanguageTool.getString("CHECK_FOR_UPDATES"), ImageLoader.CHECK_FOR_UPDATES);
		reportBugOrRequestFeature = new JMenuItem(LanguageTool.getString("REPORT_BUG_OR_REQUEST_FEATURE"), ImageLoader.REPORT_BUG_OR_REQUEST_FEATURE);
		showLogFile = new JMenuItem(LanguageTool.getString("SHOW_LOG"), ImageLoader.SHOW_LOG_FILE);
		aboutItem = new JMenuItem(LanguageTool.getString("ABOUT"));

		about.add(checkUpdates);
		about.add(reportBugOrRequestFeature);
		about.add(showLogFile);
		about.add(new JSeparator());
		about.add(aboutItem);

		add(file);
		add(edit);
		add(view);
		add(playList);
		add(device);
		add(tools);
		add(about);
	}

	/**
	 * Gets the about item.
	 * 
	 * @return the about item
	 */
	public JMenuItem getAboutItem() {
		return aboutItem;
	}

	/**
	 * Gets the add100 random songs.
	 * 
	 * @return the add100 random songs
	 */
	public JMenuItem getAdd100RandomSongs() {
		return add100RandomSongs;
	}

	/**
	 * Gets the add100 songs most played.
	 * 
	 * @return the add100 songs most played
	 */
	public JMenuItem getAdd100SongsMostPlayed() {
		return add100SongsMostPlayed;
	}

	/**
	 * Gets the add10 albums most played.
	 * 
	 * @return the add10 albums most played
	 */
	public JMenuItem getAdd10AlbumsMostPlayed() {
		return add10AlbumsMostPlayed;
	}

	/**
	 * Gets the add10 artists most played.
	 * 
	 * @return the add10 artists most played
	 */
	public JMenuItem getAdd10ArtistsMostPlayed() {
		return add10ArtistsMostPlayed;
	}

	/**
	 * Gets the add10 random songs.
	 * 
	 * @return the add10 random songs
	 */
	public JMenuItem getAdd10RandomSongs() {
		return add10RandomSongs;
	}

	/**
	 * Gets the add10 songs most played.
	 * 
	 * @return the add10 songs most played
	 */
	public JMenuItem getAdd10SongsMostPlayed() {
		return add10SongsMostPlayed;
	}

	/**
	 * Gets the add1 albums most played.
	 * 
	 * @return the add1 albums most played
	 */
	public JMenuItem getAdd1AlbumsMostPlayed() {
		return add1AlbumsMostPlayed;
	}

	/**
	 * Gets the add1 artists most played.
	 * 
	 * @return the add1 artists most played
	 */
	public JMenuItem getAdd1ArtistsMostPlayed() {
		return add1ArtistsMostPlayed;
	}

	/**
	 * Gets the add50 random songs.
	 * 
	 * @return the add50 random songs
	 */
	public JMenuItem getAdd50RandomSongs() {
		return add50RandomSongs;
	}

	/**
	 * Gets the add50 songs most played.
	 * 
	 * @return the add50 songs most played
	 */
	public JMenuItem getAdd50SongsMostPlayed() {
		return add50SongsMostPlayed;
	}

	/**
	 * Gets the add5 albums most played.
	 * 
	 * @return the add5 albums most played
	 */
	public JMenuItem getAdd5AlbumsMostPlayed() {
		return add5AlbumsMostPlayed;
	}

	/**
	 * Gets the add5 artists most played.
	 * 
	 * @return the add5 artists most played
	 */
	public JMenuItem getAdd5ArtistsMostPlayed() {
		return add5ArtistsMostPlayed;
	}

	/**
	 * Gets the add podcast feed.
	 * 
	 * @return the addPodcastFeed
	 */
	public JMenuItem getAddPodcastFeed() {
		return addPodcastFeed;
	}

	/**
	 * Gets the add radio.
	 * 
	 * @return the addRadio
	 */
	public JMenuItem getAddRadio() {
		return addRadio;
	}

	/**
	 * Gets the check updates.
	 * 
	 * @return the check updates
	 */
	public JMenuItem getCheckUpdates() {
		return checkUpdates;
	}

	/**
	 * Gets the cover navigator.
	 * 
	 * @return the coverNavigator
	 */
	public JMenuItem getCoverNavigator() {
		return coverNavigator;
	}

	/**
	 * Gets the device connect.
	 * 
	 * @return the device connect
	 */
	public JMenuItem getDeviceConnect() {
		return deviceConnect;
	}

	/**
	 * Gets the device disconnect.
	 * 
	 * @return the device disconnect
	 */
	public JMenuItem getDeviceDisconnect() {
		return deviceDisconnect;
	}

	/**
	 * Gets the device refresh.
	 * 
	 * @return the device refresh
	 */
	public JMenuItem getDeviceRefresh() {
		return deviceRefresh;
	}

	/**
	 * Gets the full screen.
	 * 
	 * @return the fullScreen
	 */
	public JMenuItem getFullScreen() {
		return fullScreen;
	}

	/**
	 * Gets the menu as popup menu.
	 * 
	 * @return the menu as popup menu
	 */
	public JPopupMenu getMenuAsPopupMenu() {
		JPopupMenu menu = new JPopupMenu();
		menu.add(file);
		menu.add(edit);
		menu.add(view);
		menu.add(playList);
		menu.add(tools);
		menu.add(about);
		return menu;
	}

	/**
	 * Gets the mute.
	 * 
	 * @return the mute
	 */
	public JMenuItem getMute() {
		return mute;
	}

	/**
	 * Gets the play list auto set genre.
	 * 
	 * @return the play list auto set genre
	 */
	public JMenuItem getPlayListAutoSetGenre() {
		return playListAutoSetGenre;
	}

	/**
	 * Gets the play list auto set lyrics.
	 * 
	 * @return the play list auto set lyrics
	 */
	public JMenuItem getPlayListAutoSetLyrics() {
		return playListAutoSetLyrics;
	}

	/**
	 * Gets the play list auto set title.
	 * 
	 * @return the play list auto set title
	 */
	public JMenuItem getPlayListAutoSetTitle() {
		return playListAutoSetTitle;
	}

	/**
	 * Gets the play list auto set track.
	 * 
	 * @return the play list auto set track
	 */
	public JMenuItem getPlayListAutoSetTrack() {
		return playListAutoSetTrack;
	}

	/**
	 * Gets the play list edit tag.
	 * 
	 * @return the play list edit tag
	 */
	public JMenuItem getPlayListEditTag() {
		return playListEditTag;
	}

	/**
	 * Gets the play list play.
	 * 
	 * @return the play list play
	 */
	public JMenuItem getPlayListPlay() {
		return playListPlay;
	}

	/**
	 * Gets the play list show buttons.
	 * 
	 * @return the play list show buttons
	 */
	public JCheckBoxMenuItem getPlayListShowButtons() {
		return playListShowButtons;
	}

	/**
	 * Gets the report bug or request feature.
	 * 
	 * @return the reportBugOrRequestFeature
	 */
	public JMenuItem getReportBugOrRequestFeature() {
		return reportBugOrRequestFeature;
	}

	/**
	 * Gets the show navigation table.
	 * 
	 * @return the show navigation table
	 */
	public JCheckBoxMenuItem getShowNavigationTable() {
		return showNavigationTable;
	}

	/**
	 * Gets the show osd.
	 * 
	 * @return the show osd
	 */
	public JCheckBoxMenuItem getShowOSD() {
		return showOSD;
	}

	/**
	 * Gets the show status bar.
	 * 
	 * @return the show status bar
	 */
	public JCheckBoxMenuItem getShowStatusBar() {
		return showStatusBar;
	}

	/**
	 * Gets the tools export.
	 * 
	 * @return the tools export
	 */
	public JMenuItem getToolsExport() {
		return toolsExport;
	}

	/**
	 * Gets the add10 unplayed songs.
	 * 
	 * @return the add10 unplayed songs
	 */
	public JMenuItem getAdd10UnplayedSongs() {
		return add10UnplayedSongs;
	}

	/**
	 * Gets the add50 unplayed songs.
	 * 
	 * @return the add50 unplayed songs
	 */
	public JMenuItem getAdd50UnplayedSongs() {
		return add50UnplayedSongs;
	}

	/**
	 * Gets the add100 unplayed songs.
	 * 
	 * @return the add100 unplayed songs
	 */
	public JMenuItem getAdd100UnplayedSongs() {
		return add100UnplayedSongs;
	}

	/**
	 * Gets the show log file.
	 * 
	 * @return the show log file
	 */
	public JMenuItem getShowLogFile() {
		return showLogFile;
	}

	/**
	 * Gets the custom search.
	 * 
	 * @return the customSearch
	 */
	public JMenuItem getCustomSearch() {
		return customSearch;
	}

	/**
	 * Gets the play list open folder.
	 * 
	 * @return the playListOpenFolder
	 */
	public JMenuItem getPlayListOpenFolder() {
		return playListOpenFolder;
	}

	/**
	 * Gets the show radio browser.
	 * 
	 * @return the show radio browser
	 */
	public JMenuItem getShowRadioBrowser() {
		return showRadioBrowser;
	}

}
