/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.views.panels;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.TableColumn;

import net.sourceforge.atunes.Constants;
import net.sourceforge.atunes.gui.Fonts;
import net.sourceforge.atunes.gui.GuiUtils;
import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.substance.SubstanceContextInformationImageJTable;
import net.sourceforge.atunes.gui.views.Renderers;
import net.sourceforge.atunes.gui.views.controls.PopUpButton;
import net.sourceforge.atunes.gui.views.controls.UrlLabel;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.modules.contextInformation.AlbumInfo;
import net.sourceforge.atunes.kernel.modules.contextInformation.ArtistInfo;
import net.sourceforge.atunes.utils.LanguageTool;

/**
 * The Class ContextInformationPanel.
 */
public class ContextInformationPanel extends JPanel {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 707242790413122482L;

	/** The Constant PREF_SIZE. */
	public static final Dimension PREF_SIZE = new Dimension(230, 0);

	/** The Constant MINIMUM_SIZE. */
	public static final Dimension MINIMUM_SIZE = new Dimension(170, 0);

	/** The tabbed pane. */
	private JTabbedPane tabbedPane;

	/** The album tab panel. */
	private JPanel albumTabPanel;

	/** The artist tab panel. */
	private JPanel artistTabPanel;

	/** The similar tab panel. */
	private JPanel similarTabPanel;

	/** The lyrics panel. */
	private JPanel lyricsPanel;

	/** The album panel. */
	private JPanel albumPanel;

	/** The artist label. */
	private UrlLabel artistLabel;

	/** The album label. */
	private UrlLabel albumLabel;

	/** The year label. */
	private UrlLabel yearLabel;

	/** The album cover label. */
	private JLabel albumCoverLabel;

	/** The tracks table. */
	private JTable tracksTable;

	/** The lyrics cover. */
	private JLabel lyricsCover;

	/** The lyrics label. */
	private JLabel lyricsLabel;

	/** The lyrics artist label. */
	private JLabel lyricsArtistLabel;

	/** The lyrics last play date label. */
	private JLabel lyricsLastPlayDateLabel;

	/** The lyrics container. */
	private JTextArea lyricsContainer;

	/** The lyric scroll pane. */
	private JScrollPane lyricScrollPane;

	/** The copy lyric to clipboard. */
	private JButton copyLyricToClipboard;

	/** The search for lyrics source. */
	private JButton openLyricsSource;

	private PopUpButton addLyrics;

	/** The search for video. */
	private JButton searchForVideo;

	/** The albums panel. */
	private JPanel albumsPanel;

	/** The albums scroll pane. */
	private JScrollPane albumsScrollPane;

	/** The artist image label. */
	private JLabel artistImageLabel;

	/** The artist albums label. */
	private UrlLabel artistAlbumsLabel;

	/** The albums table. */
	private JTable albumsTable;

	/** The artist wiki abstract. */
	private JTextArea artistWikiAbstract;

	/** The artist wiki read more. */
	private UrlLabel artistWikiReadMore;

	/** The similar artists panel. */
	private JPanel similarArtistsPanel;

	/** The similar artists table. */
	private JTable similarArtistsTable;

	/** The similar artists scroll pane. */
	private JScrollPane similarArtistsScrollPane;

	/**
	 * The text of the first tab
	 */
	private String firstTabTitle;

	/**
	 * Instantiates a new audio scrobbler panel.
	 */
	public ContextInformationPanel() {
		super(new GridBagLayout());
		setPreferredSize(PREF_SIZE);
		setMinimumSize(MINIMUM_SIZE);
		setContent();
	}

	/**
	 * Adds the album panel.
	 */
	private void addAlbumPanel() {
		albumPanel = new JPanel(new GridBagLayout());

		artistLabel = new UrlLabel();
		artistLabel.setHorizontalAlignment(SwingConstants.CENTER);
		albumLabel = new UrlLabel();
		albumLabel.setHorizontalAlignment(SwingConstants.CENTER);
		albumLabel.setFont(Fonts.CONTEXT_INFORMATION_BIG_FONT);
		albumCoverLabel = new JLabel();
		albumCoverLabel.setBorder(BorderFactory.createLineBorder(GuiUtils.getBorderColor()));
		albumCoverLabel.setVisible(false);
		yearLabel = new UrlLabel();
		yearLabel.setHorizontalAlignment(SwingConstants.CENTER);

		GridBagConstraints c = new GridBagConstraints();

		c.gridx = 0;
		c.gridy = 0;
		c.insets = new Insets(15, 0, 0, 0);
		albumPanel.add(albumCoverLabel, c);

		c.gridy = 1;
		c.insets = new Insets(5, 5, 0, 5);
		albumPanel.add(albumLabel, c);

		c.gridy = 2;
		albumPanel.add(artistLabel, c);

		c.gridy = 3;
		albumPanel.add(yearLabel, c);

		tracksTable = new JTable();
		tracksTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		tracksTable.setShowGrid(false);
		tracksTable.setDefaultRenderer(String.class, Renderers.CONTEXT_INFORMATION_TRACK_RENDERER);
		tracksTable.setDefaultRenderer(Integer.class, Renderers.CONTEXT_INFORMATION_TRACK_NUMBER_RENDERER);
		tracksTable.getTableHeader().setReorderingAllowed(true);
		tracksTable.getTableHeader().setResizingAllowed(false);
		tracksTable.setColumnModel(new DefaultTableColumnModel() {
			private static final long serialVersionUID = 1338172152164826400L;

			@Override
			public void addColumn(TableColumn column) {
				super.addColumn(column);
				if (column.getModelIndex() == 0) {
					column.setMaxWidth(25);
				}
			}
		});
		c.gridx = 0;
		c.gridy = 4;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(20, 5, 5, 5);
		JScrollPane scroll = new JScrollPane(tracksTable);
		//scroll.setBorder(BorderFactory.createEmptyBorder());
		albumPanel.add(scroll, c);

		c.gridx = 0;
		c.gridy = 1;
		c.insets = new Insets(0, 0, 0, 0);
		albumTabPanel.add(albumPanel, c);

		albumPanel.setVisible(false);
	}

	/**
	 * Adds the albums panel.
	 */
	private void addAlbumsPanel() {
		albumsPanel = new JPanel(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		artistImageLabel = new JLabel();
		artistImageLabel.setBorder(BorderFactory.createLineBorder(GuiUtils.getBorderColor()));
		artistImageLabel.setVisible(false);
		artistAlbumsLabel = new UrlLabel();
		artistAlbumsLabel.setFont(Fonts.CONTEXT_INFORMATION_BIG_FONT);
		artistAlbumsLabel.setVisible(false);
		c.gridx = 0;
		c.gridy = 0;
		c.insets = new Insets(15, 5, 0, 5);
		albumsPanel.add(artistImageLabel, c);
		c.gridx = 0;
		c.gridy = 1;
		c.insets = new Insets(5, 5, 5, 5);
		albumsPanel.add(artistAlbumsLabel, c);

		artistWikiAbstract = new JTextArea();
		artistWikiAbstract.setLineWrap(true);
		artistWikiAbstract.setWrapStyleWord(true);
		artistWikiAbstract.setEditable(false);
		artistWikiAbstract.setBorder(BorderFactory.createEmptyBorder());
		JScrollPane artistWikiScrollPane = new JScrollPane(artistWikiAbstract);
		//artistWikiScrollPane.setBorder(BorderFactory.createEmptyBorder());
		c.gridy = 2;
		c.weightx = 1;
		c.weighty = 0.3;
		c.fill = GridBagConstraints.BOTH;
		albumsPanel.add(artistWikiScrollPane, c);

		artistWikiReadMore = new UrlLabel();
		c.gridy = 3;
		c.weighty = 0;
		albumsPanel.add(artistWikiReadMore, c);

		albumsTable = new SubstanceContextInformationImageJTable();
		albumsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		albumsTable.setShowGrid(false);
		albumsTable.getTableHeader().setReorderingAllowed(false);
		albumsTable.getTableHeader().setResizingAllowed(false);
		albumsTable.setDefaultRenderer(AlbumInfo.class, Renderers.CONTEXT_INFORMATION_ALBUMS_RENDERER);
		albumsTable.setRowHeight(Constants.CONTEXT_INFORMATION_IMAGE_HEIGHT);
		albumsScrollPane = new JScrollPane(albumsTable);
		albumsScrollPane.setAutoscrolls(false);
		albumsScrollPane.setOpaque(false);
		//albumsScrollPane.setBorder(BorderFactory.createEmptyBorder());

		c.gridy = 4;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(0, 5, 0, 5);
		albumsPanel.add(albumsScrollPane, c);
		c.gridy = 5;
		c.weighty = 1;
		c.insets = new Insets(0, 0, 0, 0);
		artistTabPanel.add(albumsPanel, c);

		albumsPanel.setVisible(false);
	}

	/**
	 * Adds the lyrics panel.
	 */
	private void addLyricsPanel() {
		lyricsCover = new JLabel();
		lyricsCover.setVisible(false);
		//	lyricsCover.setBorder(BorderFactory.createLineBorder(GuiUtils.getBorderColor()));
		lyricsLabel = new JLabel();
		lyricsLabel.setHorizontalAlignment(SwingConstants.CENTER);
		lyricsLabel.setFont(Fonts.CONTEXT_INFORMATION_BIG_FONT);
		lyricsArtistLabel = new JLabel();
		lyricsArtistLabel.setHorizontalAlignment(SwingConstants.CENTER);
		lyricsLastPlayDateLabel = new JLabel();
		lyricsLastPlayDateLabel.setHorizontalAlignment(SwingConstants.CENTER);
		lyricsContainer = new JTextArea();
		lyricsContainer.setBorder(BorderFactory.createEmptyBorder());
		lyricsContainer.setEditable(false);
		lyricsContainer.setWrapStyleWord(true);
		lyricScrollPane = new JScrollPane(lyricsContainer);
		lyricScrollPane.setAutoscrolls(false);
		lyricScrollPane.setOpaque(false);
		//lyricScrollPane.setBorder(BorderFactory.createEmptyBorder());
		copyLyricToClipboard = new JButton(LanguageTool.getString("COPY_TO_CLIPBOARD"));
		copyLyricToClipboard.setToolTipText(LanguageTool.getString("COPY_TO_CLIPBOARD"));
		openLyricsSource = new JButton(LanguageTool.getString("OPEN_LYRICS_SOURCE"));
		openLyricsSource.setToolTipText(LanguageTool.getString("OPEN_LYRICS_SOURCE"));
		addLyrics = new PopUpButton(LanguageTool.getString("ADD_LYRICS"), PopUpButton.TOP_RIGHT);
		addLyrics.setToolTipText(LanguageTool.getString("ADD_LYRICS"));
		searchForVideo = new JButton(LanguageTool.getString("SEARCH_FOR_VIDEO"));
		searchForVideo.setToolTipText(LanguageTool.getString("SEARCH_FOR_VIDEO"));
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.insets = new Insets(15, 0, 0, 0);
		lyricsPanel.add(lyricsCover, c);
		c.gridx = 0;
		c.gridy = 1;
		c.weightx = 1;
		c.weighty = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(5, 10, 0, 10);
		lyricsPanel.add(lyricsLabel, c);
		c.gridy = 2;
		c.insets = new Insets(5, 10, 10, 10);
		lyricsPanel.add(lyricsArtistLabel, c);
		c.gridy = 3;
		lyricsPanel.add(lyricsLastPlayDateLabel, c);
		c.gridy = 4;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(10, 5, 5, 5);
		lyricsPanel.add(lyricScrollPane, c);
		c.gridy = 5;
		c.weighty = 0;
		c.weightx = 0;
		c.fill = GridBagConstraints.NONE;
		JPanel auxPanel = new JPanel(new GridLayout(2, 2, 10, 10));
		auxPanel.add(addLyrics);
		auxPanel.add(copyLyricToClipboard);
		auxPanel.add(openLyricsSource);
		auxPanel.add(searchForVideo);
		auxPanel.add(addLyrics);
		lyricsPanel.add(auxPanel, c);
	}

	/**
	 * Adds the similar artists panel.
	 */
	private void addSimilarArtistsPanel() {
		similarArtistsPanel = new JPanel(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		similarArtistsTable = new SubstanceContextInformationImageJTable();
		similarArtistsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		similarArtistsTable.setShowGrid(false);
		similarArtistsTable.setDefaultRenderer(ArtistInfo.class, Renderers.CONTEXT_INFORMATION_ARTISTS_RENDERER);
		similarArtistsTable.setRowHeight(Constants.CONTEXT_INFORMATION_IMAGE_HEIGHT);
		similarArtistsTable.setColumnSelectionAllowed(false);
		similarArtistsTable.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
		similarArtistsTable.setColumnModel(new DefaultTableColumnModel() {
			private static final long serialVersionUID = 1338172152164826400L;

			@Override
			public void addColumn(TableColumn column) {
				super.addColumn(column);
				if (column.getModelIndex() == 0) {
					column.setPreferredWidth(ContextInformationPanel.this.getWidth() - 50);
				} else {
					column.setResizable(false);
				}
			}
		});
		similarArtistsScrollPane = new JScrollPane(similarArtistsTable);
		similarArtistsScrollPane.setAutoscrolls(false);
		similarArtistsScrollPane.setOpaque(false);
		//similarArtistsScrollPane.setBorder(BorderFactory.createEmptyBorder());
		similarArtistsPanel.add(similarArtistsScrollPane, c);

		c.gridx = 0;
		c.gridy = 0;
		c.weighty = 0.3;
		similarTabPanel.add(similarArtistsPanel, c);

		similarArtistsPanel.setVisible(false);
	}

	/**
	 * Gets the album cover label.
	 * 
	 * @return the album cover label
	 */
	public JLabel getAlbumCoverLabel() {
		return albumCoverLabel;
	}

	/**
	 * Gets the album label.
	 * 
	 * @return the album label
	 */
	public UrlLabel getAlbumLabel() {
		return albumLabel;
	}

	/**
	 * Gets the album panel.
	 * 
	 * @return the album panel
	 */
	public JPanel getAlbumPanel() {
		return albumPanel;
	}

	/**
	 * Gets the albums panel.
	 * 
	 * @return the albums panel
	 */
	public JPanel getAlbumsPanel() {
		return albumsPanel;
	}

	/**
	 * Gets the albums scroll pane.
	 * 
	 * @return the albums scroll pane
	 */
	public JScrollPane getAlbumsScrollPane() {
		return albumsScrollPane;
	}

	/**
	 * Gets the albums table.
	 * 
	 * @return the albums table
	 */
	public JTable getAlbumsTable() {
		return albumsTable;
	}

	/**
	 * Gets the artist albums label.
	 * 
	 * @return the artist albums label
	 */
	public UrlLabel getArtistAlbumsLabel() {
		return artistAlbumsLabel;
	}

	/**
	 * Gets the artist image label.
	 * 
	 * @return the artist image label
	 */
	public JLabel getArtistImageLabel() {
		return artistImageLabel;
	}

	/**
	 * Gets the artist label.
	 * 
	 * @return the artist label
	 */
	public UrlLabel getArtistLabel() {
		return artistLabel;
	}

	/**
	 * Gets the artist wiki abstract.
	 * 
	 * @return the artistWikiAbstract
	 */
	public JTextArea getArtistWikiAbstract() {
		return artistWikiAbstract;
	}

	/**
	 * Gets the artist wiki read more.
	 * 
	 * @return the artistWikiReadMore
	 */
	public UrlLabel getArtistWikiReadMore() {
		return artistWikiReadMore;
	}

	/**
	 * Gets the copy lyric to clipboard.
	 * 
	 * @return the copy lyric to clipboard
	 */
	public JButton getCopyLyricToClipboard() {
		return copyLyricToClipboard;
	}

	/**
	 * Gets the lyrics artist label.
	 * 
	 * @return the lyrics artist label
	 */
	public JLabel getLyricsArtistLabel() {
		return lyricsArtistLabel;
	}

	/**
	 * Gets the lyrics container.
	 * 
	 * @return the lyrics container
	 */
	public JTextArea getLyricsContainer() {
		return lyricsContainer;
	}

	/**
	 * Gets the lyrics cover.
	 * 
	 * @return the lyrics cover
	 */
	public JLabel getLyricsCover() {
		return lyricsCover;
	}

	/**
	 * Gets the lyric scroll pane.
	 * 
	 * @return the lyric scroll pane
	 */
	public JScrollPane getLyricScrollPane() {
		return lyricScrollPane;
	}

	/**
	 * Gets the lyrics label.
	 * 
	 * @return the lyrics label
	 */
	public JLabel getLyricsLabel() {
		return lyricsLabel;
	}

	/**
	 * Gets the lyrics last play date label.
	 * 
	 * @return the lyrics last play date label
	 */
	public JLabel getLyricsLastPlayDateLabel() {
		return lyricsLastPlayDateLabel;
	}

	/**
	 * Gets the search for video.
	 * 
	 * @return the searchForVideo
	 */
	public JButton getSearchForVideo() {
		return searchForVideo;
	}

	/**
	 * Gets the similar artists panel.
	 * 
	 * @return the similar artists panel
	 */
	public JPanel getSimilarArtistsPanel() {
		return similarArtistsPanel;
	}

	/**
	 * Gets the similar artists scroll pane.
	 * 
	 * @return the similar artists scroll pane
	 */
	public JScrollPane getSimilarArtistsScrollPane() {
		return similarArtistsScrollPane;
	}

	/**
	 * Gets the similar artists table.
	 * 
	 * @return the similar artists table
	 */
	public JTable getSimilarArtistsTable() {
		return similarArtistsTable;
	}

	/**
	 * Gets the similar tab panel.
	 * 
	 * @return the similar tab panel
	 */
	public JPanel getSimilarTabPanel() {
		return similarTabPanel;
	}

	/**
	 * Gets the tabbed pane.
	 * 
	 * @return the tabbed pane
	 */
	public JTabbedPane getTabbedPane() {
		return tabbedPane;
	}

	/**
	 * Gets the tracks table.
	 * 
	 * @return the tracks table
	 */
	public JTable getTracksTable() {
		return tracksTable;
	}

	/**
	 * Gets the year label.
	 * 
	 * @return the year label
	 */
	public UrlLabel getYearLabel() {
		return yearLabel;
	}

	public JButton getOpenLyricsSource() {
		return openLyricsSource;
	}

	public PopUpButton getAddLyrics() {
		return addLyrics;
	}

	/**
	 * Sets the content.
	 */
	private void setContent() {
		tabbedPane = new JTabbedPane();
		albumTabPanel = new JPanel(new GridBagLayout());
		artistTabPanel = new JPanel(new GridBagLayout());
		similarTabPanel = new JPanel(new GridBagLayout());
		lyricsPanel = new JPanel(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		c.insets = new Insets(3, 2, 0, 2);
		add(tabbedPane, c);

		addAlbumPanel();
		addAlbumsPanel();
		addSimilarArtistsPanel();
		addLyricsPanel();

		// Add tabs
		showAllTabs(true);
	}

	/**
	 * Shows all tabs in tabbed pane with given first tab title and icon
	 * 
	 * @param enabled
	 */
	public void showAllTabs(String tabTitle, Icon firstTabIcon, boolean enabled) {
		firstTabTitle = tabTitle;
		tabbedPane.addTab(null, firstTabIcon, lyricsPanel);
		tabbedPane.addTab(null, ImageLoader.ALBUM, albumTabPanel);
		tabbedPane.addTab(null, ImageLoader.ARTIST, artistTabPanel);
		tabbedPane.addTab(null, ImageLoader.ARTIST_SIMILAR, similarTabPanel);
		setContextTabsText(Kernel.getInstance().state.isShowContextTabsText());
		tabbedPane.setEnabledAt(0, enabled);
		tabbedPane.setEnabledAt(1, enabled);
		tabbedPane.setEnabledAt(2, enabled);
		tabbedPane.setEnabledAt(3, enabled);
	}

	/**
	 * Show all tabs in tabbed pane for AudioFile objects
	 * 
	 * @param enabled
	 */
	public void showAllTabs(boolean enabled) {
		showAllTabs(LanguageTool.getString("SONG"), ImageLoader.AUDIO_FILE, enabled);
	}

	/**
	 * Shows only first tab in tabbed pane with given tab title
	 * 
	 * @param tabTitle
	 */
	public void showFirstTab(String tabTitle, Icon icon) {
		tabbedPane.removeAll();
		firstTabTitle = tabTitle;
		tabbedPane.addTab(tabTitle, icon, lyricsPanel);
	}

	/**
	 * Method to show (<code>true</code>) or not (<code>false</code>) context
	 * tabs text
	 * 
	 * @param set
	 */
	public void setContextTabsText(boolean set) {
		tabbedPane.setTitleAt(0, set ? firstTabTitle : null);
		tabbedPane.setTitleAt(1, set ? LanguageTool.getString("ALBUM") : null);
		tabbedPane.setTitleAt(2, set ? LanguageTool.getString("ARTIST") : null);
		tabbedPane.setTitleAt(3, set ? LanguageTool.getString("SIMILAR") : null);
		tabbedPane.setToolTipTextAt(0, firstTabTitle);
		tabbedPane.setToolTipTextAt(1, LanguageTool.getString("ALBUM"));
		tabbedPane.setToolTipTextAt(2, LanguageTool.getString("ARTIST"));
		tabbedPane.setToolTipTextAt(3, LanguageTool.getString("SIMILAR"));
	}

}
