/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.views.panels;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.ToolTipManager;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

import net.sourceforge.atunes.gui.GuiUtils;
import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.model.NavigationTableModel.Property;
import net.sourceforge.atunes.gui.views.Renderers;
import net.sourceforge.atunes.gui.views.controls.CustomButton;
import net.sourceforge.atunes.gui.views.controls.DragSourceTable;
import net.sourceforge.atunes.gui.views.controls.DragSourceTree;
import net.sourceforge.atunes.gui.views.controls.PopUpButton;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.TimeUtils;

import org.jvnet.substance.api.renderers.SubstanceDefaultTableCellRenderer;

/**
 * The Class NavigationPanel.
 * 
 * @author fleax
 */
public class NavigationPanel extends JPanel {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -2900418193013495812L;

	/** The repository tree. */
	private JTree repositoryTree;

	/** The repository tree scroll pane. */
	private JScrollPane repositoryTreeScrollPane;

	/** The navigation table. */
	private DragSourceTable navigationTable;

	/** The navigation table button panel. */
	private JPanel navigationTableButtonPanel;

	/** The repository tree model. */
	private DefaultTreeModel repositoryTreeModel;

	/** The navigation table info button. */
	private CustomButton navigationTableInfoButton;

	/** The sort by track. */
	private JRadioButtonMenuItem sortByTrack;

	/** The sort by file. */
	private JRadioButtonMenuItem sortByFile;

	/** The sort by title. */
	private JRadioButtonMenuItem sortByTitle;

	/** The sort by modification time. */
	private JRadioButtonMenuItem sortByModificationTime;

	/** The navigation table container. */
	private JPanel navigationTableContainer;

	/** The favorites tree. */
	private JTree favoritesTree;

	/** The favorites tree model. */
	private DefaultTreeModel favoritesTreeModel;

	/** The tabbed pane. */
	private JTabbedPane tabbedPane;

	/** The device tree. */
	private JTree deviceTree;

	/** The device tree scroll pane */
	private JScrollPane deviceTreeScrollPane;

	/** The device tree model. */
	private DefaultTreeModel deviceTreeModel;

	/** The radio tree. */
	private JTree radioTree;

	/** The radio tree model. */
	private DefaultTreeModel radioTreeModel;

	/** The podcast feed tree. */
	private JTree podcastFeedTree;

	/** The podcast feed tree model. */
	private DefaultTreeModel podcastFeedTreeModel;

	/** The split pane. */
	private JSplitPane splitPane;

	/** The options button. */
	private PopUpButton optionsButton;

	/** The show artist. */
	private JRadioButtonMenuItem showArtist;

	/** The show album. */
	private JRadioButtonMenuItem showAlbum;

	/** The show genre. */
	private JRadioButtonMenuItem showGenre;

	/** The show folder. */
	private JRadioButtonMenuItem showFolder;

	/** The expand tree. */
	private JMenuItem expandTree;

	/** The collapse tree. */
	private JMenuItem collapseTree;

	/** The filter label. */
	private JLabel filterLabel;

	/** The filter text field. */
	private JTextField filterTextField;

	/** The clear filter button. */
	private CustomButton clearFilterButton;

	/** The favorite menu. */
	private JPopupMenu favoriteMenu;

	/** The add to playlist menu item. */
	private JMenuItem addToPlaylistMenuItem;

	/** The set as playlist menu item. */
	private JMenuItem setAsPlaylistMenuItem;

	/** The play now menu item. */
	private JMenuItem playNowMenuItem;

	/** The remove from favorites menu item. */
	private JMenuItem removeFromFavoritesMenuItem;

	/** The non favorite menu. */
	private JPopupMenu nonFavoriteMenu;

	/** The non favorite add to playlist menu item. */
	private JMenuItem nonFavoriteAddToPlaylistMenuItem;

	/** The non favorite set as playlist menu item. */
	private JMenuItem nonFavoriteSetAsPlaylistMenuItem;

	/** The non favorite play now menu item. */
	private JMenuItem nonFavoritePlayNowMenuItem;

	/** The non favorite open folder item. */
	private JMenuItem nonFavoriteOpenFolderMenuItem;

	/** The non favorite mark podcast entry as listened. */
	private JMenuItem nonFavoriteMarkPodcastEntryAsListened;

	/** The non favorite podcast feed entry download. */
	private JMenuItem nonFavoritePodcastFeedEntryDownload;

	/** The non favorite remove old podcast feed entry. */
	private JMenuItem nonFavoriteRemoveOldPodcastFeedEntry;

	/** The non favorite set as favorite song menu item. */
	private JMenuItem nonFavoriteSetAsFavoriteSongMenuItem;

	/** The non favorite set as favorite album menu item. */
	private JMenuItem nonFavoriteSetAsFavoriteAlbumMenuItem;

	/** The non favorite set as favorite artist menu item. */
	private JMenuItem nonFavoriteSetAsFavoriteArtistMenuItem;

	/** The non favorite edit tag menu item. */
	private JMenuItem nonFavoriteEditTagMenuItem;

	/** The non favorite clear tag menu item. */
	private JMenuItem nonFavoriteClearTagMenuItem;

	/** The non favorite extract picture menu item. */
	private JMenuItem nonFavoriteExtractPictureMenuItem;

	/** The non favorite edit titles menu item. */
	private JMenuItem nonFavoriteEditTitlesMenuItem;

	/** The non favorite search. */
	private JMenuItem nonFavoriteSearch;

	/** The non favorite search at. */
	private JMenuItem nonFavoriteSearchAt;

	/** The non favorite remove physically menu item. */
	private JMenuItem nonFavoriteRemovePhysicallyMenuItem;

	/** The non favorite copy to device menu item. */
	private JMenuItem nonFavoriteCopyToDeviceMenuItem;

	/** The device menu. */
	private JPopupMenu deviceMenu;

	/** The device copy to repository menu item. */
	private JMenuItem deviceCopyToRepositoryMenuItem;

	/** The device fill menu item. */
	private JMenuItem deviceFillMenuItem;

	/** The radio menu. */
	private JPopupMenu radioMenu;

	/** The radio add radio menu item. */
	private JMenuItem radioAddRadioMenuItem;

	/** The radio refresh menu item. */
	private JMenuItem radioRefreshMenuItem;

	/** The radio add favorite menu item. */
	private JMenuItem radioAddFavoriteMenuItem;

	/** The radio rename radio menu item. */
	private JMenuItem radioRenameRadioMenuItem;

	/** The radio rename label menu item. */
	private JMenuItem radioRenameLabelMenuItem;

	/** The radio remove radio menu item. */
	private JMenuItem radioRemoveRadioMenuItem;

	/** The podcast feed menu. */
	private JPopupMenu podcastFeedMenu;

	/** The podcast feed add podcast feed menu item. */
	private JMenuItem podcastFeedAddPodcastFeedMenuItem;

	/** The podcast feed rename podcast feed menu item. */
	private JMenuItem podcastFeedRenamePodcastFeedMenuItem;

	/** The podcast feed remove podcast feed menu item. */
	private JMenuItem podcastFeedRemovePodcastFeedMenuItem;

	/** The podcast feed mark as listened menu item. */
	private JMenuItem podcastFeedMarkAsListenedMenuItem;

	/**
	 * Instantiates a new navigation panel.
	 */
	public NavigationPanel() {
		super(new BorderLayout(), true);
		addContent();
	}

	/**
	 * Adds the content.
	 */
	private void addContent() {
		JPanel treePanel = new JPanel(new BorderLayout());
		repositoryTreeModel = new DefaultTreeModel(new DefaultMutableTreeNode(LanguageTool.getString("REPOSITORY")));
		repositoryTree = new DragSourceTree(repositoryTreeModel);
		repositoryTree.setToggleClickCount(0);
		repositoryTree.setCellRenderer(Renderers.REPOSITORY_TREE_RENDERER);
		ToolTipManager.sharedInstance().registerComponent(repositoryTree);
		repositoryTreeScrollPane = new JScrollPane(repositoryTree);

		favoritesTreeModel = new DefaultTreeModel(new DefaultMutableTreeNode(LanguageTool.getString("FAVORITES")));
		favoritesTree = new DragSourceTree(favoritesTreeModel);
		favoritesTree.setToggleClickCount(0);
		favoritesTree.setCellRenderer(Renderers.FAVORITE_TREE_RENDERER);
		JScrollPane scrollPane4 = new JScrollPane(favoritesTree);

		deviceTreeModel = new DefaultTreeModel(new DefaultMutableTreeNode(LanguageTool.getString("DEVICE")));
		deviceTree = new DragSourceTree(deviceTreeModel);
		deviceTree.setToggleClickCount(0);
		deviceTree.setCellRenderer(Renderers.DEVICE_TREE_RENDERER);
		deviceTreeScrollPane = new JScrollPane(deviceTree);

		radioTreeModel = new DefaultTreeModel(new DefaultMutableTreeNode(LanguageTool.getString("RADIO")));
		radioTree = new DragSourceTree(radioTreeModel);
		radioTree.setToggleClickCount(0);
		radioTree.setCellRenderer(Renderers.RADIO_TREE_RENDERER);
		radioTree.setToolTipText(LanguageTool.getString("RADIO_VIEW_TOOLTIP"));
		JScrollPane scrollPane6 = new JScrollPane(radioTree);

		podcastFeedTreeModel = new DefaultTreeModel(new DefaultMutableTreeNode(LanguageTool.getString("PODCAST_FEEDS")));
		podcastFeedTree = new DragSourceTree(podcastFeedTreeModel);
		podcastFeedTree.setToggleClickCount(0);
		podcastFeedTree.setCellRenderer(Renderers.PODCAST_FEED_TREE_RENDERER);
		podcastFeedTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		JScrollPane scrollPane7 = new JScrollPane(podcastFeedTree);

		favoriteMenu = new JPopupMenu();
		addToPlaylistMenuItem = new JMenuItem(LanguageTool.getString("ADD_TO_PLAYLIST"), ImageLoader.ADD);
		setAsPlaylistMenuItem = new JMenuItem(LanguageTool.getString("SET_AS_PLAYLIST"), ImageLoader.PLAYLIST);
		playNowMenuItem = new JMenuItem(LanguageTool.getString("PLAY_NOW"), ImageLoader.PLAY_MENU);
		removeFromFavoritesMenuItem = new JMenuItem(LanguageTool.getString("REMOVE_FROM_FAVORITES"), ImageLoader.DELETE_TAG);
		favoriteMenu.add(addToPlaylistMenuItem);
		favoriteMenu.add(setAsPlaylistMenuItem);
		favoriteMenu.add(playNowMenuItem);
		favoriteMenu.add(removeFromFavoritesMenuItem);

		nonFavoriteMenu = new JPopupMenu();
		nonFavoriteAddToPlaylistMenuItem = new JMenuItem(LanguageTool.getString("ADD_TO_PLAYLIST"), ImageLoader.ADD);
		nonFavoriteSetAsPlaylistMenuItem = new JMenuItem(LanguageTool.getString("SET_AS_PLAYLIST"), ImageLoader.PLAYLIST);
		nonFavoritePlayNowMenuItem = new JMenuItem(LanguageTool.getString("PLAY_NOW"), ImageLoader.PLAY_MENU);
		nonFavoriteOpenFolderMenuItem = new JMenuItem(LanguageTool.getString("OPEN_FOLDER"), ImageLoader.FOLDER_ORANGE);
		nonFavoriteMarkPodcastEntryAsListened = new JMenuItem(LanguageTool.getString("MARK_PODCAST_ENTRY_AS_LISTENED"), ImageLoader.MARK_RSS_AS_READ);
		nonFavoritePodcastFeedEntryDownload = new JMenuItem(LanguageTool.getString("DOWNLOAD_PODCAST_ENTRY"), ImageLoader.DOWNLOAD_PODCAST);
		nonFavoriteRemoveOldPodcastFeedEntry = new JMenuItem(LanguageTool.getString("REMOVE_OLD_PODCAST_ENTRY"), ImageLoader.REMOVE);
		nonFavoriteEditTagMenuItem = new JMenuItem(LanguageTool.getString("EDIT_TAG"), ImageLoader.TAG);
		nonFavoriteEditTitlesMenuItem = new JMenuItem(LanguageTool.getString("EDIT_TITLES"), ImageLoader.EDIT_ALBUM);
		nonFavoriteClearTagMenuItem = new JMenuItem(LanguageTool.getString("CLEAR_TAG"), ImageLoader.DELETE_TAG);
		nonFavoriteExtractPictureMenuItem = new JMenuItem(LanguageTool.getString("EXTRACT_PICTURE"), ImageLoader.EXPORT_PICTURE);
		nonFavoriteSetAsFavoriteSongMenuItem = new JMenuItem(LanguageTool.getString("SET_FAVORITE_SONG"), ImageLoader.FAVORITE);
		nonFavoriteSetAsFavoriteAlbumMenuItem = new JMenuItem(LanguageTool.getString("SET_FAVORITE_ALBUM"), ImageLoader.FAVORITE);
		nonFavoriteSetAsFavoriteArtistMenuItem = new JMenuItem(LanguageTool.getString("SET_FAVORITE_ARTIST"), ImageLoader.FAVORITE);
		nonFavoriteSearch = new JMenuItem(LanguageTool.getString("SEARCH_ARTIST"), ImageLoader.SEARCH_ARTIST);
		nonFavoriteSearchAt = new JMenuItem(StringUtils.getString(LanguageTool.getString("SEARCH_ARTIST_AT"), "..."), ImageLoader.SEARCH_AT);
		nonFavoriteRemovePhysicallyMenuItem = new JMenuItem(LanguageTool.getString("REMOVE_FROM_DISK"), ImageLoader.DELETE_FILE);
		nonFavoriteCopyToDeviceMenuItem = new JMenuItem(LanguageTool.getString("COPY_TO_DEVICE"), ImageLoader.DEVICE);

		nonFavoriteMenu.add(nonFavoriteAddToPlaylistMenuItem);
		nonFavoriteMenu.add(nonFavoriteSetAsPlaylistMenuItem);
		nonFavoriteMenu.add(nonFavoritePlayNowMenuItem);
		nonFavoriteMenu.add(new JSeparator());
		nonFavoriteMenu.add(nonFavoriteOpenFolderMenuItem);
		nonFavoriteMenu.add(new JSeparator());
		nonFavoriteMenu.add(nonFavoritePodcastFeedEntryDownload);
		nonFavoritePodcastFeedEntryDownload.setEnabled(false);
		nonFavoriteMenu.add(nonFavoriteRemoveOldPodcastFeedEntry);
		nonFavoriteMenu.add(nonFavoriteMarkPodcastEntryAsListened);
		nonFavoriteMenu.add(new JSeparator());
		nonFavoriteMenu.add(nonFavoriteEditTagMenuItem);
		nonFavoriteMenu.add(nonFavoriteEditTitlesMenuItem);
		nonFavoriteMenu.add(nonFavoriteClearTagMenuItem);
		nonFavoriteMenu.add(nonFavoriteExtractPictureMenuItem);
		nonFavoriteMenu.add(new JSeparator());
		nonFavoriteMenu.add(nonFavoriteRemovePhysicallyMenuItem);
		nonFavoriteMenu.add(new JSeparator());
		nonFavoriteMenu.add(nonFavoriteCopyToDeviceMenuItem);
		nonFavoriteMenu.add(new JSeparator());
		nonFavoriteMenu.add(nonFavoriteSetAsFavoriteSongMenuItem);
		nonFavoriteMenu.add(nonFavoriteSetAsFavoriteAlbumMenuItem);
		nonFavoriteMenu.add(nonFavoriteSetAsFavoriteArtistMenuItem);
		nonFavoriteMenu.add(new JSeparator());
		nonFavoriteMenu.add(nonFavoriteSearch);
		nonFavoriteMenu.add(nonFavoriteSearchAt);

		deviceMenu = new JPopupMenu();
		deviceCopyToRepositoryMenuItem = new JMenuItem(LanguageTool.getString("COPY_TO_REPOSITORY"), ImageLoader.EXPORT);
		deviceFillMenuItem = new JMenuItem(LanguageTool.getString("FILL_DEVICE_WITH_RANDOM_SONGS"), ImageLoader.SHUFFLE_PLAYLIST);
		deviceMenu.add(deviceCopyToRepositoryMenuItem);
		deviceMenu.add(deviceFillMenuItem);

		radioMenu = new JPopupMenu();
		radioAddRadioMenuItem = new JMenuItem(LanguageTool.getString("ADD_RADIO"), ImageLoader.RADIO_ADD);
		radioRefreshMenuItem = new JMenuItem(LanguageTool.getString("REFRESH"), ImageLoader.REFRESH);
		radioAddFavoriteMenuItem = new JMenuItem(LanguageTool.getString("ADD_FAVORITE_RADIO"), ImageLoader.RADIO_FAVORITE);
		radioAddFavoriteMenuItem.setToolTipText(LanguageTool.getString("RADIO_VIEW_TOOLTIP"));
		radioRenameRadioMenuItem = new JMenuItem(LanguageTool.getString("RENAME_RADIO"), ImageLoader.EDIT_ALBUM);
		radioRenameLabelMenuItem = new JMenuItem(LanguageTool.getString("RENAME_LABEL"), ImageLoader.EDIT_ALBUM);
		radioRemoveRadioMenuItem = new JMenuItem(LanguageTool.getString("REMOVE_RADIO"), ImageLoader.DELETE_FILE);
		radioMenu.add(radioAddRadioMenuItem);
		radioMenu.add(radioRefreshMenuItem);
		radioMenu.add(radioAddFavoriteMenuItem);
		radioMenu.add(radioRenameRadioMenuItem);
		radioMenu.add(radioRenameLabelMenuItem);
		radioMenu.add(radioRemoveRadioMenuItem);

		podcastFeedMenu = new JPopupMenu();
		podcastFeedAddPodcastFeedMenuItem = new JMenuItem(LanguageTool.getString("ADD_PODCAST_FEED"), ImageLoader.RSS_ADD);
		podcastFeedRenamePodcastFeedMenuItem = new JMenuItem(LanguageTool.getString("RENAME_PODCAST_FEED"), ImageLoader.EDIT_ALBUM);
		podcastFeedRemovePodcastFeedMenuItem = new JMenuItem(LanguageTool.getString("REMOVE_PODCAST_FEED"), ImageLoader.DELETE_FILE);
		podcastFeedMarkAsListenedMenuItem = new JMenuItem(LanguageTool.getString("MARK_PODCAST_AS_LISTENED"), ImageLoader.MARK_RSS_AS_READ);
		podcastFeedMenu.add(podcastFeedAddPodcastFeedMenuItem);
		podcastFeedMenu.add(podcastFeedRenamePodcastFeedMenuItem);
		podcastFeedMenu.add(podcastFeedMarkAsListenedMenuItem);
		podcastFeedMenu.add(podcastFeedRemovePodcastFeedMenuItem);

		// Create tabbed pane. Set tab position from configuration
		tabbedPane = new JTabbedPane(Kernel.getInstance().state.isShowNavigatorTabsAtLeft() ? SwingConstants.LEFT : SwingConstants.TOP);
		tabbedPane.setBorder(BorderFactory.createEmptyBorder());
		tabbedPane.addTab(null, ImageLoader.AUDIO_FILE, repositoryTreeScrollPane, LanguageTool.getString("REPOSITORY_TAB_TOOLTIP"));
		tabbedPane.addTab(null, ImageLoader.FAVORITE, scrollPane4, LanguageTool.getString("FAVORITES_TAB_TOOLTIP"));
		tabbedPane.addTab(null, ImageLoader.DEVICE, deviceTreeScrollPane, LanguageTool.getString("DEVICE_TAB_TOOLTIP"));
		tabbedPane.addTab(null, ImageLoader.RADIO_LITTLE, scrollPane6, LanguageTool.getString("RADIO_TAB_TOOLTIP"));
		tabbedPane.addTab(null, ImageLoader.RSS_LITTLE, scrollPane7, LanguageTool.getString("PODCAST_FEED_TAB_TOOLTIP"));

		// Set navigator tabs text based on configuration 
		setNavigatorTabsText(Kernel.getInstance().state.isShowNavigatorTabsText());

		JPanel buttonPanel = new JPanel(new GridBagLayout());

		// Filter controls
		filterLabel = new JLabel(LanguageTool.getString("FILTER"));
		filterTextField = new JTextField();
		filterTextField.setToolTipText(LanguageTool.getString("FILTER_TEXTFIELD_TOOLTIP"));
		clearFilterButton = new CustomButton(ImageLoader.UNDO, null);
		clearFilterButton.setPreferredSize(new Dimension(20, 20));
		clearFilterButton.setToolTipText(LanguageTool.getString("CLEAR_FILTER_BUTTON_TOOLTIP"));

		GridBagConstraints c = new GridBagConstraints();

		c.gridx = 1;
		c.gridy = 0;
		c.insets = new Insets(2, 5, 0, 2);
		buttonPanel.add(filterLabel, c);

		c.gridx = 2;
		c.gridy = 0;
		c.weightx = 0.5;
		c.weighty = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.WEST;
		c.insets = new Insets(1, 1, 1, 2);
		buttonPanel.add(filterTextField, c);

		c.gridx = 3;
		c.gridy = 0;
		c.weightx = 0.0;
		c.fill = GridBagConstraints.NONE;
		c.anchor = GridBagConstraints.EAST;
		c.insets = new Insets(1, 1, 1, 0);
		buttonPanel.add(clearFilterButton, c);

		treePanel.add(tabbedPane, BorderLayout.CENTER);
		treePanel.add(buttonPanel, BorderLayout.SOUTH);

		navigationTable = new DragSourceTable();
		navigationTable.getTableHeader().setResizingAllowed(false);
		navigationTable.getTableHeader().setReorderingAllowed(false);
		navigationTable.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

		navigationTable.setAutoResizeMode(JTable.AUTO_RESIZE_NEXT_COLUMN);

		navigationTable.setDefaultRenderer(Property.class, new SubstanceDefaultTableCellRenderer() {
			private static final long serialVersionUID = 1337377851290885658L;

			@Override
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
				Component comp = super.getTableCellRendererComponent(table, "", isSelected, hasFocus, row, column);
				ImageIcon icon = ImageLoader.EMPTY;
				Property val = (Property) value;
				if (val == Property.FAVORITE) {
					icon = ImageLoader.FAVORITE;
				} else if (val == Property.NOT_LISTENED_ENTRY) {
					icon = ImageLoader.NEW_PODCAST_ENTRY;
				} else if (val == Property.DOWNLOADED_ENTRY) {
					icon = ImageLoader.DOWNLOAD_PODCAST;
				} else if (val == Property.OLD_ENTRY) {
					icon = ImageLoader.REMOVE;
				}
				((JLabel) comp).setIcon(icon);
				return comp;

			}
		});

		navigationTable.setDefaultRenderer(Integer.class, new SubstanceDefaultTableCellRenderer() {
			private static final long serialVersionUID = 8693307342964711167L;

			@Override
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
				Integer intValue = (Integer) value;
				String stringValue;
				if (intValue < 0) {
					stringValue = "";
				} else {
					stringValue = String.valueOf(intValue);
				}
				Component comp = super.getTableCellRendererComponent(table, stringValue, isSelected, hasFocus, row, column);
				((JLabel) comp).setHorizontalAlignment(SwingConstants.CENTER);
				return comp;
			}
		});

		navigationTable.setDefaultRenderer(String.class, new SubstanceDefaultTableCellRenderer() {
			private static final long serialVersionUID = 8693307342964711167L;

			@Override
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
				Component comp = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				((JLabel) comp).setToolTipText((String) value);
				GuiUtils.applyComponentOrientation(((JLabel) comp));
				return comp;
			}
		});

		navigationTable.setDefaultRenderer(Long.class, new SubstanceDefaultTableCellRenderer() {
			private static final long serialVersionUID = 7614440163302045553L;

			@Override
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
				Component comp;
				if (((Long) value) > 0) {
					comp = super.getTableCellRendererComponent(table, TimeUtils.seconds2String((Long) value), isSelected, hasFocus, row, column);
					((JLabel) comp).setHorizontalAlignment(SwingConstants.RIGHT);
				} else {
					comp = super.getTableCellRendererComponent(table, "-", isSelected, hasFocus, row, column);
					((JLabel) comp).setHorizontalAlignment(SwingConstants.CENTER);
				}
				return comp;
			}
		});

		JScrollPane scrollPane2 = new JScrollPane(navigationTable);
		//scrollPane2.setBorder(BorderFactory.createEmptyBorder());

		navigationTableContainer = new JPanel(new BorderLayout());
		//navigationTableContainer.setMinimumSize(d);
		navigationTableButtonPanel = new JPanel(new GridBagLayout());

		navigationTableInfoButton = new CustomButton(ImageLoader.INFO, null);
		navigationTableInfoButton.setPreferredSize(new Dimension(20, 20));
		navigationTableInfoButton.setToolTipText(LanguageTool.getString("INFO_BUTTON_TOOLTIP"));

		optionsButton = new PopUpButton(LanguageTool.getString("OPTIONS"), PopUpButton.TOP_RIGHT);
		showArtist = new JRadioButtonMenuItem(LanguageTool.getString("SHOW_ARTISTS"), ImageLoader.ARTIST);
		showAlbum = new JRadioButtonMenuItem(LanguageTool.getString("SHOW_ALBUMS"), ImageLoader.ALBUM);
		showGenre = new JRadioButtonMenuItem(LanguageTool.getString("SHOW_GENRE"), ImageLoader.GENRE);
		showFolder = new JRadioButtonMenuItem(LanguageTool.getString("SHOW_FOLDERS"), ImageLoader.FOLDER);
		expandTree = new JMenuItem(LanguageTool.getString("EXPAND"));
		collapseTree = new JMenuItem(LanguageTool.getString("COLLAPSE"));

		optionsButton.add(showArtist);
		optionsButton.add(showAlbum);
		optionsButton.add(showGenre);
		optionsButton.add(showFolder);
		optionsButton.add(new JSeparator());
		optionsButton.add(expandTree);
		optionsButton.add(collapseTree);
		optionsButton.add(new JSeparator());
		ButtonGroup group = new ButtonGroup();
		group.add(showArtist);
		group.add(showAlbum);
		group.add(showGenre);
		group.add(showFolder);

		sortByTrack = new JRadioButtonMenuItem(LanguageTool.getString("SORT_BY_TRACK_NUMBER"));
		sortByTitle = new JRadioButtonMenuItem(LanguageTool.getString("SORT_BY_TITLE"));
		sortByFile = new JRadioButtonMenuItem(LanguageTool.getString("SORT_BY_FILE_NAME"));
		sortByModificationTime = new JRadioButtonMenuItem(LanguageTool.getString("SORT_BY_MODIFICATION_DATE"));
		optionsButton.add(sortByTrack);
		optionsButton.add(sortByTitle);
		optionsButton.add(sortByFile);
		optionsButton.add(sortByModificationTime);
		sortByTrack.setOpaque(true);
		sortByTitle.setOpaque(true);
		sortByFile.setOpaque(true);
		sortByModificationTime.setOpaque(true);
		ButtonGroup group2 = new ButtonGroup();
		group2.add(sortByTrack);
		group2.add(sortByTitle);
		group2.add(sortByFile);
		group2.add(sortByModificationTime);

		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 0.5;
		c.weighty = 0;
		c.fill = GridBagConstraints.NONE;
		c.anchor = GridBagConstraints.WEST;
		c.insets = new Insets(1, 1, 1, 0);
		navigationTableButtonPanel.add(optionsButton, c);

		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 0;
		c.weighty = 0;
		c.anchor = GridBagConstraints.EAST;
		navigationTableButtonPanel.add(navigationTableInfoButton, c);

		splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);

		navigationTableContainer.add(scrollPane2, BorderLayout.CENTER);
		navigationTableContainer.add(navigationTableButtonPanel, BorderLayout.SOUTH);
		splitPane.add(treePanel);
		splitPane.add(navigationTableContainer);

		splitPane.setResizeWeight(0.8);

		add(splitPane);

		GuiUtils.applyComponentOrientation(favoriteMenu, nonFavoriteMenu, deviceMenu, radioMenu, podcastFeedMenu, optionsButton);
	}

	/**
	 * Gets the adds the to playlist menu item.
	 * 
	 * @return the adds the to playlist menu item
	 */
	public JMenuItem getAddToPlaylistMenuItem() {
		return addToPlaylistMenuItem;
	}

	/**
	 * Gets the clear filter button.
	 * 
	 * @return the clear filter button
	 */
	public CustomButton getClearFilterButton() {
		return clearFilterButton;
	}

	/**
	 * Gets the collapse tree.
	 * 
	 * @return the collapse tree
	 */
	public JMenuItem getCollapseTree() {
		return collapseTree;
	}

	/**
	 * Gets the device copy to repository menu item.
	 * 
	 * @return the device copy to repository menu item
	 */
	public JMenuItem getDeviceCopyToRepositoryMenuItem() {
		return deviceCopyToRepositoryMenuItem;
	}

	/**
	 * Gets the device fill menu item.
	 * 
	 * @return the device fill menu item
	 */
	public JMenuItem getDeviceFillMenuItem() {
		return deviceFillMenuItem;
	}

	/**
	 * Gets the device menu.
	 * 
	 * @return the device menu
	 */
	public JPopupMenu getDeviceMenu() {
		return deviceMenu;
	}

	/**
	 * Gets the device tree.
	 * 
	 * @return the device tree
	 */
	public JTree getDeviceTree() {
		return deviceTree;
	}

	/**
	 * Gets the device tree model.
	 * 
	 * @return the device tree model
	 */
	public DefaultTreeModel getDeviceTreeModel() {
		return deviceTreeModel;
	}

	/**
	 * Gets the expand tree.
	 * 
	 * @return the expand tree
	 */
	public JMenuItem getExpandTree() {
		return expandTree;
	}

	/**
	 * Gets the favorite menu.
	 * 
	 * @return the favorite menu
	 */
	public JPopupMenu getFavoriteMenu() {
		return favoriteMenu;
	}

	/**
	 * Gets the favorites tree.
	 * 
	 * @return the favorites tree
	 */
	public JTree getFavoritesTree() {
		return favoritesTree;
	}

	/**
	 * Gets the favorites tree model.
	 * 
	 * @return the favorites tree model
	 */
	public DefaultTreeModel getFavoritesTreeModel() {
		return favoritesTreeModel;
	}

	/**
	 * Gets the filter label.
	 * 
	 * @return the filter label
	 */
	public JLabel getFilterLabel() {
		return filterLabel;
	}

	/**
	 * Gets the filter text field.
	 * 
	 * @return the filter text field
	 */
	public JTextField getFilterTextField() {
		return filterTextField;
	}

	/**
	 * Gets the navigation table.
	 * 
	 * @return the navigation table
	 */
	public JTable getNavigationTable() {
		return navigationTable;
	}

	/**
	 * Gets the navigation table container.
	 * 
	 * @return the navigation table container
	 */
	public JPanel getNavigationTableContainer() {
		return navigationTableContainer;
	}

	/**
	 * Gets the navigation table info button.
	 * 
	 * @return the navigation table info button
	 */
	public CustomButton getNavigationTableInfoButton() {
		return navigationTableInfoButton;
	}

	/**
	 * Gets the navigation table sort button.
	 * 
	 * @return the navigationTableSortButton
	 */
	public PopUpButton getNavigationTableSortButton() {
		return optionsButton;
	}

	/**
	 * Gets the repository tree.
	 * 
	 * @return the repository tree
	 */
	public JTree getRepositoryTree() {
		return repositoryTree;
	}

	/**
	 * Gets the repository tree model.
	 * 
	 * @return the repository tree model
	 */
	public DefaultTreeModel getRepositoryTreeModel() {
		return repositoryTreeModel;
	}

	/**
	 * Gets the repository tree scroll pane.
	 * 
	 * @return the repository tree scroll pane
	 */
	public JScrollPane getRepositoryTreeScrollPane() {
		return repositoryTreeScrollPane;
	}

	/**
	 * Gets the non favorite add to playlist menu item.
	 * 
	 * @return the non favorite add to playlist menu item
	 */
	public JMenuItem getNonFavoriteAddToPlaylistMenuItem() {
		return nonFavoriteAddToPlaylistMenuItem;
	}

	/**
	 * Gets the non favorite clear tag menu item.
	 * 
	 * @return the non favorite clear tag menu item
	 */
	public JMenuItem getNonFavoriteClearTagMenuItem() {
		return nonFavoriteClearTagMenuItem;
	}

	/**
	 * Gets the non favorite copy to device menu item.
	 * 
	 * @return the nonFavoriteCopyToDeviceMenuItem
	 */
	public JMenuItem getNonFavoriteCopyToDeviceMenuItem() {
		return nonFavoriteCopyToDeviceMenuItem;
	}

	/**
	 * Gets the non favorite edit tag menu item.
	 * 
	 * @return the non favorite edit tag menu item
	 */
	public JMenuItem getNonFavoriteEditTagMenuItem() {
		return nonFavoriteEditTagMenuItem;
	}

	/**
	 * Gets the non favorite edit titles menu item.
	 * 
	 * @return the non favorite edit titles menu item
	 */
	public JMenuItem getNonFavoriteEditTitlesMenuItem() {
		return nonFavoriteEditTitlesMenuItem;
	}

	/**
	 * Gets the non favorite extract picture menu item.
	 * 
	 * @return the non favorite extract picture menu item
	 */
	public JMenuItem getNonFavoriteExtractPictureMenuItem() {
		return nonFavoriteExtractPictureMenuItem;
	}

	/**
	 * Gets the non favorite mark podcast entry as listened.
	 * 
	 * @return the non favorite mark podcast entry as listened
	 */
	public JMenuItem getNonFavoriteMarkPodcastEntryAsListened() {
		return nonFavoriteMarkPodcastEntryAsListened;
	}

	/**
	 * Gets the non favorite menu.
	 * 
	 * @return the non favorite menu
	 */
	public JPopupMenu getNonFavoriteMenu() {
		return nonFavoriteMenu;
	}

	/**
	 * Gets the non favorite play now menu item.
	 * 
	 * @return the nonFavoritePlayNowMenuItem
	 */
	public JMenuItem getNonFavoritePlayNowMenuItem() {
		return nonFavoritePlayNowMenuItem;
	}

	/**
	 * Gets the nonFavoriteOpenFolderMenuItem
	 * 
	 * @return the nonFavoriteOpenFolderMenuItem
	 */
	public JMenuItem getNonFavoriteOpenFolderMenuItem() {
		return nonFavoriteOpenFolderMenuItem;
	}

	/**
	 * Gets the non favorite remove physically menu item.
	 * 
	 * @return the nonFavoriteRemovePhysicallyMenuItem
	 */
	public JMenuItem getNonFavoriteRemovePhysicallyMenuItem() {
		return nonFavoriteRemovePhysicallyMenuItem;
	}

	/**
	 * Gets the non favorite search.
	 * 
	 * @return the non favorite search
	 */
	public JMenuItem getNonFavoriteSearch() {
		return nonFavoriteSearch;
	}

	/**
	 * Gets the non favorite search at.
	 * 
	 * @return the non favorite search at
	 */
	public JMenuItem getNonFavoriteSearchAt() {
		return nonFavoriteSearchAt;
	}

	/**
	 * Gets the non favorite set as favorite album menu item.
	 * 
	 * @return the non favorite set as favorite album menu item
	 */
	public JMenuItem getNonFavoriteSetAsFavoriteAlbumMenuItem() {
		return nonFavoriteSetAsFavoriteAlbumMenuItem;
	}

	/**
	 * Gets the non favorite set as favorite artist menu item.
	 * 
	 * @return the non favorite set as favorite artist menu item
	 */
	public JMenuItem getNonFavoriteSetAsFavoriteArtistMenuItem() {
		return nonFavoriteSetAsFavoriteArtistMenuItem;
	}

	/**
	 * Gets the non favorite set as favorite song menu item.
	 * 
	 * @return the non favorite set as favorite song menu item
	 */
	public JMenuItem getNonFavoriteSetAsFavoriteSongMenuItem() {
		return nonFavoriteSetAsFavoriteSongMenuItem;
	}

	/**
	 * Gets the non favorite set as playlist menu item.
	 * 
	 * @return the non favorite set as playlist menu item
	 */
	public JMenuItem getNonFavoriteSetAsPlaylistMenuItem() {
		return nonFavoriteSetAsPlaylistMenuItem;
	}

	/**
	 * Gets the play now menu item.
	 * 
	 * @return the playNowMenuItem
	 */
	public JMenuItem getPlayNowMenuItem() {
		return playNowMenuItem;
	}

	/**
	 * Gets the podcast feed add podcast feed menu item.
	 * 
	 * @return the podcastFeedAddPodcastFeed
	 */
	public JMenuItem getPodcastFeedAddPodcastFeedMenuItem() {
		return podcastFeedAddPodcastFeedMenuItem;
	}

	/**
	 * Gets the podcast feed mark as listened menu item.
	 * 
	 * @return the podcastFeedMarkAsListenedMenuItem
	 */
	public JMenuItem getPodcastFeedMarkAsListenedMenuItem() {
		return podcastFeedMarkAsListenedMenuItem;
	}

	/**
	 * Gets the podcast feed menu.
	 * 
	 * @return the podcastFeedMenu
	 */
	public JPopupMenu getPodcastFeedMenu() {
		return podcastFeedMenu;
	}

	/**
	 * Gets the podcast feed remove podcast feed menu item.
	 * 
	 * @return the podcastFeedRemovePodcastFeedMenuItem
	 */
	public JMenuItem getPodcastFeedRemovePodcastFeedMenuItem() {
		return podcastFeedRemovePodcastFeedMenuItem;
	}

	/**
	 * Gets the podcast feed rename podcast feed menu item.
	 * 
	 * @return the podcast feed rename podcast feed menu item
	 */
	public JMenuItem getPodcastFeedRenamePodcastFeedMenuItem() {
		return podcastFeedRenamePodcastFeedMenuItem;
	}

	/**
	 * Gets the podcast feed tree.
	 * 
	 * @return the podcastFeedTree
	 */
	public JTree getPodcastFeedTree() {
		return podcastFeedTree;
	}

	/**
	 * Gets the podcast feed tree model.
	 * 
	 * @return the podcastFeedTreeModel
	 */
	public DefaultTreeModel getPodcastFeedTreeModel() {
		return podcastFeedTreeModel;
	}

	/**
	 * Gets the radio add radio menu item.
	 * 
	 * @return the radioAddRadio
	 */
	public JMenuItem getRadioAddRadioMenuItem() {
		return radioAddRadioMenuItem;
	}

	/**
	 * Gets the radio refresh menu item.
	 * 
	 * @return the radioRefresh
	 */
	public JMenuItem getRadioRefreshMenuItem() {
		return radioRefreshMenuItem;
	}

	/**
	 * Gets the radio add favorite menu item.
	 * 
	 * @return the radioAddRadio
	 */
	public JMenuItem getRadioAddFavoriteMenuItem() {
		return radioAddFavoriteMenuItem;
	}

	/**
	 * Gets the radio menu.
	 * 
	 * @return the radioMenu
	 */
	public JPopupMenu getRadioMenu() {
		return radioMenu;
	}

	/**
	 * Gets the radio remove radio menu item.
	 * 
	 * @return the radioRemoveRadioMenuItem
	 */
	public JMenuItem getRadioRemoveRadioMenuItem() {
		return radioRemoveRadioMenuItem;
	}

	/**
	 * Gets the radio rename radio menu item.
	 * 
	 * @return the radio rename radio menu item
	 */
	public JMenuItem getRadioRenameRadioMenuItem() {
		return radioRenameRadioMenuItem;
	}

	/**
	 * Gets the radio rename label menu item.
	 * 
	 * @return the radio rename label menu item
	 */
	public JMenuItem getRadioRenameLabelMenuItem() {
		return radioRenameLabelMenuItem;
	}

	/**
	 * Gets the radio tree.
	 * 
	 * @return the radioTree
	 */
	public JTree getRadioTree() {
		return radioTree;
	}

	/**
	 * Gets the radio tree model.
	 * 
	 * @return the radioTreeModel
	 */
	public DefaultTreeModel getRadioTreeModel() {
		return radioTreeModel;
	}

	/**
	 * Gets the removes the from favorites menu item.
	 * 
	 * @return the removes the from favorites menu item
	 */
	public JMenuItem getRemoveFromFavoritesMenuItem() {
		return removeFromFavoritesMenuItem;
	}

	/**
	 * Gets the sets the as playlist menu item.
	 * 
	 * @return the sets the as playlist menu item
	 */
	public JMenuItem getSetAsPlaylistMenuItem() {
		return setAsPlaylistMenuItem;
	}

	/**
	 * Gets the show album.
	 * 
	 * @return the show album
	 */
	public JRadioButtonMenuItem getShowAlbum() {
		return showAlbum;
	}

	/**
	 * Gets the show artist.
	 * 
	 * @return the show artist
	 */
	public JRadioButtonMenuItem getShowArtist() {
		return showArtist;
	}

	/**
	 * Gets the show genre.
	 * 
	 * @return the show genre
	 */
	public JRadioButtonMenuItem getShowGenre() {
		return showGenre;
	}

	/**
	 * Gets the sort by file.
	 * 
	 * @return the sort by file
	 */
	public JRadioButtonMenuItem getSortByFile() {
		return sortByFile;
	}

	/**
	 * Gets the sort by title.
	 * 
	 * @return the sort by title
	 */
	public JRadioButtonMenuItem getSortByTitle() {
		return sortByTitle;
	}

	/**
	 * Gets the sort by track.
	 * 
	 * @return the sort by track
	 */
	public JRadioButtonMenuItem getSortByTrack() {
		return sortByTrack;
	}

	/**
	 * Gets the split pane.
	 * 
	 * @return the split pane
	 */
	public JSplitPane getSplitPane() {
		return splitPane;
	}

	/**
	 * Gets the tabbed pane.
	 * 
	 * @return the tabbed pane
	 */
	public JTabbedPane getTabbedPane() {
		return tabbedPane;
	}

	/**
	 * Gets the non favorite podcast feed entry download.
	 * 
	 * @return the non favorite podcast feed entry download
	 */
	public JMenuItem getNonFavoritePodcastFeedEntryDownload() {
		return nonFavoritePodcastFeedEntryDownload;
	}

	/**
	 * Gets the non favorite remove old podcast feed entry.
	 * 
	 * @return the non favorite remove old podcast feed entry
	 */
	public JMenuItem getNonFavoriteRemoveOldPodcastFeedEntry() {
		return nonFavoriteRemoveOldPodcastFeedEntry;
	}

	/**
	 * Gets the sort by modification time.
	 * 
	 * @return the sortByModificationTime
	 */
	public JRadioButtonMenuItem getSortByModificationTime() {
		return sortByModificationTime;
	}

	/**
	 * Gets the show folder.
	 * 
	 * @return the showFolder
	 */
	public JRadioButtonMenuItem getShowFolder() {
		return showFolder;
	}

	public void setNavigatorTabsText(boolean set) {
		tabbedPane.setTitleAt(0, set ? LanguageTool.getString("REPOSITORY") : null);
		tabbedPane.setTitleAt(1, set ? LanguageTool.getString("FAVORITES") : null);
		tabbedPane.setTitleAt(2, set ? LanguageTool.getString("DEVICE") : null);
		tabbedPane.setTitleAt(3, set ? LanguageTool.getString("RADIO") : null);
		tabbedPane.setTitleAt(4, set ? LanguageTool.getString("PODCAST_FEEDS") : null);
	}

	/**
	 * @return the deviceTreeScrollPane
	 */
	public JScrollPane getDeviceTreeScrollPane() {
		return deviceTreeScrollPane;
	}

}
