/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.contextInformation;

import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Arrays;
import java.util.Date;
import java.util.Map.Entry;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import net.sourceforge.atunes.Constants;
import net.sourceforge.atunes.gui.GuiUtils;
import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.model.ContextInformationAlbumsTableModel;
import net.sourceforge.atunes.gui.model.ContextInformationArtistsTableModel;
import net.sourceforge.atunes.gui.model.ContextInformationTracksTableModel;
import net.sourceforge.atunes.gui.views.controls.PopUpButton;
import net.sourceforge.atunes.gui.views.panels.ContextInformationPanel;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.controllers.model.PanelController;
import net.sourceforge.atunes.kernel.handlers.ContextInformationServiceHandler;
import net.sourceforge.atunes.kernel.handlers.DesktopHandler;
import net.sourceforge.atunes.kernel.handlers.PlayerHandler;
import net.sourceforge.atunes.kernel.handlers.RepositoryHandler;
import net.sourceforge.atunes.kernel.modules.contextInformation.AlbumInfo;
import net.sourceforge.atunes.kernel.modules.contextInformation.ArtistInfo;
import net.sourceforge.atunes.kernel.modules.contextInformation.Lyrics;
import net.sourceforge.atunes.kernel.modules.contextInformation.TrackInfo;
import net.sourceforge.atunes.kernel.modules.internetsearch.SearchFactory;
import net.sourceforge.atunes.kernel.modules.playlist.PlayListCommonOps;
import net.sourceforge.atunes.kernel.modules.podcast.PodcastFeedEntry;
import net.sourceforge.atunes.kernel.modules.radio.Radio;
import net.sourceforge.atunes.kernel.modules.repository.SongStats;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.utils.ClipboardFacade;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.DateUtils;
import net.sourceforge.atunes.utils.ImageUtils;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;

/**
 * The Class ContextInformationPanelController.
 */
public class ContextInformationPanelController extends PanelController<ContextInformationPanel> {

	private String lyricsSourceUrl;
	/** Query to use when searching for a video. */
	private String searchVideoQuery = null;
	private AudioObject audioObject;

	/**
	 * Instantiates a new audio scrobbler controller.
	 * 
	 * @param panel
	 *            the panel
	 */
	public ContextInformationPanelController(ContextInformationPanel panel) {
		super(panel);
		addBindings();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#addBindings()
	 */
	@Override
	protected void addBindings() {
		ContextInformationPanelListener objListener = new ContextInformationPanelListener(this);

		getPanelControlled().getTracksTable().getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				if (!e.getValueIsAdjusting()) {
					int selectedTrack = getPanelControlled().getTracksTable().getSelectedRow();
					if (selectedTrack != -1) {
						TrackInfo track = ((ContextInformationTracksTableModel) getPanelControlled().getTracksTable().getModel()).getTrack(selectedTrack);
						DesktopHandler.getInstance().openURL(track.getUrl());
					}
				}
			}
		});

		getPanelControlled().getAlbumsTable().getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				if (!e.getValueIsAdjusting()) {
					int selectedAlbum = getPanelControlled().getAlbumsTable().getSelectedRow();
					if (selectedAlbum != -1) {
						AlbumInfo album = ((ContextInformationAlbumsTableModel) getPanelControlled().getAlbumsTable().getModel()).getAlbum(selectedAlbum);
						DesktopHandler.getInstance().openURL(album.getUrl());
					}
				}
			}
		});

		getPanelControlled().getSimilarArtistsTable().getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				if (!e.getValueIsAdjusting()) {
					int selectedArtist = getPanelControlled().getSimilarArtistsTable().getSelectedRow();
					if (selectedArtist != -1) {
						ArtistInfo artist = ((ContextInformationArtistsTableModel) getPanelControlled().getSimilarArtistsTable().getModel()).getArtist(selectedArtist);
						DesktopHandler.getInstance().openURL(artist.getUrl());
					}
				}
			}
		});

		getPanelControlled().getTabbedPane().addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				int i = getPanelControlled().getTabbedPane().indexAtLocation(e.getX(), e.getY());
				if (i != -1) {
					Kernel.getInstance().state.setSelectedContextInformationTab(i);
				}
			}
		});

		getPanelControlled().getCopyLyricToClipboard().addActionListener(objListener);
		getPanelControlled().getOpenLyricsSource().addActionListener(objListener);
		getPanelControlled().getSearchForVideo().addActionListener(objListener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#addStateBindings
	 * ()
	 */
	@Override
	protected void addStateBindings() {
		// Nothing to do
	}

	/**
	 * Clear.
	 * 
	 * @param clearArtistAndSimilar
	 *            the clear artist and similar
	 */
	public void clear(boolean clearArtistAndSimilar) {
		getLogger().debug(LogCategories.CONTROLLER, new String[] { Boolean.toString(clearArtistAndSimilar) });

		lyricsSourceUrl = null;
		searchVideoQuery = null;

		getPanelControlled().getTracksTable().setModel(new ContextInformationTracksTableModel(null));
		getPanelControlled().getAlbumPanel().setVisible(false);
		getPanelControlled().getLyricsCover().setVisible(false);
		getPanelControlled().getLyricsContainer().setText("");
		getPanelControlled().getLyricsLabel().setText("");
		getPanelControlled().getLyricsLabel().setToolTipText(null);
		getPanelControlled().getLyricsArtistLabel().setText("");
		getPanelControlled().getLyricsLastPlayDateLabel().setText("");
		getPanelControlled().getCopyLyricToClipboard().setEnabled(false);
		getPanelControlled().getOpenLyricsSource().setEnabled(false);
		getPanelControlled().getAddLyrics().setEnabled(false);
		getPanelControlled().getLyricScrollPane().setEnabled(false);
		getPanelControlled().getSearchForVideo().setVisible(false);
		getPanelControlled().getCopyLyricToClipboard().setVisible(false);
		getPanelControlled().getOpenLyricsSource().setVisible(false);
		getPanelControlled().getAddLyrics().setVisible(false);
		getPanelControlled().getAddLyrics().removeAllItems();

		getPanelControlled().getLyricScrollPane().setVisible(false);

		if (clearArtistAndSimilar) {
			getPanelControlled().getArtistImageLabel().setVisible(false);
			getPanelControlled().getArtistAlbumsLabel().setText("", "");
			getPanelControlled().getAlbumsPanel().setVisible(false);
			getPanelControlled().getSimilarArtistsPanel().setVisible(false);
			getPanelControlled().getArtistLabel().setText("", "");
		}
	}

	/**
	 * Clear albums container.
	 */
	public void clearAlbumsContainer() {
		getPanelControlled().getAlbumsTable().setModel(new ContextInformationAlbumsTableModel());
	}

	/**
	 * Clear similar artists container.
	 */
	public void clearSimilarArtistsContainer() {
		getPanelControlled().getSimilarArtistsTable().setModel(new ContextInformationArtistsTableModel());
	}

	/**
	 * Copies current lyric to clipboard.
	 */
	void copyToClipboard() {
		String sLyric = getPanelControlled().getLyricsContainer().getText();
		if (sLyric == null) {
			sLyric = "";
		}
		ClipboardFacade.copyToClipboard(sLyric);
	}

	/**
	 * Fill albums list.
	 * 
	 * @param album
	 *            the album
	 * @param cover
	 *            the cover
	 */
	void fillAlbumsList(AlbumInfo album, Image cover) {
		album.setCover(ImageUtils.scaleImageBicubic(cover, Constants.CONTEXT_INFORMATION_IMAGE_WIDTH, Constants.CONTEXT_INFORMATION_IMAGE_HEIGHT));
		((ContextInformationAlbumsTableModel) getPanelControlled().getAlbumsTable().getModel()).addAlbum(album);

		getPanelControlled().getAlbumsTable().revalidate();
		getPanelControlled().getAlbumsTable().repaint();
	}

	/**
	 * Fill similar artists list.
	 * 
	 * @param artist
	 *            the artist
	 * @param img
	 *            the img
	 */
	void fillSimilarArtistsList(ArtistInfo artist, Image img) {
		artist.setImage(ImageUtils.scaleImageBicubic(img, Constants.CONTEXT_INFORMATION_IMAGE_WIDTH, Constants.CONTEXT_INFORMATION_IMAGE_HEIGHT));
		((ContextInformationArtistsTableModel) getPanelControlled().getSimilarArtistsTable().getModel()).addArtist(artist);

		getPanelControlled().getSimilarArtistsTable().revalidate();
		getPanelControlled().getSimilarArtistsTable().repaint();
	}

	/**
	 * Gets the panel controlled.
	 * 
	 * @return the panel controlled
	 */
	@Override
	protected ContextInformationPanel getPanelControlled() {
		return super.getPanelControlled();
	}

	/**
	 * Notify artist image.
	 * 
	 * @param img
	 *            the img
	 */
	public void notifyArtistImage(final Image img) {
		getLogger().debug(LogCategories.CONTROLLER);

		getPanelControlled().getArtistImageLabel().setIcon(new ImageIcon(img));
		getPanelControlled().getArtistImageLabel().setVisible(true);
	}

	/**
	 * Notify finish get album info.
	 * 
	 * @param artist
	 *            the artist
	 * @param album
	 *            the album
	 * @param img
	 *            the img
	 */
	public void notifyFinishGetAlbumInfo(final String artist, final AlbumInfo album, final Image img) {
		getPanelControlled().getArtistLabel().setText(album != null ? album.getArtist() : artist, album != null ? album.getArtistUrl() : null);
		getPanelControlled().getAlbumLabel().setText(album != null ? album.getTitle() : LanguageTool.getString("UNKNOWN_ALBUM"), album != null ? album.getUrl() : null);

		getPanelControlled().getArtistLabel().setEnabled(album != null && album.getArtistUrl() != null);
		getPanelControlled().getAlbumLabel().setEnabled(album != null && album.getUrl() != null);

		getPanelControlled().getYearLabel().setText(album != null ? album.getYear() : "",
				album != null && album.getYear() != null ? StringUtils.getString("http://en.wikipedia.org/wiki/", album.getYear()) : null);
		if (album != null) {
			getLogger().debug(LogCategories.CONTROLLER, new String[] { album.getTitle() });
		}
		if (img != null) {
			ImageIcon image = ImageUtils.resize(new ImageIcon(img), 170, 170);
			getPanelControlled().getAlbumCoverLabel().setIcon(image);
			getPanelControlled().getLyricsCover().setIcon(image);
			getPanelControlled().getLyricsCover().setBorder(BorderFactory.createLineBorder(GuiUtils.getBorderColor()));
			getPanelControlled().getLyricsCover().setVisible(true);
		}
		getPanelControlled().getAlbumCoverLabel().setVisible(img != null);
		getPanelControlled().getTracksTable().setModel(new ContextInformationTracksTableModel(album));
		getPanelControlled().getAlbumPanel().setVisible(true);
	}

	/**
	 * Notify finish get albums info.
	 * 
	 * @param album
	 *            the album
	 * @param cover
	 *            the cover
	 */
	public void notifyFinishGetAlbumsInfo(final AlbumInfo album, final Image cover) {
		getLogger().debug(LogCategories.CONTROLLER, new String[] { album.getTitle() });

		if (album.getArtist().equals(audioObject.getArtist())) {
			getPanelControlled().getArtistAlbumsLabel().setText(album.getArtist(), album.getArtistUrl());
			getPanelControlled().getArtistAlbumsLabel().setVisible(true);
		}
		if (!getPanelControlled().getAlbumsPanel().isVisible()) {
			getPanelControlled().getAlbumsPanel().setVisible(true);
		}
		fillAlbumsList(album, cover);
	}

	/**
	 * Notify finish get similar artist.
	 * 
	 * @param artist
	 *            the artist
	 * @param img
	 *            the img
	 */
	public void notifyFinishGetSimilarArtist(final ArtistInfo artist, final Image img) {
		getLogger().debug(LogCategories.CONTROLLER);

		getPanelControlled().getSimilarArtistsPanel().setVisible(true);
		fillSimilarArtistsList(artist, img);
	}

	/**
	 * Notify podcast.
	 * 
	 * @param entry
	 *            the entry
	 */
	public void notifyPodcast(PodcastFeedEntry entry) {
		clear(true);
		getPanelControlled().getLyricsCover().setIcon(ImageLoader.RSS);
		getPanelControlled().getLyricsCover().setBorder(BorderFactory.createEmptyBorder());
		getPanelControlled().getLyricsCover().setVisible(true);
		getPanelControlled().getLyricsLabel().setText(entry.getName());
		getPanelControlled().getLyricsLabel().setToolTipText(entry.getName());
		getPanelControlled().getLyricScrollPane().setVisible(true);
		getPanelControlled().getLyricsContainer().setText(entry.getDescription());
		getPanelControlled().getLyricsContainer().setCaretPosition(0);
		getPanelControlled().getLyricsContainer().setLineWrap(true);
	}

	/**
	 * Notify radio.
	 * 
	 * @param radio
	 *            the radio
	 */
	public void notifyRadio(Radio radio) {
		clear(true);
		getPanelControlled().getLyricsCover().setIcon(ImageLoader.RADIO);
		getPanelControlled().getLyricsCover().setBorder(BorderFactory.createEmptyBorder());
		getPanelControlled().getLyricsCover().setVisible(true);
		getPanelControlled().getLyricsLabel().setText(radio.getName());
		getPanelControlled().getLyricsArtistLabel().setText(radio.getUrl());
		getPanelControlled().getLyricScrollPane().setVisible(true);
		getPanelControlled().getLyricsContainer().setLineWrap(true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#notifyReload()
	 */
	@Override
	protected void notifyReload() {
		// Nothing to do
	}

	/**
	 * Opens a navigator to search for video.
	 */
	void searchForVideo() {
		if (searchVideoQuery != null) {
			DesktopHandler.getInstance().openSearch(SearchFactory.getSearchForName("YouTube"), searchVideoQuery);
		}
	}

	void openLyricsSourceUrl() {
		if (lyricsSourceUrl != null && !lyricsSourceUrl.trim().isEmpty()) {
			DesktopHandler.getInstance().openURL(lyricsSourceUrl);
		} else {
			AudioObject currentAudioObject = PlayListCommonOps.getCurrentAudioObject();
			if (currentAudioObject != null && currentAudioObject instanceof AudioFile) {
				ControllerProxy.getInstance().getEditTagDialogController().editFiles(Arrays.asList((AudioFile) currentAudioObject));
			}
		}
	}

	/**
	 * Sets the audio object info
	 * 
	 * @param audioObject
	 *            the audio object
	 */
	public void setAudioObjectInfo(AudioObject audioObject) {
		getLogger().debug(LogCategories.CONTROLLER);

		lyricsSourceUrl = null;
		this.audioObject = audioObject;

		getPanelControlled().getSearchForVideo().setVisible(true);
		// Set query string
		searchVideoQuery = StringUtils.getString(audioObject.getArtist(), " ", audioObject.getTitleOrFileName());
		getPanelControlled().getCopyLyricToClipboard().setEnabled(false);
		getPanelControlled().getCopyLyricToClipboard().setVisible(true);
		getPanelControlled().getOpenLyricsSource().setEnabled(false);
		getPanelControlled().getOpenLyricsSource().setVisible(true);
		getPanelControlled().getAddLyrics().setEnabled(false);
		getPanelControlled().getAddLyrics().setVisible(true);
		getPanelControlled().getAddLyrics().removeAllItems();
		getPanelControlled().getLyricsContainer().setText("");
		getPanelControlled().getLyricScrollPane().setVisible(true);
		getPanelControlled().getLyricsLabel().setText(audioObject.getTitleOrFileName());
		getPanelControlled().getLyricsArtistLabel().setText(audioObject.getArtist());

		// Get last date played
		SongStats stats = null;
		if (audioObject instanceof AudioFile) {
			stats = RepositoryHandler.getInstance().getSongStatistics((AudioFile) audioObject);
		}
		if (stats == null) {
			getPanelControlled().getLyricsLastPlayDateLabel().setText(LanguageTool.getString("SONG_NEVER_PLAYED"));
		} else {
			// If song is playing, take previous play, if not, take last
			Date date;
			if (PlayerHandler.getInstance().isEnginePlaying()) {
				date = stats.getPreviousPlayed();
			} else {
				date = stats.getLastPlayed();
			}

			// If date is null -> never player
			if (date == null) {
				getPanelControlled().getLyricsLastPlayDateLabel().setText(LanguageTool.getString("SONG_NEVER_PLAYED"));
			} else {
				getPanelControlled().getLyricsLastPlayDateLabel().setText(StringUtils.getString(LanguageTool.getString("LAST_DATE_PLAYED"), ": ", DateUtils.toString(date)));
			}
		}
	}

	/**
	 * Sets the lyrics.
	 * 
	 * @param audioObject
	 *            the audio object
	 * @param lyrics
	 *            the lyrics
	 */
	public void setLyrics(AudioObject audioObject, Lyrics lyrics) {
		getLogger().debug(LogCategories.CONTROLLER);

		getPanelControlled().getLyricsContainer().setLineWrap(false);
		getPanelControlled().getLyricsContainer().setText(lyrics != null ? lyrics.getLyrics() : "");
		getPanelControlled().getLyricsContainer().setCaretPosition(0);

		boolean lyricsNotEmpty = lyrics != null && !lyrics.getLyrics().trim().isEmpty();
		getPanelControlled().getCopyLyricToClipboard().setEnabled(lyricsNotEmpty);
		getPanelControlled().getOpenLyricsSource().setEnabled(lyricsNotEmpty);
		getPanelControlled().getAddLyrics().setEnabled(!lyricsNotEmpty);

		lyricsSourceUrl = lyrics != null ? lyrics.getUrl() : null;
		//getPanelControlled().getOpenLyricsSource().setEnabled(lyricsSourceUrl != null && !lyricsSourceUrl.trim().isEmpty());

		if (!lyricsNotEmpty) {
			PopUpButton button = getPanelControlled().getAddLyrics();
			button.removeAllItems();
			for (final Entry<String, String> entry : ContextInformationServiceHandler.getInstance().getUrlsForAddingLyrics(audioObject.getArtist(), audioObject.getTitle())
					.entrySet()) {
				JMenuItem mi = new JMenuItem(entry.getKey());
				mi.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent e) {
						DesktopHandler.getInstance().openURL(entry.getValue());
					}
				});
				button.add(mi);
			}
		}
	}

	/**
	 * Sets the wiki information.
	 * 
	 * @param wikiText
	 *            the wiki text
	 * @param wikiURL
	 *            the wiki url
	 */
	public void setWikiInformation(String wikiText, String wikiURL) {
		getPanelControlled().getArtistWikiAbstract().setText(wikiText);
		getPanelControlled().getArtistWikiAbstract().setCaretPosition(0);
		getPanelControlled().getArtistWikiReadMore().setText(LanguageTool.getString("READ_MORE"), wikiURL);
	}

	/**
	 * Delegate method to show all tabs disabled
	 */
	public void showAllTabsDisabled() {
		getPanelControlled().showAllTabs(false);
	}

	public void showFirstTab(String string, ImageIcon icon) {
		getPanelControlled().showFirstTab(string, icon);
	}

	public void showAllTabs(String firstTabTitle, ImageIcon firstTabIcon, boolean enabled) {
		getPanelControlled().showAllTabs(firstTabTitle, firstTabIcon, enabled);
	}

	public void showAllTabs(boolean b) {
		getPanelControlled().showAllTabs(b);
	}

	public void setSelectedIndex(int index) {
		getPanelControlled().getTabbedPane().setSelectedIndex(index);
	}

}
