/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.editTagDialog;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import net.sourceforge.atunes.Constants;
import net.sourceforge.atunes.gui.views.dialogs.EditTagDialog;
import net.sourceforge.atunes.kernel.modules.repository.tags.tag.image.ImageInfo;
import net.sourceforge.atunes.utils.ImageUtils;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

/**
 * The listener interface for receiving editTagDialogAction events.
 */
public class EditTagDialogActionListener implements ActionListener {

	private static Logger logger = new Logger();

	/** The controller. */
	private EditTagDialogController controller;

	/** The dialog. */
	private EditTagDialog dialog;

	/**
	 * Instantiates a new edits the tag dialog action listener.
	 * 
	 * @param controller
	 *            the controller
	 * @param dialog
	 *            the dialog
	 */
	public EditTagDialogActionListener(EditTagDialogController controller, EditTagDialog dialog) {
		this.controller = controller;
		this.dialog = dialog;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == dialog.getOkButton()) {
			controller.editTag();
			controller.setNewCover(null);
		} else if (e.getSource() == dialog.getCancelButton()) {
			dialog.setVisible(false);
			controller.setNewCover(null);
		} else if (e.getSource() == dialog.getCoverButton()) {
			JFileChooser fc = new JFileChooser();
			fc.setFileFilter(new FileFilter() {
				@Override
				public boolean accept(File pathname) {
					if (pathname.isDirectory()) {
						return true;
					}
					String fileName = pathname.getName().toUpperCase();
					return fileName.endsWith("JPG") || fileName.endsWith("JPEG") || fileName.endsWith("PNG");
				}

				@Override
				public String getDescription() {
					return LanguageTool.getString("IMAGES");
				}
			});
			int i = fc.showOpenDialog(dialog);
			if (i == JFileChooser.APPROVE_OPTION) {
				File f = fc.getSelectedFile();
				try {
					BufferedImage bi = ImageIO.read(f);
					controller.setShouldEditCover(true);
					BufferedImage bi2 = ImageUtils.toBufferedImage(ImageUtils.scaleImageBicubic(bi, Constants.DIALOG_IMAGE_WIDTH, Constants.DIALOG_IMAGE_HEIGHT).getImage());
					ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
					ByteArrayOutputStream byteArrayOutputStream2 = new ByteArrayOutputStream();
					ImageInfo ii = new ImageInfo();
					ImageIO.write(bi, ii.getFormatName(), byteArrayOutputStream);
					ImageIO.write(bi2, ii.getFormatName(), byteArrayOutputStream2);
					controller.setNewCover(byteArrayOutputStream.toByteArray());
					dialog.getCover().setIcon(new ImageIcon(byteArrayOutputStream2.toByteArray()));
				} catch (IOException ex) {
					controller.setShouldEditCover(false);
					controller.setNewCover(null);
					logger.error(LogCategories.FILE_READ, ex);
				}
			}
		} else if (e.getSource() == dialog.getRemoveCoverButton()) {
			dialog.getCover().setIcon(null);
			controller.setShouldEditCover(true);
			controller.setNewCover(null);
		}
	}
}
