/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.menu;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.KeyStroke;

import net.sourceforge.atunes.gui.views.menu.ApplicationMenuBar;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.controllers.model.Controller;

/**
 * The Class MenuController.
 * 
 * @author fleax
 */
public class MenuController extends Controller {

	/** The menu. */
	ApplicationMenuBar menu;

	/**
	 * Instantiates a new menu controller.
	 * 
	 * @param menu
	 *            the menu
	 */
	public MenuController(ApplicationMenuBar menu) {
		super();
		this.menu = menu;
		addBindings();
		addStateBindings();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#addBindings()
	 */
	@Override
	protected void addBindings() {
		// Keys

		menu.getFullScreen().setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F11, 0));

		menu.getMute().setAccelerator(KeyStroke.getKeyStroke('m'));

		menu.getPlayListPlay().setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0));
		menu.getPlayListEditTag().setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_INSERT, 0));
		menu.getCustomSearch().setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
		// End Keys

		// Listener
		AppMenuListener listener = new AppMenuListener(menu);
		menu.getShowStatusBar().addActionListener(listener);
		menu.getShowNavigationTable().addActionListener(listener);
		menu.getShowOSD().addActionListener(listener);
		menu.getFullScreen().addActionListener(listener);
		menu.getMute().addActionListener(listener);
		menu.getPlayListPlay().addActionListener(listener);
		menu.getPlayListOpenFolder().addActionListener(listener);
		menu.getPlayListEditTag().addActionListener(listener);
		menu.getPlayListAutoSetTrack().addActionListener(listener);
		menu.getPlayListAutoSetTitle().addActionListener(listener);
		menu.getPlayListAutoSetLyrics().addActionListener(listener);
		menu.getPlayListAutoSetGenre().addActionListener(listener);
		menu.getPlayListShowButtons().addActionListener(listener);
		menu.getDeviceConnect().addActionListener(listener);
		menu.getDeviceRefresh().addActionListener(listener);
		menu.getDeviceDisconnect().addActionListener(listener);
		//menu.getDeviceViewByTag().addActionListener(listener);
		//menu.getDeviceViewByFolder().addActionListener(listener);
		menu.getToolsExport().addActionListener(listener);
		menu.getCoverNavigator().addActionListener(listener);
		menu.getCheckUpdates().addActionListener(listener);
		menu.getReportBugOrRequestFeature().addActionListener(listener);
		menu.getShowLogFile().addActionListener(listener);
		menu.getAboutItem().addActionListener(listener);

		menu.getAdd10RandomSongs().addActionListener(listener);
		menu.getAdd50RandomSongs().addActionListener(listener);
		menu.getAdd100RandomSongs().addActionListener(listener);
		menu.getAdd10SongsMostPlayed().addActionListener(listener);
		menu.getAdd50SongsMostPlayed().addActionListener(listener);
		menu.getAdd100SongsMostPlayed().addActionListener(listener);
		menu.getAdd1AlbumsMostPlayed().addActionListener(listener);
		menu.getAdd5AlbumsMostPlayed().addActionListener(listener);
		menu.getAdd10AlbumsMostPlayed().addActionListener(listener);
		menu.getAdd1ArtistsMostPlayed().addActionListener(listener);
		menu.getAdd5ArtistsMostPlayed().addActionListener(listener);
		menu.getAdd10ArtistsMostPlayed().addActionListener(listener);
		menu.getAdd10UnplayedSongs().addActionListener(listener);
		menu.getAdd50UnplayedSongs().addActionListener(listener);
		menu.getAdd100UnplayedSongs().addActionListener(listener);

		menu.getAddRadio().addActionListener(listener);
		menu.getShowRadioBrowser().addActionListener(listener);
		menu.getAddPodcastFeed().addActionListener(listener);

		menu.getCustomSearch().addActionListener(listener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#addStateBindings
	 * ()
	 */
	@Override
	protected void addStateBindings() {
		disablePlayListItems(true, false);
		menu.getShowOSD().setSelected(Kernel.getInstance().state.isShowOSD());

		setDeviceConnected(false);
	}

	/**
	 * Disable play list items.
	 * 
	 * @param disable
	 *            the disable
	 * @param radioOrPodcastFeedEntrySelected
	 *            the radio or podcast feed entry selected
	 */
	public void disablePlayListItems(boolean disable, boolean radioOrPodcastFeedEntrySelected) {
		menu.getPlayListPlay().setEnabled(!disable);
		menu.getPlayListOpenFolder().setEnabled(!disable && !radioOrPodcastFeedEntrySelected);
		menu.getPlayListEditTag().setEnabled(!disable && !radioOrPodcastFeedEntrySelected);
		menu.getPlayListAutoSetLyrics().setEnabled(!disable && !radioOrPodcastFeedEntrySelected);
		menu.getPlayListAutoSetGenre().setEnabled(!disable && !radioOrPodcastFeedEntrySelected);
		menu.getPlayListAutoSetTitle().setEnabled(!disable && !radioOrPodcastFeedEntrySelected);
		menu.getPlayListAutoSetTrack().setEnabled(!disable && !radioOrPodcastFeedEntrySelected);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.controllers.model.Controller#notifyReload()
	 */
	@Override
	protected void notifyReload() {
		// Nothing to do
	}

	/**
	 * Sets the device connected.
	 * 
	 * @param enable
	 *            the new device connected
	 */
	public void setDeviceConnected(boolean enable) {
		menu.getDeviceConnect().setEnabled(!enable);
		menu.getDeviceRefresh().setEnabled(enable);
		menu.getDeviceDisconnect().setEnabled(enable);
	}

	/**
	 * Sets the show navigation table.
	 * 
	 * @param show
	 *            the new show navigation table
	 */
	public void setShowNavigationTable(boolean show) {
		menu.getShowNavigationTable().setSelected(show);
	}

	/**
	 * Sets the show osd.
	 * 
	 * @param enable
	 *            the new show osd
	 */
	public void setShowOSD(boolean enable) {
		menu.getShowOSD().setSelected(enable);
	}

	/**
	 * Sets the show playlist controls.
	 * 
	 * @param enable
	 *            the new show playlist controls
	 */
	public void setShowPlaylistControls(boolean enable) {
		menu.getPlayListShowButtons().setSelected(enable);
	}

	/**
	 * Sets the show status bar check box value
	 * 
	 * @param show
	 *            the new show status bar
	 */
	public void setShowStatusBar(boolean show) {
		menu.getShowStatusBar().setSelected(show);
	}

}
