/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.navigation;

import javax.swing.JComponent;

import net.sourceforge.atunes.kernel.handlers.RepositoryHandler.SortType;

/**
 * The Class NavigationControllerState.
 */
public class NavigationControllerState {

	/**
	 * The Enum ViewMode.
	 */
	public enum ViewMode {

		/** The ARTIST. */
		ARTIST,

		/** The ALBUM. */
		ALBUM,

		/** The GENRE. */
		GENRE,

		/** The FOLDER. */
		FOLDER
	}

	/** The view mode. */
	private ViewMode viewMode;

	/** The sort type. */
	private SortType sortType;

	/** The current filter. */
	private String currentFilter;

	/** The navigation view. */
	private int navigationView;

	/** The popupmenu caller. */
	private JComponent popupmenuCaller;

	/**
	 * Gets the current filter.
	 * 
	 * @return the current filter
	 */
	public String getCurrentFilter() {
		return currentFilter;
	}

	/**
	 * Gets the navigation view.
	 * 
	 * @return the navigation view
	 */
	public int getNavigationView() {
		return navigationView;
	}

	/**
	 * Gets the popupmenu caller.
	 * 
	 * @return the popupmenu caller
	 */
	public JComponent getPopupmenuCaller() {
		return popupmenuCaller;
	}

	/**
	 * Gets the sort type.
	 * 
	 * @return the sort type
	 */
	public SortType getSortType() {
		return sortType;
	}

	/**
	 * Gets the view mode.
	 * 
	 * @return the view mode
	 */
	public ViewMode getViewMode() {
		return viewMode;
	}

	/**
	 * Sets the current filter.
	 * 
	 * @param currentFilter
	 *            the new current filter
	 */
	public void setCurrentFilter(String currentFilter) {
		this.currentFilter = currentFilter;
	}

	/**
	 * Sets the navigation view.
	 * 
	 * @param view
	 *            the new navigation view
	 */
	public void setNavigationView(int view) {
		navigationView = view;
	}

	/**
	 * Sets the popupmenu caller.
	 * 
	 * @param popupmenuCaller
	 *            the new popupmenu caller
	 */
	public void setPopupmenuCaller(JComponent popupmenuCaller) {
		this.popupmenuCaller = popupmenuCaller;
	}

	/**
	 * Sets the sort type.
	 * 
	 * @param sortType
	 *            the new sort type
	 */
	public void setSortType(SortType sortType) {
		this.sortType = sortType;
	}

	/**
	 * Sets the view mode.
	 * 
	 * @param viewMode
	 *            the new view mode
	 */
	public void setViewMode(ViewMode viewMode) {
		this.viewMode = viewMode;
	}
}
