/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.navigation.listeners;

import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.swing.SwingUtilities;

import net.sourceforge.atunes.gui.views.panels.NavigationPanel;
import net.sourceforge.atunes.kernel.controllers.navigation.NavigationController;

public class NavigationTreeControlsKeyListener extends KeyAdapter {

	/**
	 * Navigation controller
	 */
	NavigationController controller;

	/**
	 * Navigation Panel
	 */
	NavigationPanel panelControlled;

	/**
	 * Constructor
	 * 
	 * @param controller
	 * @param panel
	 */
	public NavigationTreeControlsKeyListener(NavigationController controller, NavigationPanel panel) {
		this.controller = controller;
		this.panelControlled = panel;
	}

	@Override
	public void keyPressed(final KeyEvent e) {
		if (e.getSource() == panelControlled.getFilterTextField()) {
			// Search as user type and clear filter when user pressed ESC
			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
						panelControlled.getFilterTextField().setText("");
						controller.getState().setCurrentFilter(null);
					} else {
						controller.getState().setCurrentFilter(panelControlled.getFilterTextField().getText());
					}
					controller.refreshTreeContent();
				}
			});
		}
	}

}
