/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.executors.processes;

import java.util.List;

import javax.swing.SwingUtilities;

import net.sourceforge.atunes.kernel.handlers.VisualHandler;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.repository.tags.writer.TagModifier;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

/**
 * The Class ClearTagsProcess.
 */
public class ClearTagsProcess implements Runnable {

	/** The logger. */
	private Logger logger = new Logger();

	/** The files to clear. */
	List<AudioFile> filesToClear;

	/**
	 * Instantiates a new clear tags process.
	 * 
	 * @param filesToClear
	 *            the files to clear
	 */
	public ClearTagsProcess(List<AudioFile> filesToClear) {
		super();
		this.filesToClear = filesToClear;
	}

	/**
	 * Removes tags from a list of files.
	 */
	@Override
	public void run() {
		logger.debug(LogCategories.PROCESS);

		VisualHandler.getInstance().showIndeterminateProgressDialog(StringUtils.getString(LanguageTool.getString("PERFORMING_CHANGES"), "..."));

		for (AudioFile file : filesToClear) {
			TagModifier.deleteTags(file);
		}

		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				TagModifier.refreshAfterTagModify(filesToClear);
			}
		});
		VisualHandler.getInstance().hideIndeterminateProgressDialog();
	}

}
