/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.executors.processes;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.SwingUtilities;

import net.sourceforge.atunes.kernel.handlers.ContextInformationServiceHandler;
import net.sourceforge.atunes.kernel.handlers.VisualHandler;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.repository.tags.writer.TagModifier;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

/**
 * The Class SetGenresProcess.
 */
public class SetGenresProcess implements Runnable {

	/** The logger. */
	private Logger logger = new Logger();

	/** The files. */
	private List<AudioFile> files;

	/**
	 * Instantiates a new sets the genres process.
	 * 
	 * @param files
	 *            the files
	 */
	public SetGenresProcess(List<AudioFile> files) {
		super();
		this.files = files;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Runnable#run()
	 */

	@Override
	public void run() {
		logger.debug(LogCategories.PROCESS);
		VisualHandler.getInstance().showIndeterminateProgressDialog(StringUtils.getString(LanguageTool.getString("PERFORMING_CHANGES"), "..."));
		Map<AudioFile, String> filesAndGenres = ContextInformationServiceHandler.getInstance().getGenresForFiles(files);
		if (!filesAndGenres.isEmpty()) {
			final List<AudioFile> filesToEdit = new ArrayList<AudioFile>();
			logger.info(LogCategories.PROCESS, StringUtils.getString("Changing Genre to ", filesAndGenres.size(), " Songs"));
			for (Map.Entry<AudioFile, String> entry : filesAndGenres.entrySet()) {
				String genre = entry.getValue();
				AudioFile file = entry.getKey();
				// If file has already genre setted, avoid
				if (!file.getGenre().equals(genre)) {
					TagModifier.setGenre(file, genre);
					filesToEdit.add(file);
				}
			}
			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					TagModifier.refreshAfterTagModify(filesToEdit);
				}
			});
		}
		VisualHandler.getInstance().hideIndeterminateProgressDialog();
	}
}
