/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.handlers;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.filechooser.FileFilter;

import net.sourceforge.atunes.Constants;
import net.sourceforge.atunes.gui.Frame;
import net.sourceforge.atunes.gui.GuiUtils;
import net.sourceforge.atunes.gui.MultipleFrame;
import net.sourceforge.atunes.gui.StandardFrame;
import net.sourceforge.atunes.gui.ToolBar;
import net.sourceforge.atunes.gui.model.PlayListTableModel;
import net.sourceforge.atunes.gui.popup.FadingPopupFactory;
import net.sourceforge.atunes.gui.views.controls.playList.PlayListColumnSelector;
import net.sourceforge.atunes.gui.views.controls.playList.PlayListTable;
import net.sourceforge.atunes.gui.views.controls.playList.PlayListTable.PlayState;
import net.sourceforge.atunes.gui.views.dialogs.AboutDialog;
import net.sourceforge.atunes.gui.views.dialogs.AddPodcastFeedDialog;
import net.sourceforge.atunes.gui.views.dialogs.AddRadioDialog;
import net.sourceforge.atunes.gui.views.dialogs.CopyProgressDialog;
import net.sourceforge.atunes.gui.views.dialogs.CoverNavigatorFrame;
import net.sourceforge.atunes.gui.views.dialogs.CustomSearchDialog;
import net.sourceforge.atunes.gui.views.dialogs.EditTagDialog;
import net.sourceforge.atunes.gui.views.dialogs.EditTitlesDialog;
import net.sourceforge.atunes.gui.views.dialogs.EqualizerDialog;
import net.sourceforge.atunes.gui.views.dialogs.ExportOptionsDialog;
import net.sourceforge.atunes.gui.views.dialogs.FileSelectionDialog;
import net.sourceforge.atunes.gui.views.dialogs.ImageDialog;
import net.sourceforge.atunes.gui.views.dialogs.IndeterminateProgressDialog;
import net.sourceforge.atunes.gui.views.dialogs.InputDialog;
import net.sourceforge.atunes.gui.views.dialogs.MultiFolderSelectionDialog;
import net.sourceforge.atunes.gui.views.dialogs.OSDDialog;
import net.sourceforge.atunes.gui.views.dialogs.PodcastFeedEntryDownloadProgressDialog;
import net.sourceforge.atunes.gui.views.dialogs.RadioBrowserDialog;
import net.sourceforge.atunes.gui.views.dialogs.RepositoryProgressDialog;
import net.sourceforge.atunes.gui.views.dialogs.RepositorySelectionInfoDialog;
import net.sourceforge.atunes.gui.views.dialogs.RipCdDialog;
import net.sourceforge.atunes.gui.views.dialogs.RipperProgressDialog;
import net.sourceforge.atunes.gui.views.dialogs.SearchDialog;
import net.sourceforge.atunes.gui.views.dialogs.SearchResultsDialog;
import net.sourceforge.atunes.gui.views.dialogs.SplashScreenDialog;
import net.sourceforge.atunes.gui.views.dialogs.StatsDialog;
import net.sourceforge.atunes.gui.views.dialogs.editPreferences.EditPreferencesDialog;
import net.sourceforge.atunes.gui.views.dialogs.fullScreen.FullScreenDialog;
import net.sourceforge.atunes.gui.views.dialogs.properties.PropertiesDialog;
import net.sourceforge.atunes.gui.views.menu.ApplicationMenuBar;
import net.sourceforge.atunes.gui.views.panels.ContextInformationPanel;
import net.sourceforge.atunes.gui.views.panels.FilePropertiesPanel;
import net.sourceforge.atunes.gui.views.panels.NavigationPanel;
import net.sourceforge.atunes.gui.views.panels.PlayListPanel;
import net.sourceforge.atunes.gui.views.panels.PlayerControlsPanel;
import net.sourceforge.atunes.kernel.ApplicationFinishListener;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.actions.Actions;
import net.sourceforge.atunes.kernel.actions.ShowToolbarAction;
import net.sourceforge.atunes.kernel.controllers.coverNavigator.CoverNavigatorController;
import net.sourceforge.atunes.kernel.modules.playlist.PlayList;
import net.sourceforge.atunes.kernel.modules.podcast.PodcastFeed;
import net.sourceforge.atunes.kernel.modules.podcast.PodcastFeedEntry;
import net.sourceforge.atunes.kernel.modules.radio.Radio;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.utils.SystemProperties;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.TimeUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

/**
 * The Class VisualHandler.
 */
public class VisualHandler implements ApplicationFinishListener {

	/** The instance. */
	private static VisualHandler instance = new VisualHandler();

	/** Logger. */
	Logger logger = new Logger();

	/** Reference to the Kernel. */
	Kernel kernel;

	/** Frames. */
	Frame frame;

	/** Dialogs. */
	private OSDDialog osdDialog;

	/** The progress dialog. */
	private RepositoryProgressDialog progressDialog;

	/** The edit tag dialog. */
	private EditTagDialog editTagDialog;

	/** The export dialog. */
	private ExportOptionsDialog exportDialog;

	/** The stats dialog. */
	private StatsDialog statsDialog;

	/** The export progress dialog. */
	private CopyProgressDialog exportProgressDialog;

	/** The copy progress dialog. */
	private CopyProgressDialog copyProgressDialog;

	/** The search dialog. */
	private SearchDialog searchDialog;

	/** The rip cd dialog. */
	private RipCdDialog ripCdDialog;

	/** The ripper progress dialog. */
	private RipperProgressDialog ripperProgressDialog;

	/** The indeterminate progress dialog. */
	private IndeterminateProgressDialog indeterminateProgressDialog;

	/** The edit titles dialog. */
	private EditTitlesDialog editTitlesDialog;

	/** The edit preferences dialog. */
	private EditPreferencesDialog editPreferencesDialog;

	/** The equalizer dialog. */
	private EqualizerDialog equalizerDialog;

	/** The about dialog. */
	private AboutDialog aboutDialog;

	/** The splash screen dialog. */
	private SplashScreenDialog splashScreenDialog;

	/** The custom search dialog. */
	private CustomSearchDialog customSearchDialog;

	/** The search results dialog. */
	private SearchResultsDialog searchResultsDialog;

	/** The radio browser dialog. */
	private RadioBrowserDialog radioBrowserDialog;

	/** The full screen frame. */
	private FullScreenDialog fullScreenFrame;

	/** The full frame state listener. */
	private WindowAdapter fullFrameStateListener;

	/**
	 * Instantiates a new visual handler.
	 */
	private VisualHandler() {
		this.kernel = Kernel.getInstance();
		if (!kernel.state.isMultipleWindow()) {
			frame = new StandardFrame();
		} else {
			frame = new MultipleFrame();
		}
		Kernel.getInstance().addFinishListener(this);
	}

	/**
	 * Gets the single instance of VisualHandler.
	 * 
	 * @return single instance of VisualHandler
	 */
	public static VisualHandler getInstance() {
		return instance;
	}

	/**
	 * Finish.
	 */
	public void finish() {
		if (!kernel.state.isShowSystemTray()) {
			kernel.finish();
		}
	}

	/**
	 * Gets the context information panel.
	 * 
	 * @return the context information panel
	 */
	public ContextInformationPanel getContextInformationPanel() {
		return frame.getContextInformationPanel();
	}

	/**
	 * Gets the copy progress dialog.
	 * 
	 * @return the copy progress dialog
	 */
	public CopyProgressDialog getCopyProgressDialog() {
		if (copyProgressDialog == null) {
			copyProgressDialog = new CopyProgressDialog(frame.getFrame(), StringUtils.getString(LanguageTool.getString("COPYING"), "..."));
		}
		return copyProgressDialog;
	}

	/**
	 * Gets the edits the preferences dialog.
	 * 
	 * @return the edits the preferences dialog
	 */
	public EditPreferencesDialog getEditPreferencesDialog() {
		if (editPreferencesDialog == null) {
			editPreferencesDialog = new EditPreferencesDialog(frame.getFrame());
		}
		return editPreferencesDialog;
	}

	/**
	 * Gets the edits the tag dialog.
	 * 
	 * @return the edits the tag dialog
	 */
	public EditTagDialog getEditTagDialog() {
		if (editTagDialog == null) {
			editTagDialog = new EditTagDialog(frame.getFrame());
		}
		return editTagDialog;
	}

	/**
	 * Gets the edits the titles dialog.
	 * 
	 * @return the edits the titles dialog
	 */
	public EditTitlesDialog getEditTitlesDialog() {
		if (editTitlesDialog == null) {
			editTitlesDialog = new EditTitlesDialog(frame.getFrame());
		}
		return editTitlesDialog;
	}

	/**
	 * Gets the equalizer dialog.
	 * 
	 * @return the equalizer dialog
	 */
	public EqualizerDialog getEqualizerDialog() {
		if (equalizerDialog == null) {
			equalizerDialog = new EqualizerDialog(frame.getFrame());
		}
		return equalizerDialog;
	}

	/**
	 * Gets the export dialog.
	 * 
	 * @return the export dialog
	 */
	public ExportOptionsDialog getExportDialog() {
		if (exportDialog == null) {
			exportDialog = new ExportOptionsDialog(frame.getFrame());
		}
		return exportDialog;
	}

	/**
	 * Gets the export progress dialog.
	 * 
	 * @return the export progress dialog
	 */
	public CopyProgressDialog getExportProgressDialog() {
		if (exportProgressDialog == null) {
			exportProgressDialog = new CopyProgressDialog(frame.getFrame(), StringUtils.getString(LanguageTool.getString("EXPORTING"), "..."));
		}
		return exportProgressDialog;
	}

	/**
	 * Gets the file selection dialog.
	 * 
	 * @param dirOnly
	 *            the dir only
	 * 
	 * @return the file selection dialog
	 */
	public FileSelectionDialog getFileSelectionDialog(boolean dirOnly) {
		return new FileSelectionDialog(frame.getFrame(), dirOnly);
	}

	/**
	 * Gets the frame.
	 * 
	 * @return the frame
	 */
	public Frame getFrame() {
		return frame;
	}

	/**
	 * Gets the full screen frame.
	 * 
	 * @return the fullScreenFrame
	 */
	public FullScreenDialog getFullScreenFrame() {
		if (fullScreenFrame == null) {
			JDialog.setDefaultLookAndFeelDecorated(false);
			fullScreenFrame = new FullScreenDialog(frame.getFrame());
			JDialog.setDefaultLookAndFeelDecorated(true);
		}
		return fullScreenFrame;
	}

	/**
	 * Gets the indeterminate progress dialog.
	 * 
	 * @return the indeterminate progress dialog
	 */
	IndeterminateProgressDialog getIndeterminateProgressDialog() {
		if (indeterminateProgressDialog == null) {
			indeterminateProgressDialog = new IndeterminateProgressDialog(frame.getFrame());
		}
		return indeterminateProgressDialog;
	}

	/**
	 * Gets the menu bar.
	 * 
	 * @return the menu bar
	 */
	public ApplicationMenuBar getMenuBar() {
		return frame.getAppMenuBar();
	}

	/**
	 * Gets the multi folder selection dialog.
	 * 
	 * @return the multi folder selection dialog
	 */
	public MultiFolderSelectionDialog getMultiFolderSelectionDialog() {
		return new MultiFolderSelectionDialog(frame.getFrame());
	}

	/**
	 * Gets the navigation panel.
	 * 
	 * @return the navigation panel
	 */
	public NavigationPanel getNavigationPanel() {
		return frame.getNavigationPanel();
	}

	/**
	 * Gets the new podcast feed entry download progress dialog.
	 * 
	 * @param title
	 *            the title
	 * 
	 * @return the new podcast feed entry download progress dialog
	 */
	public PodcastFeedEntryDownloadProgressDialog getNewPodcastFeedEntryDownloadProgressDialog(String title) {
		return new PodcastFeedEntryDownloadProgressDialog(title);
	}

	/**
	 * Gets the oSD dialog.
	 * 
	 * @return the oSD dialog
	 */
	public OSDDialog getOSDDialog() {
		if (osdDialog == null) {
			JDialog.setDefaultLookAndFeelDecorated(false);
			osdDialog = new OSDDialog();
			JDialog.setDefaultLookAndFeelDecorated(true);
		}
		return osdDialog;
	}

	/**
	 * Gets the player controls.
	 * 
	 * @return the player controls
	 */
	public PlayerControlsPanel getPlayerControls() {
		return frame.getPlayerControls();
	}

	/**
	 * Gets the play list column selector.
	 * 
	 * @return the play list column selector
	 */
	public PlayListColumnSelector getPlayListColumnSelector() {
		return new PlayListColumnSelector(frame.getFrame());
	}

	/**
	 * Gets the play list panel.
	 * 
	 * @return the play list panel
	 */
	public PlayListPanel getPlayListPanel() {
		return frame.getPlayListPanel();
	}

	/**
	 * Gets the play list table.
	 * 
	 * @return the play list table
	 */
	public PlayListTable getPlayListTable() {
		return frame.getPlayListTable();
	}

	/**
	 * Gets the play list table model.
	 * 
	 * @return the play list table model
	 */
	public PlayListTableModel getPlayListTableModel() {
		return (PlayListTableModel) getPlayListTable().getModel();
	}

	/**
	 * Gets the progress dialog.
	 * 
	 * @return the progress dialog
	 */
	public RepositoryProgressDialog getProgressDialog() {
		if (progressDialog == null) {
			progressDialog = new RepositoryProgressDialog(frame.getFrame());
			progressDialog.getCancelButton().addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					RepositoryHandler.getInstance().notifyCancel();
				}
			});
		}
		return progressDialog;
	}

	/**
	 * Gets the properties panel.
	 * 
	 * @return the properties panel
	 */
	public FilePropertiesPanel getPropertiesPanel() {
		return frame.getPropertiesPanel();
	}

	/**
	 * Gets the radio browser dialog.
	 * 
	 * @return the radio browser dialog
	 */
	public RadioBrowserDialog getRadioBrowserDialog() {
		if (radioBrowserDialog == null) {
			radioBrowserDialog = new RadioBrowserDialog();
		}
		return radioBrowserDialog;
	}

	/**
	 * Gets the rip cd dialog.
	 * 
	 * @return the rip cd dialog
	 */
	public RipCdDialog getRipCdDialog() {
		if (ripCdDialog == null) {
			ripCdDialog = new RipCdDialog(frame.getFrame());
		}
		return ripCdDialog;
	}

	/**
	 * Gets the ripper progress dialog.
	 * 
	 * @return the ripper progress dialog
	 */
	public RipperProgressDialog getRipperProgressDialog() {
		if (ripperProgressDialog == null) {
			ripperProgressDialog = new RipperProgressDialog();
			ripperProgressDialog.getCancelButton().addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					RipperHandler.getInstance().cancelProcess();
				}
			});
		}
		return ripperProgressDialog;
	}

	/**
	 * Gets the search dialog.
	 * 
	 * @return the search dialog
	 */
	public SearchDialog getSearchDialog() {
		if (searchDialog == null) {
			searchDialog = new SearchDialog(frame.getFrame());
		}
		return searchDialog;
	}

	/**
	 * Gets the standard window listener.
	 * 
	 * @return the standard window listener
	 */
	public ComponentListener getStandardWindowListener() {
		final int minHeight = 410;
		return new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent e) {
				JFrame f = frame.getFrame();

				if (frame.getPlayListPanel().getSize().width < StandardFrame.PLAY_LIST_PANEL_WIDTH) {
					f.setSize(f.getWidth() + (StandardFrame.PLAY_LIST_PANEL_WIDTH - frame.getPlayListPanel().getSize().width + 10), f.getHeight());
				}

				if (f.getHeight() < minHeight) {
					f.setSize(f.getWidth(), minHeight);
				}
			}
		};
	}

	/**
	 * Gets the stats dialog.
	 * 
	 * @return the stats dialog
	 */
	public StatsDialog getStatsDialog() {
		if (statsDialog == null) {
			statsDialog = new StatsDialog();
		}
		return statsDialog;
	}

	/**
	 * Gets the tool bar.
	 * 
	 * @return the tool bar
	 */
	public ToolBar getToolBar() {
		return frame.getToolBar();
	}

	/**
	 * Gets the window location.
	 * 
	 * @return the window location
	 */
	public Point getWindowLocation() {
		return frame.getLocation();
	}

	/**
	 * Gets the window size.
	 * 
	 * @return the window size
	 */
	public Dimension getWindowSize() {
		return frame.getSize();
	}

	/**
	 * Gets the window state listener.
	 * 
	 * @return the window state listener
	 */
	public WindowAdapter getWindowStateListener() {
		if (fullFrameStateListener == null) {
			fullFrameStateListener = new WindowAdapter() {
				@Override
				public void windowGainedFocus(WindowEvent e) {
					logger.debug(LogCategories.DESKTOP, "Window Gained Lost");
				}

				@Override
				public void windowLostFocus(WindowEvent e) {
					logger.debug(LogCategories.DESKTOP, "Window Focus Lost");
				}

				@Override
				public void windowStateChanged(WindowEvent e) {
					if ((e.getNewState() & java.awt.Frame.ICONIFIED) == java.awt.Frame.ICONIFIED) {
						if (kernel.state.isShowSystemTray()) {
							frame.setVisible(false);
						}
						logger.debug(LogCategories.DESKTOP, "Window Iconified");
					} else if ((e.getNewState() & java.awt.Frame.NORMAL) == java.awt.Frame.NORMAL) {
						logger.debug(LogCategories.DESKTOP, "Window Deiconified");
						ControllerProxy.getInstance().getPlayListController().scrollPlayList();
					}
				}
			};
		}
		return fullFrameStateListener;
	}

	/**
	 * Hide indeterminate progress dialog.
	 */
	public void hideIndeterminateProgressDialog() {
		try {
			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					getIndeterminateProgressDialog().setVisible(false);
				}
			});
		} catch (Exception e) {
			logger.internalError(e);
		}
	}

	/**
	 * Hide splash screen.
	 */
	public void hideSplashScreen() {
		if (splashScreenDialog != null) {
			splashScreenDialog.setVisible(false);
			splashScreenDialog.dispose();
			splashScreenDialog = null;
		}
	}

	@Override
	public void applicationFinish() {
		Window[] windows = Window.getWindows();
		for (Window window : windows) {
			window.setVisible(false);
		}
	}

	/**
	 * Checks if is maximized.
	 * 
	 * @return true, if is maximized
	 */
	public boolean isMaximized() {
		return frame.getExtendedState() == java.awt.Frame.MAXIMIZED_BOTH;
	}

	/**
	 * Checks if is multiple window.
	 * 
	 * @return true, if is multiple window
	 */
	public boolean isMultipleWindow() {
		return frame instanceof MultipleFrame;
	}

	/**
	 * Repaint.
	 */
	public void repaint() {
		frame.getFrame().invalidate();
		frame.getFrame().validate();
		frame.getFrame().repaint();
	}

	/**
	 * Sets the center status bar text.
	 * 
	 * @param text
	 *            the text
	 * @param toolTip
	 *            the tool tip
	 */
	public void setCenterStatusBarText(String text, String toolTip) {
		frame.setCenterStatusBar(text, toolTip);
	}

	/**
	 * Sets the frame default close operation.
	 * 
	 * @param op
	 *            the new frame default close operation
	 */
	public void setFrameDefaultCloseOperation(int op) {
		frame.setDefaultCloseOperation(op);
	}

	/**
	 * Sets the full frame extended state.
	 * 
	 * @param state
	 *            the new full frame extended state
	 */
	public void setFullFrameExtendedState(int state) {
		frame.setExtendedState(state);
	}

	/**
	 * Sets the full frame location.
	 * 
	 * @param location
	 *            the new full frame location
	 */
	public void setFullFrameLocation(Point location) {
		frame.setLocation(location);
	}

	/**
	 * Sets the full frame location relative to.
	 * 
	 * @param c
	 *            the new full frame location relative to
	 */
	public void setFullFrameLocationRelativeTo(Component c) {
		frame.setLocationRelativeTo(c);
	}

	/**
	 * Attribute to call setWindowSize of StandardFram on first call to
	 * setFullFrameVisible This workaround fixes setExtendedState bug in Linux
	 * systems:
	 * 
	 * See http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6365898
	 */
	private boolean firstTimeShow = true;

	/**
	 * Sets the full frame visible.
	 * 
	 * @param visible
	 *            the new full frame visible
	 */
	public void setFullFrameVisible(boolean visible) {
		frame.setVisible(visible);

		// Workaround fot setExtendedState bug: setExtendedState must be called after
		// setVisible in Linux systems
		if (firstTimeShow && frame instanceof StandardFrame) {
			((StandardFrame) frame).setWindowSize();
		}
		firstTimeShow = false;
	}

	/**
	 * Sets the left status bar text.
	 * 
	 * @param text
	 *            the text
	 * @param toolTip
	 *            the tool tip
	 */
	public void setLeftStatusBarText(String text, String toolTip) {
		frame.setLeftStatusBarText(text, toolTip);
	}

	/**
	 * Sets the playing.
	 * 
	 * @param playing
	 *            the new playing
	 */
	public void setPlaying(boolean playing) {
		ControllerProxy.getInstance().getPlayerControlsController().setPlaying(playing);
		VisualHandler.getInstance().getFullScreenFrame().setPlaying(playing);
		SystemTrayHandler.getInstance().setPlaying(playing);
	}

	/**
	 * Sets the right status bar text.
	 * 
	 * @param text
	 *            the text
	 * @param toolTip
	 *            the tool tip
	 */
	public void setRightStatusBarText(String text, String toolTip) {
		frame.setRightStatusBar(text, toolTip);
	}

	/**
	 * Sets title bar text, adding app name and version.
	 * 
	 * @param text
	 *            the text
	 */
	public void setTitleBar(String text) {
		StringBuilder strBuilder = new StringBuilder();
		if (!text.equals("")) {
			strBuilder.append(text);
			strBuilder.append(" - ");
		}
		strBuilder.append(Constants.APP_NAME);
		strBuilder.append(" ");
		strBuilder.append(Constants.VERSION.toShortString());

		String result = strBuilder.toString();

		frame.setTitle(result);
		SystemTrayHandler.getInstance().setTrayToolTip(result);
	}

	/**
	 * Sets the volume.
	 * 
	 * @param volume
	 *            the new volume
	 */
	public void setVolume(int volume) {
		frame.getPlayerControls().getVolumeSlider().setValue(volume);
	}

	/**
	 * Show about dialog.
	 */
	public void showAboutDialog() {
		if (aboutDialog == null) {
			aboutDialog = new AboutDialog(frame.getFrame());
		}
		aboutDialog.setVisible(true);
	}

	/**
	 * Gets the custom search dialog.
	 * 
	 * @return the custom search dialog
	 */
	public CustomSearchDialog getCustomSearchDialog() {
		if (customSearchDialog == null) {
			customSearchDialog = new CustomSearchDialog(frame.getFrame());
		}
		return customSearchDialog;
	}

	/**
	 * Gets the search results dialog.
	 * 
	 * @return the search results dialog
	 */
	public SearchResultsDialog getSearchResultsDialog() {
		if (searchResultsDialog == null) {
			searchResultsDialog = new SearchResultsDialog(frame.getFrame());
		}
		return searchResultsDialog;
	}

	/**
	 * Show add podcast feed dialog.
	 * 
	 * @return the podcast feed
	 */
	public PodcastFeed showAddPodcastFeedDialog() {
		AddPodcastFeedDialog dialog = new AddPodcastFeedDialog(frame.getFrame());
		dialog.setVisible(true);
		return dialog.getPodcastFeed();
	}

	/**
	 * Show add radio dialog.
	 * 
	 * @return the radio
	 */
	public Radio showAddRadioDialog() {
		AddRadioDialog dialog = new AddRadioDialog(frame.getFrame());
		dialog.setVisible(true);
		return dialog.getRadio();
	}

	/**
	 * Show audio scrobbler panel.
	 * 
	 * @param show
	 *            the show
	 * @param changeSize
	 *            the change size
	 */
	public void showContextInformationPanel(boolean show, boolean changeSize) {
		// If show value is the same, do nothing 
		if (kernel.state.isUseContextInformation() == show) {
			return;
		}

		kernel.state.setUseContextInformation(show);
		frame.showContextInformationPanel(show, changeSize);
		if (show) {
			ContextInformationServiceHandler.getInstance().retrieveInfoAndShowInPanel(MultiplePlaylistHandler.getInstance().getCurrentPlayList().getCurrentAudioObject());
		}
	}

	/**
	 * Show confirmation dialog.
	 * 
	 * @param message
	 *            the message
	 * @param title
	 *            the title
	 * 
	 * @return the int
	 */
	public int showConfirmationDialog(String message, String title) {
		return JOptionPane.showConfirmDialog(frame.getFrame(), message, title, JOptionPane.YES_NO_OPTION);
	}

	/**
	 * Show cover navigator.
	 */
	public void showCoverNavigator() {
		CoverNavigatorFrame coverNavigator = new CoverNavigatorFrame(RepositoryHandler.getInstance().getArtists());
		new CoverNavigatorController(coverNavigator);
		coverNavigator.setVisible(true);
	}

	/**
	 * Show error dialog.
	 * 
	 * @param message
	 *            the message
	 */
	public void showErrorDialog(String message) {
		JOptionPane.showMessageDialog(frame.getFrame(), message, LanguageTool.getString("ERROR"), JOptionPane.ERROR_MESSAGE);
	}

	/**
	 * Show error dialog.
	 * 
	 * @param message
	 *            the message
	 * @param parent
	 *            the parent
	 */
	public void showErrorDialog(String message, Component parent) {
		JOptionPane.showMessageDialog(parent, message, LanguageTool.getString("ERROR"), JOptionPane.ERROR_MESSAGE);
	}

	/**
	 * Switches playlist filter on and off. It will just show (if hidden) or
	 * hide (if visible) the filter each time called.
	 * 
	 */
	public void togglePlayListFiler() {
		if (frame.getPlayListPanel().getPlayListFilter().isVisible()) {
			frame.getPlayListPanel().getPlayListFilter().setVisible(false);
		} else {
			frame.getPlayListPanel().getPlayListFilter().setVisible(true);
		}
		// If show, then set focus to text field
		if (frame.getPlayListPanel().getPlayListFilter().isVisible()) {
			frame.getPlayListPanel().getPlayListFilter().setFocusToTextField();
		}
	}

	/**
	 * Show icon on status bar.
	 * 
	 * @param img
	 *            the img
	 * @param text
	 *            the text
	 */
	public void showDeviceInfoOnStatusBar(String text) {
		frame.setStatusBarDeviceLabelText(text);
		frame.showStatusBarDeviceLabel(true);
	}

	public void hideDeviceInfoOnStatusBar() {
		frame.showStatusBarDeviceLabel(false);
	}

	public void showNewPodcastFeedEntriesInfo() {
		frame.showStatusBarNewPodcastFeedEntriesLabel(true);

		if (isMultipleWindow() || !Kernel.getInstance().state.isShowStatusBar()) {
			showMessage(LanguageTool.getString("NEW_PODCAST_ENTRIES"));
		} else {
			frame.showStatusBarNewPodcastFeedEntriesLabel(true);
		}
	}

	/**
	 * Show image dialog.
	 * 
	 * @param file
	 *            the file
	 */
	public void showImageDialog(AudioObject file) {
		new ImageDialog(frame.getFrame(), file);
	}

	/**
	 * Show indeterminate progress dialog.
	 * 
	 * @param text
	 *            the text
	 */
	public void showIndeterminateProgressDialog(final String text) {
		try {
			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					getIndeterminateProgressDialog().setTitle(text);
					getIndeterminateProgressDialog().setVisible(true);
				}
			});
		} catch (Exception e) {
			logger.internalError(e);
		}
	}

	/**
	 * Show info.
	 */
	public void showInfo() {
		PlayListTable table = getPlayListTable();
		int[] rows = table.getSelectedRows();
		if (rows.length == 1) {
			AudioObject file = ((PlayListTableModel) table.getModel()).getFileAt(rows[0]);
			showPropertiesDialog(file);
		}
	}

	/**
	 * Show input dialog.
	 * 
	 * @param title
	 *            the title
	 * @param text
	 *            the text
	 * 
	 * @return the string
	 */
	public String showInputDialog(String title, String text) {
		return showInputDialog(title, text, null);
	}

	/**
	 * Show input dialog.
	 * 
	 * @param title
	 *            the title
	 * @param text
	 *            the text
	 * @param icon
	 *            the icon
	 * 
	 * @return the string
	 */
	public String showInputDialog(String title, String text, Image icon) {
		InputDialog id = new InputDialog(frame.getFrame(), title, icon);
		id.show(text);
		return id.getResult();
	}

	/**
	 * Show message.
	 * 
	 * @param message
	 *            the message
	 */
	public void showMessage(String message) {
		JOptionPane.showMessageDialog(frame.getFrame(), message);
	}

	/**
	 * Show navigation panel.
	 * 
	 * @param show
	 *            the show
	 * @param changeSize
	 *            the change size
	 */
	public void showNavigationPanel(boolean show, boolean changeSize) {
		kernel.state.setShowNavigationPanel(show);
		frame.showNavigationPanel(show, changeSize);
	}

	/**
	 * Show navigation table.
	 * 
	 * @param show
	 *            the show
	 */
	public void showNavigationTable(boolean show) {
		kernel.state.setShowNavigationTable(show);
		frame.showNavigationTable(show);
	}

	/**
	 * Show open dialog.
	 * 
	 * @param fileChooser
	 *            the file chooser
	 * @param filter
	 *            the filter
	 * 
	 * @return the int
	 */
	public int showOpenDialog(JFileChooser fileChooser, FileFilter filter) {
		fileChooser.setFileFilter(filter);
		return fileChooser.showOpenDialog(frame.getFrame());
	}

	/**
	 * Show osd dialog.
	 */
	public void showOSDDialog() {
		ControllerProxy.getInstance().getOSDDialogController().showOSD(MultiplePlaylistHandler.getInstance().getCurrentPlayList().getCurrentAudioObject());
	}

	/**
	 * Show play list information.
	 * 
	 * @param playList
	 *            the play list
	 */

	public void showPlayListInformation(PlayList playList) {
		int audioFiles = playList.getNumberOfAudioFiles();
		int radios = playList.getNumberOfRadios();
		int podcastFeedEntries = playList.getNumberOfPodcastFeedEntries();
		int audioObjects = playList.size();

		Object[] strs = new Object[20];
		strs[0] = LanguageTool.getString("PLAYLIST");
		strs[1] = ": ";
		strs[2] = audioObjects;
		strs[3] = " ";
		strs[4] = LanguageTool.getString("SONGS");
		strs[5] = " (";
		strs[6] = playList.getLength();
		strs[7] = ") ";
		strs[8] = " - ";
		strs[9] = audioFiles;
		strs[10] = " ";
		strs[11] = LanguageTool.getString("SONGS");
		strs[12] = " / ";
		strs[13] = radios;
		strs[14] = " ";
		strs[15] = LanguageTool.getString("RADIOS");
		strs[16] = " / ";
		strs[17] = podcastFeedEntries;
		strs[18] = " ";
		// Check if differenciation is required (needed by some slavic languages)
		if (LanguageTool.getString("PODCAST_ENTRIES_COUNTER").isEmpty()) {
			strs[19] = LanguageTool.getString("PODCAST_ENTRIES");
		} else {
			strs[19] = LanguageTool.getString("PODCAST_ENTRIES_COUNTER");
		}

		Object[] strs2 = new Object[9];
		strs2[0] = LanguageTool.getString("PLAYLIST");
		strs2[1] = ": ";
		strs2[2] = audioObjects;
		strs2[3] = " - ";
		strs2[4] = audioFiles;
		strs2[5] = " / ";
		strs2[6] = radios;
		strs2[7] = " / ";
		strs2[8] = podcastFeedEntries;

		String toolTip = StringUtils.getString(strs);
		String text = StringUtils.getString(strs2);
		setRightStatusBarText(text, toolTip);
	}

	/**
	 * Show progress bar.
	 * 
	 * @param show
	 *            the show
	 * @param tooltip
	 *            the tooltip
	 */
	public void showProgressBar(boolean show, String tooltip) {
		frame.showProgressBar(show, tooltip);
	}

	/**
	 * Show properties dialog.
	 * 
	 * @param file
	 *            the file
	 */
	public void showPropertiesDialog(AudioObject file) {
		PropertiesDialog dialog = PropertiesDialog.newInstance(file);
		dialog.setVisible(true);
	}

	/**
	 * Show repository selection info dialog.
	 */
	public void showRepositorySelectionInfoDialog() {
		new RepositorySelectionInfoDialog(frame.getFrame()).setVisible(true);
	}

	//TODO RTL component orientation
	/**
	 * Show repository song number.
	 * 
	 * @param size
	 *            the size
	 * @param sizeInBytes
	 *            the size in bytes
	 * @param duration
	 *            the duration
	 */

	public void showRepositorySongNumber(long size, long sizeInBytes, long duration) {
		// Check if differenciation is required (needed by some slavic languages)
		if (LanguageTool.getString("SONGS_IN_REPOSITORY").isEmpty()) {
			String text = StringUtils.getString(LanguageTool.getString("REPOSITORY"), ": ", size, " ", LanguageTool.getString("SONGS"));
			String toolTip = StringUtils.getString(LanguageTool.getString("REPOSITORY"), ": ", size, " ", LanguageTool.getString("SONGS"), " - ", StringUtils
					.fromByteToMegaOrGiga(sizeInBytes), " - ", StringUtils.fromSecondsToHoursAndDays(duration));
			setCenterStatusBarText(text, toolTip);
		} else {
			String text = StringUtils.getString(LanguageTool.getString("REPOSITORY"), ": ", size, " ", LanguageTool.getString("SONGS_IN_REPOSITORY"));
			String toolTip = StringUtils.getString(LanguageTool.getString("REPOSITORY"), ": ", size, " ", LanguageTool.getString("SONGS_IN_REPOSITORY"), " - ", StringUtils
					.fromByteToMegaOrGiga(sizeInBytes), " - ", StringUtils.fromSecondsToHoursAndDays(duration));
			setCenterStatusBarText(text, toolTip);
		}
	}

	/**
	 * Show save dialog.
	 * 
	 * @param fileChooser
	 *            the file chooser
	 * @param filter
	 *            the filter
	 * 
	 * @return the int
	 */
	public int showSaveDialog(JFileChooser fileChooser, FileFilter filter) {
		fileChooser.setFileFilter(filter);
		return fileChooser.showSaveDialog(frame.getFrame());
	}

	/**
	 * Show song properties.
	 * 
	 * @param show
	 *            the show
	 * @param update
	 *            the update
	 */
	public void showSongProperties(boolean show, boolean update) {
		kernel.state.setShowSongProperties(show);
		frame.showSongProperties(show);
		if (show && update) {
			if (MultiplePlaylistHandler.getInstance().getCurrentPlayList().getCurrentAudioObject() != null) {
				ControllerProxy.getInstance().getFilePropertiesController().updateValues(MultiplePlaylistHandler.getInstance().getCurrentPlayList().getCurrentAudioObject());
			} else {
				ControllerProxy.getInstance().getFilePropertiesController().onlyShowPropertiesPanel();
			}
		}
	}

	/**
	 * Show splash screen.
	 */
	public void showSplashScreen() {
		if (Kernel.getInstance().state.isShowTitle()) {
			JDialog.setDefaultLookAndFeelDecorated(false);
			splashScreenDialog = new SplashScreenDialog();
			JDialog.setDefaultLookAndFeelDecorated(true);
			splashScreenDialog.setVisible(true);
			splashScreenDialog.toFront();
		}
	}

	/**
	 * Show status bar.
	 * 
	 * @param show
	 *            the show
	 */
	public void showStatusBar(boolean show) {
		Kernel.getInstance().state.setShowStatusBar(show);
		frame.showStatusBar(show);
		repaint();
	}

	public void showToolBar(boolean show) {
		Actions.getAction(ShowToolbarAction.class).putValue(AbstractAction.SELECTED_KEY, show);
		Kernel.getInstance().state.setShowToolBar(show);
		frame.showToolBar(show);
		repaint();
	}

	/**
	 * Start visualization.
	 */
	public void startVisualization() {
		logger.debug(LogCategories.START, "Starting visualization");

		if (SystemProperties.isJava6Update10OrLater) {
			FadingPopupFactory.install();
		}

		frame.create();

		// Create drag and drop listener
		DragAndDropHandler.getInstance().registerListeners();

		showProgressBar(false, null);
		hideDeviceInfoOnStatusBar();

		//SwingUtilities.updateComponentTreeUI(frame.getFrame());
		logger.debug(LogCategories.START, "Start visualization done");
	}

	/**
	 * Toggle window visibility.
	 */
	public void toggleWindowVisibility() {
		frame.setVisible(!frame.isVisible());
		frame.getFrame().toFront();
		frame.getFrame().setState(java.awt.Frame.NORMAL);
	}

	/**
	 * Update play state.
	 * 
	 * @param state
	 *            the state
	 */
	public void updatePlayState(PlayState state) {
		getPlayListTable().setPlayState(state);
	}

	/**
	 * Update status bar.
	 * 
	 * @param song
	 *            the song
	 */
	public void updateStatusBar(AudioObject song) {
		if (GuiUtils.getComponentOrientation().isLeftToRight()) {
			String text = StringUtils.getString("<html>", LanguageTool.getString("PLAYING"), ": ");
			if (song instanceof Radio || song instanceof PodcastFeedEntry || ((AudioFile) song).getTag() == null) {
				text = StringUtils.getString(text, "<b>", song.getTitleOrFileName(), "</b></html>");
			} else {
				if (((AudioFile) song).getTag().getTitle() == null || ((AudioFile) song).getTag().getTitle().equals("")) {
					text = StringUtils.getString(text, "<b>", ((AudioFile) song).getName(), "</b> - ");
				} else {
					text = StringUtils.getString(text, "<b>", ((AudioFile) song).getTag().getTitle(), "</b> - ");
				}
				if (((AudioFile) song).getTag().getArtist() == null || ((AudioFile) song).getTag().getArtist().equals("")) {
					text = StringUtils.getString(text, "<b>", LanguageTool.getString("UNKNOWN_ARTIST"), "</b> ");
				} else {
					text = StringUtils.getString(text, "<b>", ((AudioFile) song).getTag().getArtist(), "</b> ");
				}
				text = StringUtils.getString(text, "(", TimeUtils.seconds2String(song.getDuration()), ")</html>");
			}
			setLeftStatusBarText(text, text);
		} else {
			String text = "";
			if (song instanceof Radio || song instanceof PodcastFeedEntry || ((AudioFile) song).getTag() == null) {
				text = StringUtils.getString("<html><b>", song.getTitleOrFileName(), "</b></html>");
			} else {
				text = StringUtils.getString("<html>(", TimeUtils.seconds2String(song.getDuration()), ") ");
				if (((AudioFile) song).getTag().getArtist() == null || ((AudioFile) song).getTag().getArtist().equals("")) {
					text = StringUtils.getString(text, "<b>", LanguageTool.getString("UNKNOWN_ARTIST"), "</b> ");
				} else {
					text = StringUtils.getString(text, "<b>", ((AudioFile) song).getTag().getArtist(), "</b> ");
				}
				if (((AudioFile) song).getTag().getTitle() == null || ((AudioFile) song).getTag().getTitle().equals("")) {
					text = StringUtils.getString(text, " - <b>", ((AudioFile) song).getName(), "</b>");
				} else {
					text = StringUtils.getString(text, " - <b>", ((AudioFile) song).getTag().getTitle(), "</b>");
				}
			}
			text = StringUtils.getString(text, " :", LanguageTool.getString("PLAYING"), "</html>");
			setLeftStatusBarText(text, text);
		}
	}

	/**
	 * Update status bar.
	 * 
	 * @param text
	 *            the text
	 */
	public void updateStatusBar(String text) {
		setLeftStatusBarText(text, text);
	}

	/**
	 * Update title bar.
	 * 
	 * @param song
	 *            the song
	 */
	public void updateTitleBar(AudioObject song) {
		if (song != null) {
			if (song instanceof Radio) {
				updateTitleBar(StringUtils.getString(((Radio) song).getName(), " (", ((Radio) song).getUrl(), ")"));
				return;
			}

			if (song instanceof PodcastFeedEntry) {
				updateTitleBar(StringUtils.getString(((PodcastFeedEntry) song).getName(), " (", ((PodcastFeedEntry) song).getUrl(), ")"));
				return;
			}

			StringBuilder strBuilder = new StringBuilder();
			if (((AudioFile) song).getTag() == null) {
				strBuilder.append(((AudioFile) song).getName());
			} else {
				strBuilder.append(song.getTitleOrFileName());
				strBuilder.append(" - ");
				strBuilder.append(song.getArtist());
				strBuilder.append(" - ");
				strBuilder.append(song.getAlbum());
			}
			strBuilder.append(" (");
			strBuilder.append(TimeUtils.seconds2String(song.getDuration()));
			strBuilder.append(")");
			updateTitleBar(strBuilder.toString());
		} else {
			updateTitleBar("");
		}
	}

	/**
	 * Update title bar.
	 * 
	 * @param text
	 *            the text
	 */
	public void updateTitleBar(String text) {
		VisualHandler.getInstance().setTitleBar(text);
	}
}
