/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.contextInformation.lyrics;

import net.sourceforge.atunes.kernel.modules.contextInformation.Lyrics;
import net.sourceforge.atunes.kernel.utils.XMLUtils;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

/**
 * The Class LyricWikiEngine.
 */
public class LyricWikiEngine extends LyricsEngine {

	/** Logger. */
	private static Logger logger = new Logger();

	/**
	 * Artists and songs with spaces are replaced with an underscore "_" and the
	 * first character of every word is converted to uppercase.
	 * 
	 * @param str
	 *            the str
	 * 
	 * @return the string encoded
	 */
	private static String getStringForWiki(String str) {
		StringBuilder b = new StringBuilder();
		for (String s : str.toLowerCase().split(" ")) {
			b.append(StringUtils.convertFirstCharacterToUppercase(s));
			b.append(" ");
		}
		return b.toString().trim().replaceAll(" ", "_");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.sourceforge.atunes.kernel.modules.lyrics.LyricsEngine#getLyricsFor
	 * (java.lang.String, java.lang.String)
	 */
	@Override
	public Lyrics getLyricsFor(String artist, String title) {
		try {

			String url = "http://lyricwiki.org/api.php?func=getSong&artist=" + encodeString(getStringForWiki(artist)) + "&song=" + encodeString(getStringForWiki(title))
					+ "&fmt=xml";
			String xml = readURL(getConnection(url), "UTF-8");
			String lyrics = XMLUtils.getChildElementContent(XMLUtils.getXMLDocument(xml).getDocumentElement(), "lyrics");

			return lyrics == null || !lyrics.trim().equalsIgnoreCase("Not found") ? new Lyrics(lyrics, getUrl(artist, title)) : null;
		} catch (Exception e) {
			logger.error(LogCategories.SERVICE, e);
			return null;
		}
	}

	private String getUrl(String artist, String title) {
		StringBuilder builder = new StringBuilder();
		builder.append("http://lyricwiki.org/");
		builder.append(encodeString(getStringForWiki(artist)));
		builder.append(":");
		builder.append(encodeString(getStringForWiki(title)));
		String url = builder.toString();
		return url;
	}

	@Override
	public String getLyricsProviderName() {
		return "LyricWiki";
	}

	@Override
	public String getUrlForAddingNewLyrics(String artist, String title) {
		return getUrl(artist, title);
	}
}