/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.contextInformation.lyrics;

import java.io.IOException;
import java.net.UnknownHostException;

import net.sourceforge.atunes.kernel.modules.contextInformation.Lyrics;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

import org.htmlparser.Node;
import org.htmlparser.NodeFilter;
import org.htmlparser.Parser;
import org.htmlparser.util.NodeList;
import org.htmlparser.util.ParserException;

public class LyricsModeEngine extends LyricsEngine {

	private static final String BASE_URL = "http://www.lyricsmode.com/lyrics/";
	private static final String CHARSET = "UTF-8";

	private Logger logger = new Logger();

	@Override
	public Lyrics getLyricsFor(String artist, String title) {
		StringBuilder sb = new StringBuilder();
		sb.append(BASE_URL);
		sb.append(getString(artist, title));
		sb.append(".html");
		try {
			String html = readURL(getConnection(sb.toString()), CHARSET);
			Parser parser = Parser.createParser(html, CHARSET);

			// Get <div id="songlyrics"> element
			NodeList nodeList = parser.extractAllNodesThatMatch(new NodeFilter() {
				private static final long serialVersionUID = 4555419135319490609L;

				@Override
				public boolean accept(Node node) {
					return node.getText().contains("div id=\"songlyrics\"");
				}
			});

			String lyrics = nodeList.asString().trim();
			return !lyrics.isEmpty() ? new Lyrics(lyrics, sb.toString()) : null;
		} catch (UnknownHostException e) {
			logger.error(LogCategories.SERVICE, e);
			return null;
		} catch (IOException e) {
			logger.info(LogCategories.SERVICE, StringUtils.getString("No lyrics found at ", BASE_URL, " for ", artist, " - ", title));
			return null;
		} catch (ParserException e) {
			logger.error(LogCategories.SERVICE, e);
			return null;
		}

	}

	private String convertString(String string) {
		return string.replace("â", "a").replace("á", "a").replace("à", "a").replace("î", "i").replace("í", "i").replace("ì", "i").replace("ê", "e").replace("é", "e").replace("è",
				"e").replace("ô", "o").replace("ó", "o").replace("ò", "o").replace("û", "u").replace("ú", "u").replace("ù", "u").replace("'", "").replace("´", "").replace("`", "")
				.replace("-", " ").trim().replaceAll(" +", " ").replace(" ", "_").replaceAll("\\W", "");
	}

	private String getString(String artist, String title) {
		StringBuilder sb = new StringBuilder();
		String a = artist.trim().toLowerCase();
		String t = title.trim().toLowerCase();
		a = encodeString(convertString(a));
		t = encodeString(convertString(t));
		String firstCharacterFromArtist = a.substring(0, 1);
		sb.append(firstCharacterFromArtist);
		sb.append("/");
		sb.append(a);
		sb.append("/");
		sb.append(t);
		return sb.toString();
	}

	@Override
	public String getLyricsProviderName() {
		return "LyricsMode";
	}

	@Override
	public String getUrlForAddingNewLyrics(String artist, String title) {
		return "http://www.lyricsmode.com/lyrics_submit.php";
	}

}
