/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.playlist;

import java.util.List;

import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.handlers.MultiplePlaylistHandler;
import net.sourceforge.atunes.kernel.handlers.PlayListHandler;
import net.sourceforge.atunes.kernel.handlers.PlayerHandler;
import net.sourceforge.atunes.kernel.handlers.VisualHandler;
import net.sourceforge.atunes.model.AudioObject;

/**
 * The Class PlayListCommonOps.
 */
public class PlayListCommonOps {

	/**
	 * Adds the to play list and play.
	 * 
	 * @param audioObjects
	 *            the audio objects
	 */
	public static void addToPlayListAndPlay(List<? extends AudioObject> audioObjects) {
		if (audioObjects == null || audioObjects.isEmpty()) {
			return;
		}

		int playListCurrentSize = getPlayList().size();
		PlayListHandler.getInstance().addToPlayList(audioObjects);
		PlayerHandler.getInstance().setPlayListPositionToPlay(playListCurrentSize);
		PlayerHandler.getInstance().playCurrentAudioObject(false);
	}

	/**
	 * Adds the to play list.
	 * 
	 * @param audioObjects
	 *            the audio objects
	 */
	public static void addToPlayList(List<? extends AudioObject> audioObjects) {
		PlayListHandler.getInstance().addToPlayList(audioObjects);
	}

	/**
	 * Gets the current audio object.
	 * 
	 * @return the current audio object
	 */
	public static AudioObject getCurrentAudioObject() {
		return MultiplePlaylistHandler.getInstance().getCurrentPlayList().getCurrentAudioObject();
	}

	/**
	 * Gets the play list.
	 * 
	 * @return the play list
	 */
	public static PlayList getPlayList() {
		return MultiplePlaylistHandler.getInstance().getCurrentPlayList();
	}

	/**
	 * Gets the selected songs.
	 * 
	 * @return the selected songs
	 */
	public static List<AudioObject> getSelectedSongs() {
		return ControllerProxy.getInstance().getPlayListController().getSelectedAudioObjects();
	}

	/**
	 * Refresh play list.
	 */
	public static void refreshPlayList() {
		VisualHandler.getInstance().getPlayListTableModel().refresh();
	}

}
