/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.repository;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.repository.model.Album;
import net.sourceforge.atunes.kernel.modules.repository.model.Artist;
import net.sourceforge.atunes.utils.RankList;

/**
 * The Class RepositoryStats.
 */
public class RepositoryStats implements Serializable {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = -3603928907730394504L;

	/** The total plays. */
	private int totalPlays;

	/** The different songs played. */
	private int differentSongsPlayed;

	/** The songs ranking. */
	private RankList<AudioFile> songsRanking;

	/** The albums ranking. */
	private RankList<Album> albumsRanking;

	/** The artists ranking. */
	private RankList<Artist> artistsRanking;

	/** The songs stats. */
	private Map<String, SongStats> songsStats;

	/**
	 * Instantiates a new repository stats.
	 */
	protected RepositoryStats() {
		songsRanking = new RankList<AudioFile>();
		albumsRanking = new RankList<Album>();
		artistsRanking = new RankList<Artist>();
		songsStats = new HashMap<String, SongStats>();
	}

	/**
	 * Gets the albums ranking.
	 * 
	 * @return the albums ranking
	 */
	public RankList<Album> getAlbumsRanking() {
		return albumsRanking;
	}

	/**
	 * Gets the artists ranking.
	 * 
	 * @return the artists ranking
	 */
	public RankList<Artist> getArtistsRanking() {
		return artistsRanking;
	}

	/**
	 * Gets the different songs played.
	 * 
	 * @return the different songs played
	 */
	public int getDifferentSongsPlayed() {
		return differentSongsPlayed;
	}

	/**
	 * Gets the songs ranking.
	 * 
	 * @return the songs ranking
	 */
	public RankList<AudioFile> getSongsRanking() {
		return songsRanking;
	}

	/**
	 * Gets the songs stats.
	 * 
	 * @return the songs stats
	 */
	public Map<String, SongStats> getSongsStats() {
		return songsStats;
	}

	/**
	 * Gets the stats for file.
	 * 
	 * @param song
	 *            the song
	 * 
	 * @return the stats for file
	 */
	public SongStats getStatsForFile(AudioFile song) {
		if (song != null) {
			return songsStats.get(song.getUrl());
		}
		return null;
	}

	/**
	 * Gets the total plays.
	 * 
	 * @return the total plays
	 */
	public int getTotalPlays() {
		return totalPlays;
	}

	/**
	 * Sets the albums ranking.
	 * 
	 * @param albumsRanking
	 *            the new albums ranking
	 */
	public void setAlbumsRanking(RankList<Album> albumsRanking) {
		this.albumsRanking = albumsRanking;
	}

	/**
	 * Sets the artists ranking.
	 * 
	 * @param artistsRanking
	 *            the new artists ranking
	 */
	public void setArtistsRanking(RankList<Artist> artistsRanking) {
		this.artistsRanking = artistsRanking;
	}

	/**
	 * Sets the different songs played.
	 * 
	 * @param differentSongsPlayed
	 *            the new different songs played
	 */
	public void setDifferentSongsPlayed(int differentSongsPlayed) {
		this.differentSongsPlayed = differentSongsPlayed;
	}

	/**
	 * Sets the songs ranking.
	 * 
	 * @param songsRanking
	 *            the new songs ranking
	 */
	public void setSongsRanking(RankList<AudioFile> songsRanking) {
		this.songsRanking = songsRanking;
	}

	/**
	 * Sets the songs stats.
	 * 
	 * @param songsStats
	 *            the songs stats
	 */
	public void setSongsStats(Map<String, SongStats> songsStats) {
		this.songsStats = songsStats;
	}

	/**
	 * Sets the total plays.
	 * 
	 * @param totalPlays
	 *            the new total plays
	 */
	public void setTotalPlays(int totalPlays) {
		this.totalPlays = totalPlays;
	}

}
